@@ -1,21 +1,37 @@
 /*
- * flags.c - all the flag matching/conversion functions in one neat
- * package :)
+ * flags.c -- handles:
+ *   all the flag matching/conversion functions in one neat package :)
+ *
+ * $Id: flags.c,v 1.19 2002/01/02 03:46:35 guppy Exp $
  */
 /*
- * see main.c for licencing info 
+ * Copyright (C) 1997 Robey Pointer
+ * Copyright (C) 1999, 2000, 2001, 2002 Eggheads Development Team
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
 #include "main.h"
 
-extern int use_console_r;
-extern int debug_output;
-extern int require_p;
-extern struct dcc_t *dcc;
-extern int noshare;
-extern int allow_dk_cmds;
 
-int use_console_r = 0;		/* allow users to set their console +r */
+extern int		 use_console_r, debug_output, require_p, noshare,
+			 allow_dk_cmds;
+extern struct dcc_t	*dcc;
+
+int	use_console_r = 0;	/* Allow users to set their console +r	*/
+
 
 int logmodes(char *s)
 {
@@ -117,7 +133,7 @@ int logmodes(char *s)
 
 char *masktype(int x)
 {
-  static char s[24];		/* change this if you change the levels */
+  static char s[24];		/* Change this if you change the levels */
   char *p = s;
 
   if (x & LOG_MSGS)
@@ -174,7 +190,7 @@ char *masktype(int x)
 
 char *maskname(int x)
 {
-  static char s[207];		/* change this if you change the levels */
+  static char s[207];		/* Change this if you change the levels */
   int i = 0;
 
   s[0] = 0;
@@ -231,7 +247,8 @@ char *maskname(int x)
   return s;
 }
 
-/* some flags are mutually exclusive -- this roots them out */
+/* Some flags are mutually exclusive -- this roots them out
+ */
 int sanity_check(int atr)
 {
   if ((atr & USER_BOT) &&
@@ -245,22 +262,23 @@ int sanity_check(int atr)
     atr &= ~(USER_VOICE | USER_QUIET);
   if ((atr & USER_GVOICE) && (atr & USER_QUIET))
     atr &= ~(USER_GVOICE | USER_QUIET);
-  /* can't be owner without also being master */
+  /* Can't be owner without also being master */
   if (atr & USER_OWNER)
     atr |= USER_MASTER;
-  /* master implies botmaster, op, friend and janitor */
+  /* Master implies botmaster, op and janitor */
   if (atr & USER_MASTER)
-    atr |= USER_BOTMAST | USER_OP | USER_FRIEND | USER_JANITOR;
-  /* can't be botnet master without party-line access */
+    atr |= USER_BOTMAST | USER_OP | USER_JANITOR;
+  /* Can't be botnet master without party-line access */
   if (atr & USER_BOTMAST)
     atr |= USER_PARTY;
-  /* janitors can use the file area */
+  /* Janitors can use the file area */
   if (atr & USER_JANITOR)
     atr |= USER_XFER;
   return atr;
 }
 
-/* sanity check on channel attributes */
+/* Sanity check on channel attributes
+ */
 int chan_sanity_check(int chatr, int atr)
 {
   if ((chatr & USER_OP) && (chatr & USER_DEOP))
@@ -271,28 +289,29 @@ int chan_sanity_check(int chatr, int atr)
     chatr &= ~(USER_VOICE | USER_QUIET);
   if ((chatr & USER_GVOICE) && (chatr & USER_QUIET))
     chatr &= ~(USER_GVOICE | USER_QUIET);
-  /* can't be channel owner without also being channel master */
+  /* Can't be channel owner without also being channel master */
   if (chatr & USER_OWNER)
     chatr |= USER_MASTER;
-  /* master implies friend & op */
+  /* Master implies op */
   if (chatr & USER_MASTER)
-    chatr |= USER_OP | USER_FRIEND;
-  /* can't be +s on chan unless you're a bot */
+    chatr |= USER_OP ;
+  /* Can't be +s on chan unless you're a bot */
   if (!(atr & USER_BOT))
     chatr &= ~BOT_SHARE;
   return chatr;
 }
 
-/* get icon symbol for a user (depending on access level)
+/* Get icon symbol for a user (depending on access level)
+ *
  * (*)owner on any channel
  * (+)master on any channel
  * (%) botnet master
  * (@) op on any channel
- * (-) other */
+ * (-) other
+ */
 char geticon(int idx)
 {
-  struct flag_record fr =
-  {FR_GLOBAL | FR_CHAN | FR_ANYWH, 0, 0, 0, 0, 0};
+  struct flag_record fr = {FR_GLOBAL | FR_CHAN | FR_ANYWH, 0, 0, 0, 0, 0};
 
   if (!dcc[idx].user)
     return '-';
@@ -308,12 +327,12 @@ char geticon(int idx)
   return '-';
 }
 
-void break_down_flags(char *string, struct flag_record *plus,
+void break_down_flags(const char *string, struct flag_record *plus,
 		      struct flag_record *minus)
 {
-  struct flag_record *which = plus;
-  int mode = 0;			/* 0 = glob, 1 = chan, 2 = bot */
-  int flags = plus->match;
+  struct flag_record	*which = plus;
+  int			 mode = 0;	/* 0 = glob, 1 = chan, 2 = bot */
+  int			 flags = plus->match;
 
   if (!(flags & FR_GLOBAL)) {
     if (flags & FR_BOT)
@@ -321,14 +340,14 @@ void break_down_flags(char *string, struct flag_record *plus,
     else if (flags & FR_CHAN)
       mode = 1;
     else
-      return;			/* we dont actually want any..huh? */
+      return;			/* We dont actually want any..huh? */
   }
-  bzero(plus, sizeof(struct flag_record));
+  egg_bzero(plus, sizeof(struct flag_record));
 
   if (minus)
-    bzero(minus, sizeof(struct flag_record));
+    egg_bzero(minus, sizeof(struct flag_record));
 
-  plus->match = FR_OR;		/* befault binding type OR */
+  plus->match = FR_OR;		/* Default binding type OR */
   while (*string) {
     switch (*string) {
     case '+':
@@ -339,7 +358,7 @@ void break_down_flags(char *string, struct flag_record *plus,
       break;
     case '|':
     case '&':
-      if (mode == 0) {
+      if (!mode) {
 	if (*string == '|')
 	  plus->match = FR_OR;
 	else
@@ -376,7 +395,7 @@ void break_down_flags(char *string, struct flag_record *plus,
 	}
       } else if ((*string >= '0') && (*string <= '9')) {
 	switch (mode) {
-	  /* map 0->9 to A->K for glob/chan so they are not lost */
+	  /* Map 0->9 to A->K for glob/chan so they are not lost */
 	case 0:
 	  which->udef_global |= 1 << (*string - '0');
 	  break;
@@ -510,7 +529,7 @@ int flagrec_ok(struct flag_record *req,
   } else if (req->match & FR_OR) {
     int hav = have->global;
 
-    /* exception 1 - global +d/+k cant use -|-, unless they are +p */
+    /* Exception 1 - global +d/+k cant use -|-, unless they are +p */
     if (!req->chan && !req->global && !req->udef_global &&
 	!req->udef_chan) {
       if (!allow_dk_cmds) {
@@ -523,8 +542,9 @@ int flagrec_ok(struct flag_record *req,
       }
       return 1;
     }
-    /* the +n/+m checks arent needed anymore since +n/+m
-     * automatically add lower flags */
+    /* The +n/+m checks arent needed anymore since +n/+m
+     * automatically add lower flags
+     */
     if (!require_p && ((hav & USER_OP) || (have->chan & USER_OWNER)))
       hav |= USER_PARTY;
     if (hav & req->global)
@@ -584,9 +604,9 @@ int flagrec_eq(struct flag_record *req, struct flag_record *have)
 }
 
 void set_user_flagrec(struct userrec *u, struct flag_record *fr,
-		      char *chname)
+		      const char *chname)
 {
-  struct chanuserrec *cr;
+  struct chanuserrec *cr = NULL;
   int oldflags = fr->match;
   char buffer[100];
   struct chanset_t *ch;
@@ -605,20 +625,19 @@ void set_user_flagrec(struct userrec *u, struct flag_record *fr,
   }
   if ((oldflags & FR_BOT) && (u->flags & USER_BOT))
     set_user(&USERENTRY_BOTFL, u, (void *) fr->bot);
-  /* dont share bot attrs */
+  /* Don't share bot attrs */
   if ((oldflags & FR_CHAN) && chname) {
     for (cr = u->chanrec; cr; cr = cr->next)
       if (!rfc_casecmp(chname, cr->channel))
 	break;
-    ch = findchan(chname);
+    ch = findchan_by_dname(chname);
     if (!cr && ch) {
       cr = user_malloc(sizeof(struct chanuserrec));
-      bzero(cr, sizeof(struct chanuserrec));
+      egg_bzero(cr, sizeof(struct chanuserrec));
 
       cr->next = u->chanrec;
       u->chanrec = cr;
-      strncpy(cr->channel, chname, 80);
-      cr->channel[80] = 0;
+      strncpyz(cr->channel, chname, sizeof cr->channel);
     }
     if (cr && ch) {
       cr->flags = fr->chan;
@@ -633,23 +652,22 @@ void set_user_flagrec(struct userrec *u, struct flag_record *fr,
   fr->match = oldflags;
 }
 
+/* Always pass the dname (display name) to this function for chname <cybah>
+ */
 void get_user_flagrec(struct userrec *u, struct flag_record *fr,
-		      char *chname)
+		      const char *chname)
 {
-  struct chanuserrec *cr;
+  struct chanuserrec *cr = NULL;
 
   if (!u) {
     fr->global = fr->udef_global = fr->chan = fr->udef_chan = fr->bot = 0;
-
     return;
   }
   if (fr->match & FR_GLOBAL) {
     fr->global = u->flags;
-
     fr->udef_global = u->flags_udef;
   } else {
     fr->global = 0;
-
     fr->udef_global = 0;
   }
   if (fr->match & FR_BOT) {
@@ -661,7 +679,7 @@ void get_user_flagrec(struct userrec *u, struct flag_record *fr,
       fr->chan = u->flags;
       fr->udef_chan = u->flags_udef;
       for (cr = u->chanrec; cr; cr = cr->next)
-	if (findchan(cr->channel)) {
+	if (findchan_by_dname(cr->channel)) {
 	  fr->chan |= cr->flags;
 	  fr->udef_chan |= cr->flags_udef;
 	}
@@ -670,7 +688,7 @@ void get_user_flagrec(struct userrec *u, struct flag_record *fr,
 	for (cr = u->chanrec; cr; cr = cr->next)
 	  if (!rfc_casecmp(chname, cr->channel))
 	    break;
-      if (chname && cr) {
+      if (cr) {
 	fr->chan = cr->flags;
 	fr->udef_chan = cr->flags_udef;
       } else {
@@ -683,61 +701,39 @@ void get_user_flagrec(struct userrec *u, struct flag_record *fr,
 
 static int botfl_unpack(struct userrec *u, struct user_entry *e)
 {
-  struct flag_record fr =
-  {FR_BOT, 0, 0, 0, 0, 0};
+  struct flag_record fr = {FR_BOT, 0, 0, 0, 0, 0};
 
-  context;
-  if (e->name) {
-    char *p;
-
-    p = e->u.list->extra;
-    e->u.list->extra = NULL;
-    list_type_kill(e->u.list);
-    break_down_flags(p, &fr, NULL);
-    e->u.ulong = fr.bot;
-    nfree(p);
-  }
+  break_down_flags(e->u.list->extra, &fr, NULL);
+  list_type_kill(e->u.list);
+  e->u.ulong = fr.bot;
   return 1;
 }
 
 static int botfl_pack(struct userrec *u, struct user_entry *e)
 {
   char x[100];
-  struct flag_record fr =
-  {FR_BOT, 0, 0, 0, 0, 0};
-  int l;
+  struct flag_record fr = {FR_BOT, 0, 0, 0, 0, 0};
 
-  if (!e->name) {
-    char *p;
-
-    e->name = user_malloc(strlen(e->type->name) + 1);
-    strcpy(e->name, e->type->name);
-    fr.bot = e->u.ulong;
-    l = build_flags(x, &fr, NULL);
-    p = user_malloc(l + 1);
-    e->u.list = user_malloc(sizeof(struct list_type));
-
-    e->u.list->next = NULL;
-    e->u.list->extra = p;
-    strcpy(p, x);
-  }
+  fr.bot = e->u.ulong;
+  e->u.list = user_malloc(sizeof(struct list_type));
+  e->u.list->next = NULL;
+  e->u.list->extra = user_malloc (build_flags (x, &fr, NULL) + 1);
+  strcpy(e->u.list->extra, x);
   return 1;
 }
 
 static int botfl_kill(struct user_entry *e)
 {
-  context;
   nfree(e);
   return 1;
 }
 
-static int botfl_write_userfile(FILE * f, struct userrec *u, struct user_entry *e)
+static int botfl_write_userfile(FILE *f, struct userrec *u,
+				struct user_entry *e)
 {
   char x[100];
-  struct flag_record fr =
-  {FR_BOT, 0, 0, 0, 0, 0};
+  struct flag_record fr = {FR_BOT, 0, 0, 0, 0, 0};
 
-  context;
   fr.bot = e->u.ulong;
   build_flags(x, &fr, NULL);
   if (fprintf(f, "--%s %s\n", e->type->name, x) == EOF)
@@ -749,9 +745,10 @@ static int botfl_set(struct userrec *u, struct user_entry *e, void *buf)
 {
   register long atr = ((long) buf & BOT_VALID);
 
-  context;
   if (!(u->flags & USER_BOT))
-    return 1;			/* don't even bother trying to set the flags for a non-bot */
+    return 1;			/* Don't even bother trying to set the
+				   flags for a non-bot */
+
   if ((atr & BOT_HUB) && (atr & BOT_ALT))
     atr &= ~BOT_ALT;
   if (atr & BOT_REJECT) {
@@ -765,16 +762,14 @@ static int botfl_set(struct userrec *u, struct user_entry *e, void *buf)
   if (!(atr & BOT_SHARE))
     atr &= ~BOT_GLOBAL;
   e->u.ulong = atr;
-  context;
   return 1;
 }
 
-static int botfl_tcl_get(Tcl_Interp * interp, struct userrec *u,
+static int botfl_tcl_get(Tcl_Interp *interp, struct userrec *u,
 			 struct user_entry *e, int argc, char **argv)
 {
   char x[100];
-  struct flag_record fr =
-  {FR_BOT, 0, 0, 0, 0, 0};
+  struct flag_record fr = {FR_BOT, 0, 0, 0, 0, 0};
 
   fr.bot = e->u.ulong;
   build_flags(x, &fr, NULL);
@@ -782,34 +777,30 @@ static int botfl_tcl_get(Tcl_Interp * interp, struct userrec *u,
   return TCL_OK;
 }
 
-static int botfl_tcl_set(Tcl_Interp * irp, struct userrec *u,
+static int botfl_tcl_set(Tcl_Interp *irp, struct userrec *u,
 			 struct user_entry *e, int argc, char **argv)
 {
-  struct flag_record fr =
-  {FR_BOT, 0, 0, 0, 0, 0};
+  struct flag_record fr = {FR_BOT, 0, 0, 0, 0, 0};
 
   BADARGS(4, 4, " handle BOTFL flags");
   if (u->flags & USER_BOT) {
-    /* silently ignore for users */
+    /* Silently ignore for users */
     break_down_flags(argv[3], &fr, NULL);
-    def_set(u, e, (void *) fr.bot);
+    botfl_set(u, e, (void *) fr.bot);
   }
   return TCL_OK;
 }
 
 static int botfl_expmem(struct user_entry *e)
 {
-  context;
   return 0;
 }
 
 static void botfl_display(int idx, struct user_entry *e)
 {
-  struct flag_record fr =
-  {FR_BOT, 0, 0, 0, 0, 0};
+  struct flag_record fr = {FR_BOT, 0, 0, 0, 0, 0};
   char x[100];
 
-  context;
   fr.bot = e->u.ulong;
   build_flags(x, &fr, NULL);
   dprintf(idx, "  BOT FLAGS: %s\n", x);
