@@ -1,31 +1,46 @@
-/* 
- * assoc.c - the assoc module, moved here mainly from botnet.c
- *           for module work
+/*
+ * assoc.c -- part of assoc.mod
+ *   the assoc code, moved here mainly from botnet.c for module work
+ *
+ * $Id: assoc.c,v 1.19 2002/01/02 08:06:14 tothwolf Exp $
  */
-/* 
- * This file is part of the eggdrop source code
- * copyright (c) 1997 Robey Pointer
- * and is distributed according to the GNU general public license.
- * For full details, read the top of 'main.c' or the file called
- * COPYING that was distributed with this code.
+/*
+ * Copyright (C) 1997 Robey Pointer
+ * Copyright (C) 1999, 2000, 2001, 2002 Eggheads Development Team
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#define MAKING_ASSOC
 #define MODULE_NAME "assoc"
-#include "../module.h"
-#include "../../tandem.h"
+#define MAKING_ASSOC
+#include "src/mod/module.h"
+#include "src/tandem.h"
 #include <stdlib.h>
+#include "assoc.h"
+
 #undef global
 static Function *global = NULL;
 
-/* keep track of channel associations */
-typedef struct travis {
+/* Keep track of channel associations */
+typedef struct assoc_t_ {
   char name[21];
   unsigned int channel;
-  struct travis *next;
+  struct assoc_t_ *next;
 } assoc_t;
 
-/* channel name-number associations */
+/* Channel name-number associations */
 static assoc_t *assoc;
 
 static void botnet_send_assoc(int idx, int chan, char *nick,
@@ -44,14 +59,11 @@ static void botnet_send_assoc(int idx, int chan, char *nick,
 
 static int assoc_expmem()
 {
-  assoc_t *a = assoc;
+  assoc_t *a;
   int size = 0;
 
-  context;
-  while (a != NULL) {
+  for (a = assoc; a; a = a->next)
     size += sizeof(assoc_t);
-    a = a->next;
-  }
   return size;
 }
 
@@ -59,20 +71,16 @@ static void link_assoc(char *bot, char *via)
 {
   char x[1024];
 
-  if (!strcasecmp(via, botnetnick)) {
+  if (!egg_strcasecmp(via, botnetnick)) {
     int idx = nextbot(bot);
-    assoc_t *a = assoc;
+    assoc_t *a;
 
     if (!(bot_flags(dcc[idx].user) & BOT_ISOLATE)) {
-      context;
-      while (a != NULL) {
-	if (a->name[0]) {
-	  simple_sprintf(x, "assoc %D %s %s", a->channel, botnetnick,
-			 a->name);
+      for (a = assoc; a && a->name[0]; a = a->next) {
+          simple_sprintf(x, "assoc %D %s %s", (int) a->channel, botnetnick,
+           		a->name);
 	  botnet_send_zapf(idx, botnetnick, dcc[idx].nick, x);
 	}
-	a = a->next;
-      }
     }
   }
 }
@@ -81,8 +89,7 @@ static void kill_assoc(int chan)
 {
   assoc_t *a = assoc, *last = NULL;
 
-  context;
-  while (a != NULL) {
+  while (a) {
     if (a->channel == chan) {
       if (last != NULL)
 	last->next = a->next;
@@ -99,59 +106,49 @@ static void kill_assoc(int chan)
 
 static void kill_all_assoc()
 {
-  assoc_t *a = assoc, *x;
+  assoc_t *a, *x;
 
-  context;
-  while (a != NULL) {
-    x = a;
-    a = a->next;
-    nfree(x);
+  for (a = assoc; a; a = x) {
+    x = a->next;
+    nfree(a);
   }
   assoc = NULL;
 }
 
 static void add_assoc(char *name, int chan)
 {
-  assoc_t *a = assoc, *b, *old = NULL;
+  assoc_t *a, *b, *old = NULL;
 
-  context;
-  while (a != NULL) {
-    if ((name[0] != 0) && (!strcasecmp(a->name, name))) {
+  for (a = assoc; a; a = a->next) {
+    if (name[0] != 0 && !egg_strcasecmp(a->name, name)) {
       kill_assoc(a->channel);
       add_assoc(name, chan);
       return;
     }
     if (a->channel == chan) {
-      strncpy(a->name, name, 20);
-      a->name[20] = 0;
+      strncpyz(a->name, name, sizeof a->name);
       return;
     }
-    a = a->next;
   }
-  /* add in numerical order */
-  a = assoc;
-  while (a != NULL) {
+  /* Add in numerical order */
+  for (a = assoc; a; old = a, a = a->next) {
     if (a->channel > chan) {
       b = (assoc_t *) nmalloc(sizeof(assoc_t));
       b->next = a;
       b->channel = chan;
-      strncpy(b->name, name, 20);
-      b->name[20] = 0;
+      strncpyz(b->name, name, sizeof b->name);
       if (old == NULL)
 	assoc = b;
       else
 	old->next = b;
       return;
     }
-    old = a;
-    a = a->next;
   }
-  /* add at the end */
+  /* Add at the end */
   b = (assoc_t *) nmalloc(sizeof(assoc_t));
   b->next = NULL;
   b->channel = chan;
-  strncpy(b->name, name, 20);
-  b->name[20] = 0;
+  strncpyz(b->name, name, sizeof b->name);
   if (old == NULL)
     assoc = b;
   else
@@ -160,27 +157,21 @@ static void add_assoc(char *name, int chan)
 
 static int get_assoc(char *name)
 {
-  assoc_t *a = assoc;
+  assoc_t *a;
 
-  context;
-  while (a != NULL) {
-    if (!strcasecmp(a->name, name))
+  for (a = assoc; a; a = a->next)
+    if (!egg_strcasecmp(a->name, name))
       return a->channel;
-    a = a->next;
-  }
   return -1;
 }
 
 static char *get_assoc_name(int chan)
 {
-  assoc_t *a = assoc;
+  assoc_t *a;
 
-  context;
-  while (a != NULL) {
+  for (a = assoc; a; a = a->next)
     if (a->channel == chan)
       return a->name;
-    a = a->next;
-  }
   return NULL;
 }
 
@@ -188,18 +179,14 @@ static void dump_assoc(int idx)
 {
   assoc_t *a = assoc;
 
-  context;
   if (a == NULL) {
-    dprintf(idx, "No channel names.\n");
+    dprintf(idx, "%s\n", ASSOC_NOCHNAMES);
     return;
   }
-  dprintf(idx, " Chan  Name\n");
-  while (a != NULL) {
-    if (a->name[0])
+  dprintf(idx, " %s  %s\n", ASSOC_CHAN, ASSOC_NAME);
+  for (; a && a->name[0]; a = a->next)
       dprintf(idx, "%c%5d %s\n", (a->channel < 100000) ? ' ' : '*',
 	      a->channel % 100000, a->name);
-    a = a->next;
-  }
   return;
 }
 
@@ -208,59 +195,59 @@ static int cmd_assoc(struct userrec *u, int idx, char *par)
   char *num;
   int chan;
 
-  context;
   if (!par[0]) {
     putlog(LOG_CMDS, "*", "#%s# assoc", dcc[idx].nick);
     dump_assoc(idx);
   } else if (!u || !(u->flags & USER_BOTMAST)) {
-    dprintf(idx, "What? You need '.help'.\n");
+    dprintf(idx, "%s", ASSOC_NOSUCHCMD);
   } else {
     num = newsplit(&par);
     if (num[0] == '*') {
       chan = 100000 + atoi(num + 1);
       if ((chan < 100000) || (chan > 199999)) {
-	dprintf(idx, "Channel # out of range: must be *0-*99999\n");
+	   dprintf(idx, "%s\n", ASSOC_LCHAN_RANGE);
 	return 0;
       }
     } else {
       chan = atoi(num);
       if (chan == 0) {
-	dprintf(idx, "You can't name the main party line; it's just a party line.\n");
+	    dprintf(idx, "%s\n", ASSOC_PARTYLINE);
 	return 0;
       } else if ((chan < 1) || (chan > 99999)) {
-	dprintf(idx, "Channel # out of range: must be 1-99999\n");
+	    dprintf(idx, "%s\n", ASSOC_CHAN_RANGE);
 	return 0;
       }
     }
     if (!par[0]) {
-      /* remove an association */
+      /* Remove an association */
       if (get_assoc_name(chan) == NULL) {
-	dprintf(idx, "Channel %s%d has no name.\n",
-		(chan < 100000) ? "" : "*", chan % 100000);
+	    dprintf(idx, ASSOC_NONAME_CHAN, (chan < 100000) ? "" : "*",
+	            chan % 100000);
 	return 0;
       }
       kill_assoc(chan);
       putlog(LOG_CMDS, "*", "#%s# assoc %d", dcc[idx].nick, chan);
-      dprintf(idx, "Okay, removed name for channel %s%d.\n",
-	      (chan < 100000) ? "" : "*", chan % 100000);
-      chanout_but(-1, chan, "--- %s removed this channel's name.\n", dcc[idx].nick);
+      dprintf(idx, ASSOC_REMNAME_CHAN, (chan < 100000) ? "" : "*",
+              chan % 100000);
+      chanout_but(-1, chan, ASSOC_REMOUT_CHAN, dcc[idx].nick);
       if (chan < 100000)
 	botnet_send_assoc(-1, chan, dcc[idx].nick, "0");
       return 0;
     }
     if (strlen(par) > 20) {
-      dprintf(idx, "Channel's name can't be that long (20 chars max).\n");
+      dprintf(idx, "%s\n", ASSOC_CHNAME_TOOLONG);
       return 0;
     }
     if ((par[0] >= '0') && (par[0] <= '9')) {
-      dprintf(idx, "First character of the channel name can't be a digit.\n");
+      dprintf(idx, "%s\n", ASSOC_CHNAME_FIRSTCHAR);
       return 0;
     }
     add_assoc(par, chan);
     putlog(LOG_CMDS, "*", "#%s# assoc %d %s", dcc[idx].nick, chan, par);
-    dprintf(idx, "Okay, channel %s%d is '%s' now.\n",
-	    (chan < 100000) ? "" : "*", chan % 100000, par);
-    chanout_but(-1, chan, "--- %s named this channel '%s'\n", dcc[idx].nick, par);
+    dprintf(idx, ASSOC_NEWNAME_CHAN, (chan < 100000) ? "" : "*",
+            chan % 100000, par);
+    chanout_but(-1, chan, ASSOC_NEWOUT_CHAN, dcc[idx].nick,
+		par);
     if (chan < 100000)
       botnet_send_assoc(-1, chan, dcc[idx].nick, par);
   }
@@ -270,11 +257,10 @@ static int cmd_assoc(struct userrec *u, int idx, char *par)
 static int tcl_killassoc STDVAR {
   int chan;
 
-  context;
   BADARGS(2, 2, " chan");
-  if (argv[1][0] == '&') {
+  if (argv[1][0] == '&')
     kill_all_assoc();
-  } else {
+  else {
     chan = atoi(argv[1]);
     if ((chan < 1) || (chan > 199999)) {
       Tcl_AppendResult(irp, "invalid channel #", NULL);
@@ -290,7 +276,6 @@ static int tcl_assoc STDVAR {
   int chan;
   char name[21], *p;
 
-  context;
   BADARGS(2, 3, " chan ?name?");
   if ((argc == 2) && ((argv[1][0] < '0') || (argv[1][0] > '9'))) {
     chan = get_assoc(argv[1]);
@@ -328,9 +313,8 @@ static void zapf_assoc(char *botnick, char *code, char *par)
   char *s, *s1, *nick;
   int linking = 0, chan;
 
-  context;
   if ((idx >= 0) && !(bot_flags(dcc[idx].user) & BOT_ISOLATE)) {
-    if (!strcasecmp(dcc[idx].nick, botnick))
+    if (!egg_strcasecmp(dcc[idx].nick, botnick))
       linking = b_status(idx) & STAT_LINKING;
     s = newsplit(&par);
     chan = base64_to_int(s);
@@ -342,35 +326,31 @@ static void zapf_assoc(char *botnick, char *code, char *par)
 				       dcc[idx].user) & BOT_HUB)))) {
 	add_assoc(par, chan);
 	botnet_send_assoc(idx, chan, nick, par);
-	chanout_but(-1, chan, "--- (%s) named this channel '%s'.\n",
-		    nick, par);
+	chanout_but(-1, chan, ASSOC_CHNAME_NAMED, nick, par);
       } else if (par[0] == '0') {
 	kill_assoc(chan);
-	chanout_but(-1, chan, "--- (%s) %s removed this channel's name.\n",
-		    botnick, nick);
+	chanout_but(-1, chan, ASSOC_CHNAME_REM, botnick, nick);
       } else if (get_assoc(par) != chan) {
-	/* new one i didn't know about -- pass it on */
+	/* New one i didn't know about -- pass it on */
 	s1 = get_assoc_name(chan);
 	add_assoc(par, chan);
-	chanout_but(-1, chan, "--- (%s) %s named this channel '%s'.\n",
-		    botnick, nick, par);
+	chanout_but(-1, chan, ASSOC_CHNAME_NAMED2, botnick, nick, par);
       }
     }
   }
 }
 
-/* a report on the module status */
+/* A report on the module status.
+ */
 static void assoc_report(int idx, int details)
 {
-  assoc_t *a = assoc;
+  assoc_t *a;
   int size = 0, count = 0;;
 
-  context;
   if (details) {
-    while (a != NULL) {
+    for (a = assoc; a; a = a->next) {
       count++;
       size += sizeof(assoc_t);
-      a = a->next;
     }
     dprintf(idx, "    %d assocs using %d bytes\n",
 	    count, size);
@@ -379,39 +359,43 @@ static void assoc_report(int idx, int details)
 
 static cmd_t mydcc[] =
 {
-  {"assoc", "", cmd_assoc, NULL},
+  {"assoc",	"",	cmd_assoc,		NULL},
+  {NULL, 	NULL,	NULL,			NULL}
 };
 
 static cmd_t mybot[] =
 {
-  {"assoc", "", (Function) zapf_assoc, NULL},
+  {"assoc",	"",	(Function) zapf_assoc,	NULL},
+  {NULL,        NULL,   NULL,                   NULL}
 };
 
 static cmd_t mylink[] =
 {
-  {"*", "", (Function) link_assoc, "assoc"},
+  {"*",		"",	(Function) link_assoc,	"assoc"},
+  {NULL,	NULL,	NULL,			NULL}
 };
 
 static tcl_cmds mytcl[] =
 {
-  {"assoc", tcl_assoc},
-  {"killassoc", tcl_killassoc},
-  {0, 0}
+  {"assoc",		tcl_assoc},
+  {"killassoc",		tcl_killassoc},
+  {NULL,		NULL}
 };
 
 static char *assoc_close()
 {
-  context;
-  rem_builtins(H_dcc, mydcc, 1);
-  rem_builtins(H_bot, mybot, 1);
-  rem_builtins(H_link, mylink, 1);
-  module_undepend(MODULE_NAME);
+  kill_all_assoc();
+  rem_builtins(H_dcc, mydcc);
+  rem_builtins(H_bot, mybot);
+  rem_builtins(H_link, mylink);
   rem_tcl_commands(mytcl);
   rem_help_reference("assoc.help");
+  del_lang_section("assoc");
+  module_undepend(MODULE_NAME);
   return NULL;
 }
 
-char *assoc_start();
+EXPORT_SCOPE char *assoc_start();
 
 static Function assoc_table[] =
 {
@@ -425,14 +409,16 @@ char *assoc_start(Function * global_funcs)
 {
   global = global_funcs;
 
-  context;
   module_register(MODULE_NAME, assoc_table, 2, 0);
-  if (!module_depend(MODULE_NAME, "eggdrop", 103, 0))
-    return "This module requires eggdrop1.3.0 or later";
+  if (!module_depend(MODULE_NAME, "eggdrop", 106, 0)) {
+    module_undepend(MODULE_NAME);
+    return "This module requires eggdrop1.6.0 or later";
+  }
   assoc = NULL;
-  add_builtins(H_dcc, mydcc, 1);
-  add_builtins(H_bot, mybot, 1);
-  add_builtins(H_link, mylink, 1);
+  add_builtins(H_dcc, mydcc);
+  add_builtins(H_bot, mybot);
+  add_builtins(H_link, mylink);
+  add_lang_section("assoc");
   add_tcl_commands(mytcl);
   add_help_reference("assoc.help");
   return NULL;
