@@ -1,22 +1,41 @@
-/* 
- * blowfish.c - part of blowfish.mod
- * handles: encryption and decryption of passwords
+/*
+ * blowfish.c -- part of blowfish.mod
+ *   encryption and decryption of passwords
+ *
+ * $Id: blowfish.c,v 1.21 2002/01/02 03:46:37 guppy Exp $
  */
-/* 
- * The first half of this is very lightly edited from public domain
- * sourcecode.  For simplicity, this entire module will remain public
- * domain.
+/*
+ * Copyright (C) 1997 Robey Pointer
+ * Copyright (C) 1999, 2000, 2001, 2002 Eggheads Development Team
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ */
+/*
+ * This code was originally in the public domain.
  */
 
-#define MAKING_BLOWFISH
 #define MODULE_NAME "encryption"
-#include "../module.h"
+#define MAKING_ENCRYPTION
+
+#include "src/mod/module.h"
 #include "blowfish.h"
 #include "bf_tab.h"		/* P-box P-array, S-box */
 #undef global
 static Function *global = NULL;
 
-/* each box takes up 4k so be very careful here */
+/* Each box takes up 4k so be very careful here */
 #define BOXES 3
 
 /* #define S(x,i) (bf_S[i][x.w.byte##i]) */
@@ -27,35 +46,34 @@ static Function *global = NULL;
 #define bf_F(x) (((S0(x) + S1(x)) ^ S2(x)) + S3(x))
 #define ROUND(a,b,n) (a.word ^= bf_F(b) ^ bf_P[n])
 
-/* keep a set of rotating P & S boxes */
+/* Keep a set of rotating P & S boxes */
 static struct box_t {
-  UWORD_32bits *P;
-  UWORD_32bits **S;
+  u_32bit_t *P;
+  u_32bit_t **S;
   char key[81];
   char keybytes;
   time_t lastuse;
 } box[BOXES];
 
-/* static UWORD_32bits bf_P[bf_N+2]; */
-/* static UWORD_32bits bf_S[4][256]; */
-static UWORD_32bits *bf_P;
-static UWORD_32bits **bf_S;
+/* static u_32bit_t bf_P[bf_N+2]; */
+/* static u_32bit_t bf_S[4][256]; */
+static u_32bit_t *bf_P;
+static u_32bit_t **bf_S;
 
 static int blowfish_expmem()
 {
   int i, tot = 0;
 
-  context;
   for (i = 0; i < BOXES; i++)
     if (box[i].P != NULL) {
-      tot += ((bf_N + 2) * sizeof(UWORD_32bits));
-      tot += (4 * sizeof(UWORD_32bits *));
-      tot += (4 * 256 * sizeof(UWORD_32bits));
+      tot += ((bf_N + 2) * sizeof(u_32bit_t));
+      tot += (4 * sizeof(u_32bit_t *));
+      tot += (4 * 256 * sizeof(u_32bit_t));
     }
   return tot;
 }
 
-static void blowfish_encipher(UWORD_32bits * xl, UWORD_32bits * xr)
+static void blowfish_encipher(u_32bit_t * xl, u_32bit_t * xr)
 {
   union aword Xl;
   union aword Xr;
@@ -86,7 +104,7 @@ static void blowfish_encipher(UWORD_32bits * xl, UWORD_32bits * xr)
   *xl = Xr.word;
 }
 
-static void blowfish_decipher(UWORD_32bits * xl, UWORD_32bits * xr)
+static void blowfish_decipher(u_32bit_t * xl, u_32bit_t * xr)
 {
   union aword Xl;
   union aword Xr;
@@ -136,29 +154,35 @@ static void blowfish_report(int idx, int details)
   }
 }
 
-static void blowfish_init(UBYTE_08bits * key, short keybytes)
+static void blowfish_init(u_8bit_t * key, int keybytes)
 {
   int i, j, bx;
   time_t lowest;
-  UWORD_32bits data;
-  UWORD_32bits datal;
-  UWORD_32bits datar;
+  u_32bit_t data;
+  u_32bit_t datal;
+  u_32bit_t datar;
   union aword temp;
 
-  /* is buffer already allocated for this? */
+  /* drummer: Fixes crash if key is longer than 80 char. This may cause the key
+   *          to not end with \00 but that's no problem.
+   */
+  if (keybytes > 80)
+    keybytes = 80;
+
+  /* Is buffer already allocated for this? */
   for (i = 0; i < BOXES; i++)
     if (box[i].P != NULL) {
       if ((box[i].keybytes == keybytes) &&
 	  (!strncmp((char *) (box[i].key), (char *) key, keybytes))) {
-	/* match! */
+	/* Match! */
 	box[i].lastuse = now;
 	bf_P = box[i].P;
 	bf_S = box[i].S;
 	return;
       }
     }
-  /* no pre-allocated buffer: make new one */
-  /* set 'bx' to empty buffer */
+  /* No pre-allocated buffer: make new one */
+  /* Set 'bx' to empty buffer */
   bx = (-1);
   for (i = 0; i < BOXES; i++) {
     if (box[i].P == NULL) {
@@ -167,7 +191,7 @@ static void blowfish_init(UBYTE_08bits * key, short keybytes)
     }
   }
   if (bx < 0) {
-    /* find oldest */
+    /* Find oldest */
     lowest = now;
     for (i = 0; i < BOXES; i++)
       if (box[i].lastuse <= lowest) {
@@ -179,20 +203,22 @@ static void blowfish_init(UBYTE_08bits * key, short keybytes)
       nfree(box[bx].S[i]);
     nfree(box[bx].S);
   }
-  /* initialize new buffer */
+  /* Initialize new buffer */
   /* uh... this is over 4k */
-  box[bx].P = (UWORD_32bits *) nmalloc((bf_N + 2) * sizeof(UWORD_32bits));
-  box[bx].S = (UWORD_32bits **) nmalloc(4 * sizeof(UWORD_32bits *));
+  box[bx].P = (u_32bit_t *) nmalloc((bf_N + 2) * sizeof(u_32bit_t));
+  box[bx].S = (u_32bit_t **) nmalloc(4 * sizeof(u_32bit_t *));
   for (i = 0; i < 4; i++)
-    box[bx].S[i] = (UWORD_32bits *) nmalloc(256 * sizeof(UWORD_32bits));
+    box[bx].S[i] = (u_32bit_t *) nmalloc(256 * sizeof(u_32bit_t));
   bf_P = box[bx].P;
   bf_S = box[bx].S;
   box[bx].keybytes = keybytes;
   strncpy(box[bx].key, key, keybytes);
+  box[bx].key[keybytes] = 0;
   box[bx].lastuse = now;
-  /* robey: reset blowfish boxes to initial state */
-  /* (i guess normally it just keeps scrambling them, but here it's
-   * important to get the same encrypted result each time) */
+  /* Robey: Reset blowfish boxes to initial state
+   * (I guess normally it just keeps scrambling them, but here it's
+   * important to get the same encrypted result each time)
+   */
   for (i = 0; i < bf_N + 2; i++)
     bf_P[i] = initbf_P[i];
   for (i = 0; i < 4; i++)
@@ -200,15 +226,17 @@ static void blowfish_init(UBYTE_08bits * key, short keybytes)
       bf_S[i][j] = initbf_S[i][j];
 
   j = 0;
-  for (i = 0; i < bf_N + 2; ++i) {
-    temp.word = 0;
-    temp.w.byte0 = key[j];
-    temp.w.byte1 = key[(j + 1) % keybytes];
-    temp.w.byte2 = key[(j + 2) % keybytes];
-    temp.w.byte3 = key[(j + 3) % keybytes];
-    data = temp.word;
-    bf_P[i] = bf_P[i] ^ data;
-    j = (j + 4) % keybytes;
+  if (keybytes > 0) { /* drummer: fixes crash if key=="" */
+    for (i = 0; i < bf_N + 2; ++i) {
+      temp.word = 0;
+      temp.w.byte0 = key[j];
+      temp.w.byte1 = key[(j + 1) % keybytes];
+      temp.w.byte2 = key[(j + 2) % keybytes];
+      temp.w.byte3 = key[(j + 3) % keybytes];
+      data = temp.word;
+      bf_P[i] = bf_P[i] ^ data;
+      j = (j + 4) % keybytes;
+    }
   }
   datal = 0x00000000;
   datar = 0x00000000;
@@ -226,14 +254,13 @@ static void blowfish_init(UBYTE_08bits * key, short keybytes)
   }
 }
 
-/* stuff below this line was written by robey for eggdrop use */
-
-/* of course, if you change either of these, then your userfile will
- * no longer be able to be shared. :) */
+/* Of course, if you change either of these, then your userfile will
+ * no longer be able to be shared. :)
+ */
 #define SALT1  0xdeadd061
 #define SALT2  0x23f6b095
 
-/* convert 64-bit encrypted password to text for userfile */
+/* Convert 64-bit encrypted password to text for userfile */
 static char *base64 = "./0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
 
 static int base64dec(char c)
@@ -248,11 +275,11 @@ static int base64dec(char c)
 
 static void blowfish_encrypt_pass(char *text, char *new)
 {
-  UWORD_32bits left, right;
+  u_32bit_t left, right;
   int n;
   char *p;
 
-  blowfish_init(text, strlen(text));
+  blowfish_init((unsigned char *) text, strlen(text));
   left = SALT1;
   right = SALT2;
   blowfish_encipher(&left, &right);
@@ -273,23 +300,27 @@ static void blowfish_encrypt_pass(char *text, char *new)
   *p = 0;
 }
 
-/* returned string must be freed when done with it! */
+/* Returned string must be freed when done with it!
+ */
 static char *encrypt_string(char *key, char *str)
 {
-  UWORD_32bits left, right;
-  char *p, *s, *dest, *d;
+  u_32bit_t left, right;
+  unsigned char *p;
+  char *s, *dest, *d;
   int i;
 
-  dest = (char *) nmalloc((strlen(str) + 9) * 2);
-  /* pad fake string with 8 bytes to make sure there's enough */
+  /* Pad fake string with 8 bytes to make sure there's enough */
   s = (char *) nmalloc(strlen(str) + 9);
   strcpy(s, str);
+  if ((!key) || (!key[0]))
+    return s;
   p = s;
+  dest = (char *) nmalloc((strlen(str) + 9) * 2);
   while (*p)
     p++;
   for (i = 0; i < 8; i++)
     *p++ = 0;
-  blowfish_init(key, strlen(key));
+  blowfish_init((unsigned char *) key, strlen(key));
   p = s;
   d = dest;
   while (*p) {
@@ -316,23 +347,26 @@ static char *encrypt_string(char *key, char *str)
   return dest;
 }
 
-/* returned string must be freed when done with it! */
+/* Returned string must be freed when done with it!
+ */
 static char *decrypt_string(char *key, char *str)
 {
-  UWORD_32bits left, right;
+  u_32bit_t left, right;
   char *p, *s, *dest, *d;
   int i;
 
-  dest = (char *) nmalloc(strlen(str) + 12);
-  /* pad encoded string with 0 bits in case it's bogus */
+  /* Pad encoded string with 0 bits in case it's bogus */
   s = (char *) nmalloc(strlen(str) + 12);
   strcpy(s, str);
+  if ((!key) || (!key[0]))
+    return s;
   p = s;
+  dest = (char *) nmalloc(strlen(str) + 12);
   while (*p)
     p++;
   for (i = 0; i < 12; i++)
     *p++ = 0;
-  blowfish_init(key, strlen(key));
+  blowfish_init((unsigned char *) key, strlen(key));
   p = s;
   d = dest;
   while (*p) {
@@ -358,13 +392,9 @@ static int tcl_encrypt STDVAR
   char *p;
 
   BADARGS(3, 3, " key string");
-  if (strlen(argv[1]) > 0) {
-    p = encrypt_string(argv[1], argv[2]);
-    Tcl_AppendResult(irp, p, NULL);
-    nfree(p);
-  } else {
-    Tcl_AppendResult(irp, argv[2], NULL);
-  }
+  p = encrypt_string(argv[1], argv[2]);
+  Tcl_AppendResult(irp, p, NULL);
+  nfree(p);
   return TCL_OK;
 }
 
@@ -373,45 +403,40 @@ static int tcl_decrypt STDVAR
   char *p;
 
   BADARGS(3, 3, " key string");
-  if (strlen(argv[1]) > 0) {
-    p = decrypt_string(argv[1], argv[2]);
-    Tcl_AppendResult(irp, p, NULL);
-    nfree(p);
-  } else {
-    Tcl_AppendResult(irp, argv[2], NULL);
-  }
+  p = decrypt_string(argv[1], argv[2]);
+  Tcl_AppendResult(irp, p, NULL);
+  nfree(p);
   return TCL_OK;
 }
 
 static int tcl_encpass STDVAR
 {
-  char p[512];
-
   BADARGS(2, 2, " string");
-  if (strlen(argv[1])>0) {
+  if (strlen(argv[1]) > 0) {
+    char p[16];
     blowfish_encrypt_pass(argv[1], p);
     Tcl_AppendResult(irp, p, NULL);
-  } else {
+  } else
     Tcl_AppendResult(irp, "", NULL);
-  }
   return TCL_OK;
 }
 
 static tcl_cmds mytcls[] =
 {
-  {"encrypt", tcl_encrypt},
-  {"decrypt", tcl_decrypt},
-  {"encpass", tcl_encpass},
-  {0, 0}
+  {"encrypt",	tcl_encrypt},
+  {"decrypt",	tcl_decrypt},
+  {"encpass",	tcl_encpass},
+  {NULL,	NULL}
 };
 
-/* you CANT -module an encryption module , so -module just resets it */
+/* You CANT -module an encryption module , so -module just resets it.
+ */
 static char *blowfish_close()
 {
   return "You can't unload an encryption module";
 }
 
-char *blowfish_start(Function *);
+EXPORT_SCOPE char *blowfish_start(Function *);
 
 static Function blowfish_table[] =
 {
@@ -425,27 +450,35 @@ static Function blowfish_table[] =
   (Function) decrypt_string,
 };
 
-/* initialize buffered boxes */
-char *blowfish_start(Function * global_funcs)
+char *blowfish_start(Function *global_funcs)
 {
   int i;
 
+  /* `global_funcs' is NULL if eggdrop is recovering from a restart.
+   *
+   * As the encryption module is never unloaded, only initialise stuff
+   * that got reset during restart, e.g. the tcl bindings.
+   */
   if (global_funcs) {
     global = global_funcs;
 
     if (!module_rename("blowfish", MODULE_NAME))
       return "Already loaded.";
+    /* Initialize buffered boxes */
     for (i = 0; i < BOXES; i++) {
       box[i].P = NULL;
       box[i].S = NULL;
       box[i].key[0] = 0;
       box[i].lastuse = 0L;
     }
-    context;
-    module_register(MODULE_NAME, blowfish_table, 2, 0);
-    if (!module_depend(MODULE_NAME, "eggdrop", 103, 0))
-      return "This module requires eggdrop1.3.0 or later";
-    add_hook(HOOK_ENCRYPT_PASS, blowfish_encrypt_pass);
+    module_register(MODULE_NAME, blowfish_table, 2, 1);
+    if (!module_depend(MODULE_NAME, "eggdrop", 106, 3)) {
+      module_undepend(MODULE_NAME);
+      return "This module requires eggdrop1.6.3 or later";
+    }
+    add_hook(HOOK_ENCRYPT_PASS, (Function) blowfish_encrypt_pass);
+    add_hook(HOOK_ENCRYPT_STRING, (Function) encrypt_string);
+    add_hook(HOOK_DECRYPT_STRING, (Function) decrypt_string);
   }
   add_tcl_commands(mytcls);
   return NULL;
