@@ -1,13 +1,33 @@
-/* 
- * console.c - part of console.mod
- * saved console settings based on console.tcl by
- * cmwagner/billyjoe/D. Senso
+/*
+ * console.c -- part of console.mod
+ *   saved console settings based on console.tcl
+ *   by cmwagner/billyjoe/D. Senso
+ *
+ * $Id: console.c,v 1.22 2002/01/02 03:46:38 guppy Exp $
+ */
+/*
+ * Copyright (C) 1999, 2000, 2001, 2002 Eggheads Development Team
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#define MAKING_CONSOLE
 #define MODULE_NAME "console"
-#include "../module.h"
+#define MAKING_CONSOLE
+#include "src/mod/module.h"
 #include <stdlib.h>
+#include "console.h"
 
 static Function *global = NULL;
 static int console_autosave = 0;
@@ -23,55 +43,53 @@ struct console_info {
   int conchan;
 };
 
+static struct user_entry_type USERENTRY_CONSOLE;
+
+
 static int console_unpack(struct userrec *u, struct user_entry *e)
 {
-  context;
-  if (e->name) {
-    char *p, *w, *o;
-    struct console_info *i;
+  struct console_info *ci = user_malloc(sizeof(struct console_info));
+  char *par, *arg;
 
-    p = e->u.list->extra;
-    e->u.list->extra = NULL;
-    list_type_kill(e->u.list);
-    e->u.extra = i = user_malloc(sizeof(struct console_info));
-
-    o = p;
-    w = newsplit(&p);
-    i->channel = user_malloc(strlen(w) + 1);
-    strcpy(i->channel, w);
-    w = newsplit(&p);
-    i->conflags = logmodes(w);
-    w = newsplit(&p);
-    i->stripflags = stripmodes(w);
-    w = newsplit(&p);
-    i->echoflags = (w[0] == '1') ? 1 : 0;
-    w = newsplit(&p);
-    i->page = atoi(w);
-    w = newsplit(&p);
-    i->conchan = atoi(w);
-    nfree(o);
-  }
+  par = e->u.list->extra;
+  arg = newsplit(&par);
+  ci->channel = user_malloc(strlen(arg) + 1);
+  strcpy(ci->channel, arg);
+  arg = newsplit(&par);
+  ci->conflags = logmodes(arg);
+  arg = newsplit(&par);
+  ci->stripflags = stripmodes(arg);
+  arg = newsplit(&par);
+  ci->echoflags = (arg[0] == '1') ? 1 : 0;
+  arg = newsplit(&par);
+  ci->page = atoi(arg);
+  arg = newsplit(&par);
+  ci->conchan = atoi(arg);
+  list_type_kill(e->u.list);
+  e->u.extra = ci;
   return 1;
 }
 
 static int console_pack(struct userrec *u, struct user_entry *e)
 {
-  if (!e->name) {
-    char work[1024];
-    struct console_info *i = e->u.extra;
-    int c = simple_sprintf(work, "%s %s %s %d %d %d",
-			   i->channel, masktype(i->conflags),
-			   stripmasktype(i->stripflags), i->echoflags,
-			   i->page, i->conchan);
+  char work[1024];
+  struct console_info *ci;
+  int l;
 
-    e->u.list = user_malloc(sizeof(struct list_type));
+  ci = (struct console_info *) e->u.extra;
 
-    e->u.list->next = NULL;
-    e->u.list->extra = user_malloc(c + 1);
-    strcpy(e->u.list->extra, work);
-    nfree(i->channel);
-    nfree(i);
-  }
+  l = simple_sprintf(work, "%s %s %s %d %d %d",
+		     ci->channel, masktype(ci->conflags),
+		     stripmasktype(ci->stripflags), ci->echoflags,
+		     ci->page, ci->conchan);
+
+  e->u.list = user_malloc(sizeof(struct list_type));
+  e->u.list->next = NULL;
+  e->u.list->extra = user_malloc(l + 1);
+  strcpy(e->u.list->extra, work);
+
+  nfree(ci->channel);
+  nfree(ci);
   return 1;
 }
 
@@ -79,18 +97,17 @@ static int console_kill(struct user_entry *e)
 {
   struct console_info *i = e->u.extra;
 
-  context;
   nfree(i->channel);
   nfree(i);
   nfree(e);
   return 1;
 }
 
-static int console_write_userfile(FILE * f, struct userrec *u, struct user_entry *e)
+static int console_write_userfile(FILE *f, struct userrec *u,
+				  struct user_entry *e)
 {
   struct console_info *i = e->u.extra;
 
-  context;
   if (fprintf(f, "--CONSOLE %s %s %s %d %d %d\n",
 	      i->channel, masktype(i->conflags),
 	      stripmasktype(i->stripflags), i->echoflags,
@@ -101,12 +118,24 @@ static int console_write_userfile(FILE * f, struct userrec *u, struct user_entry
 
 static int console_set(struct userrec *u, struct user_entry *e, void *buf)
 {
-  e->u.extra = buf;
-  context;
+  struct console_info *ci = (struct console_info *) e->u.extra;
+
+  if (!ci && !buf)
+    return 1;
+
+  if (ci != buf) {
+    if (ci) {
+      nfree(ci->channel);
+      nfree(ci);
+    }
+    ci = e->u.extra = buf;
+  }
+
+  /* Note: Do not share console info */
   return 1;
 }
 
-static int console_tcl_get(Tcl_Interp * irp, struct userrec *u,
+static int console_tcl_get(Tcl_Interp *irp, struct userrec *u,
 			   struct user_entry *e, int argc, char **argv)
 {
   char work[1024];
@@ -120,7 +149,7 @@ static int console_tcl_get(Tcl_Interp * irp, struct userrec *u,
   return TCL_OK;
 }
 
-int console_tcl_set(Tcl_Interp * irp, struct userrec *u,
+static int console_tcl_set(Tcl_Interp *irp, struct userrec *u,
 		    struct user_entry *e, int argc, char **argv)
 {
   struct console_info *i = e->u.extra;
@@ -129,7 +158,7 @@ int console_tcl_set(Tcl_Interp * irp, struct userrec *u,
   BADARGS(4, 9, " handle CONSOLE channel flags strip echo page conchan");
   if (!i) {
     i = user_malloc(sizeof(struct console_info));
-    bzero(i, sizeof(struct console_info));
+    egg_bzero(i, sizeof(struct console_info));
   }
   if (i->channel)
     nfree(i->channel);
@@ -153,30 +182,31 @@ int console_tcl_set(Tcl_Interp * irp, struct userrec *u,
       }
     }
   }
+  set_user(&USERENTRY_CONSOLE, u, i);
   return TCL_OK;
 }
 
-int console_expmem(struct user_entry *e)
+static int console_expmem(struct user_entry *e)
 {
   struct console_info *i = e->u.extra;
 
-  context;
   return sizeof(struct console_info) + strlen(i->channel) + 1;
 }
 
-void console_display(int idx, struct user_entry *e)
+static void console_display(int idx, struct user_entry *e)
 {
   struct console_info *i = e->u.extra;
 
-  context;
   if (dcc[idx].user && (dcc[idx].user->flags & USER_MASTER)) {
-    dprintf(idx, "  Saved Console Settings:\n");
-    dprintf(idx, "    Channel: %s\n", i->channel);
-    dprintf(idx, "    Console flags: %s, Strip flags: %s, Echo: %s\n",
-	    masktype(i->conflags), stripmasktype(i->stripflags),
-	    i->echoflags ? "yes" : "no");
-    dprintf(idx, "    Page setting: %d, Console channel: %d\n",
-	    i->page, i->conchan);
+    dprintf(idx, "  %s\n", CONSOLE_SAVED_SETTINGS);
+    dprintf(idx, "    %s %s\n", CONSOLE_CHANNEL, i->channel);
+    dprintf(idx, "    %s %s, %s %s, %s %s\n", CONSOLE_FLAGS,
+    	masktype(i->conflags), CONSOLE_STRIPFLAGS,
+	    stripmasktype(i->stripflags), CONSOLE_ECHO,
+	    i->echoflags ? CONSOLE_YES : CONSOLE_NO);
+    dprintf(idx, "    %s %d, %s %s%d\n", CONSOLE_PAGE_SETTING, i->page,
+            CONSOLE_CHANNEL2, (i->conchan < 100000) ? "" : "*",
+            i->conchan % 100000);
   }
 }
 
@@ -195,14 +225,14 @@ static int console_dupuser(struct userrec *new, struct userrec *old,
 
 static struct user_entry_type USERENTRY_CONSOLE =
 {
-  0,				/* always 0 ;) */
-  0,
+  NULL,				/* always 0 ;) */
+  NULL,
   console_dupuser,
   console_unpack,
   console_pack,
   console_write_userfile,
   console_kill,
-  0,
+  NULL,
   console_set,
   console_tcl_get,
   console_tcl_set,
@@ -219,10 +249,8 @@ static int console_chon(char *handle, int idx)
     if (i) {
       if (i->channel && i->channel[0])
 	strcpy(dcc[idx].u.chat->con_chan, i->channel);
-      if (i->conflags)
-	dcc[idx].u.chat->con_flags = i->conflags;
-      if (i->stripflags)
-	dcc[idx].u.chat->strip_flags = i->stripflags;
+      dcc[idx].u.chat->con_flags = i->conflags;
+      dcc[idx].u.chat->strip_flags = i->stripflags;
       if (i->echoflags)
 	dcc[idx].status |= STAT_ECHO;
       else
@@ -248,6 +276,7 @@ static int console_chon(char *handle, int idx)
       if (p) {
 	if (dcc[idx].u.chat->channel >= 0) {
 	    char x[1024];
+
 	    chanout_but(-1, dcc[idx].u.chat->channel,
 			"*** [%s] %s\n", dcc[idx].nick, p);
 	    simple_sprintf(x, "[%s] %s", dcc[idx].nick, p);
@@ -266,7 +295,7 @@ static int console_store(struct userrec *u, int idx, char *par)
 
   if (!i) {
     i = user_malloc(sizeof(struct console_info));
-    bzero(i, sizeof(struct console_info));
+    egg_bzero(i, sizeof(struct console_info));
   }
   if (i->channel)
     nfree(i->channel);
@@ -281,19 +310,21 @@ static int console_store(struct userrec *u, int idx, char *par)
     i->page = 0;
   i->conchan = dcc[idx].u.chat->channel;
   if (par) {
-    dprintf(idx, "Saved Console your Settings:\n");
-    dprintf(idx, "  Channel: %s\n", i->channel);
-    dprintf(idx, "  Console flags: %s, Strip flags: %s, Echo: %s\n",
-	    masktype(i->conflags), stripmasktype(i->stripflags),
-	    i->echoflags ? "yes" : "no");
-    dprintf(idx, "  Page setting: %d, Console channel: %d\n",
-	    i->page, i->conchan);
+    dprintf(idx, "%s\n", CONSOLE_SAVED_SETTINGS2);
+    dprintf(idx, "  %s %s\n", i->channel, CONSOLE_CHANNEL);
+    dprintf(idx, "  %s %s, %s %s, %s %s\n", CONSOLE_FLAGS,
+	    masktype(i->conflags), CONSOLE_STRIPFLAGS,
+	    stripmasktype(i->stripflags), CONSOLE_ECHO,
+	    i->echoflags ? CONSOLE_YES : CONSOLE_NO);
+    dprintf(idx, "  %s %d, %s %d\n", CONSOLE_PAGE_SETTING, i->page,
+            CONSOLE_CHANNEL2, i->conchan);
   }
   set_user(&USERENTRY_CONSOLE, u, i);
   return 0;
 }
 
-static int console_chof(char *handle, int idx)
+/* cmds.c:cmd_console calls this, better than chof bind - drummer,07/25/1999 */
+static int console_dostore(int idx)
 {
   if (console_autosave)
     console_store(dcc[idx].user, idx, NULL);
@@ -302,64 +333,62 @@ static int console_chof(char *handle, int idx)
 
 static tcl_ints myints[] =
 {
-  {"console-autosave", &console_autosave, 0},
-  {"force-channel", &force_channel, 0},
-  {"info-party", &info_party, 0},
-  {0, 0, 0}
+  {"console-autosave",	&console_autosave,	0},
+  {"force-channel",	&force_channel,		0},
+  {"info-party",	&info_party,		0},
+  {NULL,		NULL,			0}
 };
 
 static cmd_t mychon[] =
 {
-  {"*", "", console_chon, "console:chon"}
-};
-
-static cmd_t mychof[] =
-{
-  {"*", "", console_chof, "console:chof"}
+  {"*",		"",	console_chon,		"console:chon"},
+  {NULL,	NULL,	NULL,			NULL}
 };
 
 static cmd_t mydcc[] =
 {
-  {"store", "", console_store, NULL}
+  {"store",	"",	console_store,		NULL},
+  {NULL,	NULL,	NULL,			NULL}
 };
 
 static char *console_close()
 {
-  context;
-  rem_builtins(H_chon, mychon, 1);
-  rem_builtins(H_chof, mychof, 1);
-  rem_builtins(H_dcc, mydcc, 1);
+  rem_builtins(H_chon, mychon);
+  rem_builtins(H_dcc, mydcc);
   rem_tcl_ints(myints);
   rem_help_reference("console.help");
   del_entry_type(&USERENTRY_CONSOLE);
+  del_lang_section("console");
   module_undepend(MODULE_NAME);
   return NULL;
 }
 
-char *console_start();
+EXPORT_SCOPE char *console_start();
 
 static Function console_table[] =
 {
   (Function) console_start,
   (Function) console_close,
-  (Function) 0,
-  (Function) 0,
+  (Function) NULL,
+  (Function) NULL,
+  (Function) console_dostore,
 };
 
 char *console_start(Function * global_funcs)
 {
   global = global_funcs;
 
-  context;
-  module_register(MODULE_NAME, console_table, 1, 0);
-  if (!module_depend(MODULE_NAME, "eggdrop", 103, 0))
-    return "This module requires eggdrop1.3.0 or later";
-  add_builtins(H_chon, mychon, 1);
-  add_builtins(H_chof, mychof, 1);
-  add_builtins(H_dcc, mydcc, 1);
+  module_register(MODULE_NAME, console_table, 1, 1);
+  if (!module_depend(MODULE_NAME, "eggdrop", 106, 0)) {
+    module_undepend(MODULE_NAME);
+    return "This module requires eggdrop1.6.0 or later";
+  }
+  add_builtins(H_chon, mychon);
+  add_builtins(H_dcc, mydcc);
   add_tcl_ints(myints);
   add_help_reference("console.help");
   USERENTRY_CONSOLE.get = def_get;
   add_entry_type(&USERENTRY_CONSOLE);
+  add_lang_section("console");
   return NULL;
 }
