@@ -1,27 +1,44 @@
-/* 
- * seen.c   - Implement the seen.tcl script functionality via module.
+/*
+ * seen.c -- part of seen.mod
+ *  Implement the seen.tcl script functionality via module
  *
- *            by ButchBub - Scott G. Taylor (staylor@mrynet.com) 
+ * by ButchBub - Scott G. Taylor (staylor@mrynet.com)
  *
- *      REQUIRED: Eggdrop Module version 1.2.0
+ * 0.1     1997-07-29      Initial. [BB]
+ * 1.0     1997-07-31      Release. [BB]
+ * 1.1     1997-08-05      Add nick->handle lookup for NICK's. [BB]
+ * 1.2     1997-08-20      Minor fixes. [BB]
+ * 1.2a    1997-08-24      Minor fixes. [BB]
  *
- *      0.1     1997-07-29      Initial. [BB]
- *      1.0     1997-07-31      Release. [BB]
- *      1.1     1997-08-05      Add nick->handle lookup for NICK's. [BB]
- *      1.2     1997-08-20      Minor fixes. [BB]
- *      1.2a    1997-08-24      Minor fixes. [BB]
+ * $Id: seen.c,v 1.24 2002/01/02 03:46:39 guppy Exp $
+ */
+/*
+ * Copyright (C) 1999, 2000, 2001, 2002 Eggheads Development Team
  *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-/* 
+/*
  *  Currently, PUB, DCC and MSG commands are supported.  No party-line
  *      filtering is performed.
  *
  *  For boyfriend/girlfriend support, this module relies on the XTRA
  *      fields in the userfile to use BF and GF, respectively, for
- *      these fields.  
+ *      these fields.
  *
- *  userinfo1.0.tcl nicely compliments this script by providing 
+ *  userinfo1.0.tcl nicely compliments this script by providing
  *      the necessary commands to facilitate modification of these
  *      fields via DCC and IRC MSG commands.
  *
@@ -46,22 +63,22 @@
  *              your    owner
  *                      admin
  *                      (other)
- *              NICK's  boyfriend   
+ *              NICK's  boyfriend
  *                      bf
  *                      girlfriend
  *                      gf
  *
  */
 
-#define MAKING_SEEN
 #define MODULE_NAME "seen"
+#define MAKING_SEEN
 
 #include <time.h>
 
-#include "../module.h"
-#include "../../users.h"
-#include "../../chan.h"
-#include "../channels.mod/channels.h"
+#include "src/mod/module.h"
+#include "src/users.h"
+#include "src/chan.h"
+#include "channels.mod/channels.h"
 
 static Function *global = NULL;
 static void wordshift();
@@ -77,13 +94,13 @@ typedef struct {
 
 static trig_data trigdata[] =
 {
-  {"god", "Let's not get into a religious discussion, %s"},
-  {"jesus", "Let's not get into a religious discussion, %s"},
-  {"shit", "Here's looking at you, %s"},
-  {"yourself", "Yeah, whenever I look in a mirror..."},
-  {NULL, "You found me, %s!"},
-  {"elvis", "Last time I was on the moon man."},
-  {0, 0}
+  {"god",	"Let's not get into a religious discussion, %s"},
+  {"jesus",	"Let's not get into a religious discussion, %s"},
+  {"shit",	"Here's looking at you, %s"},
+  {"yourself",	"Yeah, whenever I look in a mirror..."},
+  {NULL,	"You found me, %s!"},
+  {"elvis",	"Last time I was on the moon man."},
+  {NULL,	NULL}
 };
 
 static int seen_expmem()
@@ -98,12 +115,11 @@ static int pub_seen(char *nick, char *host, char *hand,
 		    char *channel, char *text)
 {
   char prefix[50];
-  struct chanset_t *chan = findchan(channel);
+  struct chanset_t *chan = findchan_by_dname(channel);
 
-  context;
   if ((chan != NULL) && channel_seen(chan)) {
-    sprintf(prefix, "PRIVMSG %s :", channel);
-    do_seen(DP_HELP, prefix, nick, hand, channel, text);
+    sprintf(prefix, "PRIVMSG %s :", chan->name);
+    do_seen(DP_HELP, prefix, nick, hand, chan->dname, text);
   }
   return 0;
 }
@@ -112,12 +128,11 @@ static int msg_seen(char *nick, char *host, struct userrec *u, char *text)
 {
   char prefix[50];
 
-  context;
   if (!u) {
-    putlog(LOG_MISC, "*", "[%s!%s] seen %s", nick, host, text);
+    putlog(LOG_CMDS, "*", "[%s!%s] seen %s", nick, host, text);
     return 0;
   }
-  putlog(LOG_MISC, "*", "(%s!%s) !%s! SEEN %s", nick, host, u->handle, text);
+  putlog(LOG_CMDS, "*", "(%s!%s) !%s! SEEN %s", nick, host, u->handle, text);
   sprintf(prefix, "PRIVMSG %s :", nick);
   do_seen(DP_SERVER, prefix, nick, u->handle, "", text);
   return 0;
@@ -125,14 +140,13 @@ static int msg_seen(char *nick, char *host, struct userrec *u, char *text)
 
 static int dcc_seen(struct userrec *u, int idx, char *par)
 {
-  context;
   putlog(LOG_CMDS, "*", "#%s# seen %s", dcc[idx].nick, par);
   do_seen(idx, "", dcc[idx].nick, dcc[idx].nick, "", par);
   return 0;
 }
 
-static void do_seen(int idx, char *prefix, char *nick, char *hand, char *channel,
-		    char *text)
+static void do_seen(int idx, char *prefix, char *nick, char *hand,
+		    char *channel, char *text)
 {
   char stuff[512];
   char word1[512], word2[512];
@@ -144,7 +158,7 @@ static void do_seen(int idx, char *prefix, char *nick, char *hand, char *channel
   struct chanset_t *chan;
   struct laston_info *li;
   struct chanuserrec *cr;
-  memberlist *m;
+  memberlist *m = NULL;
   int onchan = 0;
   int i;
   time_t laston = 0;
@@ -156,10 +170,8 @@ static void do_seen(int idx, char *prefix, char *nick, char *hand, char *channel
   object[0] = 0;
   /* Was ANYONE specified */
   if (!text[0]) {
-    sprintf(stuff,
-	    "%sUm, %s, it might help if you ask me about _someone_...\n",
+    dprintf(idx, "%sUm, %s, it might help if you ask me about _someone_...\n",
 	    prefix, nick);
-    dprintf(idx, stuff);
     return;
   }
   wordshift(word1, text);
@@ -167,7 +179,6 @@ static void do_seen(int idx, char *prefix, char *nick, char *hand, char *channel
   /* Have we got a NICK's target? */
   if (oix == word1)
     return;			/* Skip anything starting with ' */
-  context;
   if (oix && *oix &&
       ((oix[1] && (oix[1] == 's' || oix[1] == 'S') && !oix[2]) ||
        (!oix[1] &&
@@ -185,11 +196,12 @@ static void do_seen(int idx, char *prefix, char *nick, char *hand, char *channel
       chan = chanset;
       while (chan) {
 	onchan = 0;
-	if ((m = ismember(chan, object))) {
+	m = ismember(chan, object);
+	if (m) {
 	  onchan = 1;
 	  sprintf(stuff, "%s!%s", object, m->userhost);
 	  urec = get_user_by_host(stuff);
-	  if (!urec || !strcasecmp(object, urec->handle))
+	  if (!urec || !egg_strcasecmp(object, urec->handle))
 	    break;
 	  strcat(whoredirect, object);
 	  strcat(whoredirect, " is ");
@@ -206,24 +218,24 @@ static void do_seen(int idx, char *prefix, char *nick, char *hand, char *channel
 	return;
       }
     }
-    if (!strcasecmp(word1, "bf") || !strcasecmp(word1, "boyfriend")) {
+    if (!egg_strcasecmp(word1, "bf") || !egg_strcasecmp(word1, "boyfriend")) {
       strcpy(whotarget, getxtra(object, "BF"));
       if (whotarget[0]) {
 	sprintf(whoredirect, "%s boyfriend is %s, and ",
 		fixnick(object), whotarget);
-	goto TARGETCONT;
+	goto targetcont;
       }
       dprintf(idx,
 	      "%sI don't know who %s boyfriend is, %s.\n",
 	      prefix, fixnick(object), nick);
       return;
     }
-    if (!strcasecmp(word1, "gf") || !strcasecmp(word1, "girlfriend")) {
+    if (!egg_strcasecmp(word1, "gf") || !egg_strcasecmp(word1, "girlfriend")) {
       strcpy(whotarget, getxtra(object, "GF"));
       if (whotarget[0]) {
 	sprintf(whoredirect, "%s girlfriend is %s, and ",
 		fixnick(object), whotarget);
-	goto TARGETCONT;
+	goto targetcont;
       }
       dprintf(idx,
 	      "%sI don't know who %s girlfriend is, %s.\n",
@@ -236,7 +248,7 @@ static void do_seen(int idx, char *prefix, char *nick, char *hand, char *channel
     return;
   }
   /* Keyword "my" */
-  if (!strcasecmp(word1, "my")) {
+  if (!egg_strcasecmp(word1, "my")) {
     wordshift(word1, text);
     if (!word1[0]) {
       dprintf(idx, "%sYour what, %s?\n", prefix, nick);
@@ -250,7 +262,7 @@ static void do_seen(int idx, char *prefix, char *nick, char *hand, char *channel
       return;
     }
     /* "my boyfriend" */
-    if (!strcasecmp(word1, "boyfriend") || !strcasecmp(word1, "bf")) {
+    if (!egg_strcasecmp(word1, "boyfriend") || !egg_strcasecmp(word1, "bf")) {
       strcpy(whotarget, getxtra(hand, "BF"));
       if (whotarget[0]) {
 	sprintf(whoredirect, "%s, your boyfriend is %s, and ",
@@ -263,7 +275,8 @@ static void do_seen(int idx, char *prefix, char *nick, char *hand, char *channel
       }
     }
     /* "my girlfriend" */
-    else if (!strcasecmp(word1, "girlfriend") || !strcasecmp(word1, "gf")) {
+    else if (!egg_strcasecmp(word1, "girlfriend") ||
+	      !egg_strcasecmp(word1, "gf")) {
       strcpy(whotarget, getxtra(hand, "GF"));
       if (whotarget[0]) {
 	sprintf(whoredirect, "%s, your girlfriend is %s, and ",
@@ -282,19 +295,19 @@ static void do_seen(int idx, char *prefix, char *nick, char *hand, char *channel
     }
   }
   /* "your" keyword */
-  else if (!strcasecmp(word1, "your")) {
+  else if (!egg_strcasecmp(word1, "your")) {
     wordshift(word1, text);
     /* "your admin" */
-    if (!strcasecmp(word1, "owner") || !strcasecmp(word1, "admin")) {
+    if (!egg_strcasecmp(word1, "owner") || !egg_strcasecmp(word1, "admin")) {
       if (admin[0]) {
 	strcpy(word2, admin);
 	wordshift(whotarget, word2);
 	strcat(whoredirect, "My owner is ");
 	strcat(whoredirect, whotarget);
 	strcat(whoredirect, ", and ");
-	if (!strcasecmp(whotarget, hand)) {
+	if (!egg_strcasecmp(whotarget, hand)) {
 	  strcat(whoredirect, "that's YOU");
-	  if (!strcasecmp(hand, nick)) {
+	  if (!egg_strcasecmp(hand, nick)) {
 	    strcat(whoredirect, "!!!");
 	  } else {
 	    strcat(whoredirect, ", ");
@@ -323,11 +336,10 @@ static void do_seen(int idx, char *prefix, char *nick, char *hand, char *channel
     return;
   }
   /* Otherwise, make the target to the first word and continue */
-  else {
+  else
     strcpy(whotarget, word1);
-  }
-  TARGETCONT:
-  context;
+
+targetcont:
   /* Looking for ones own nick? */
   if (!rfc_casecmp(nick, whotarget)) {
     dprintf(idx, "%s%sLooking for yourself, eh %s?\n",
@@ -337,11 +349,12 @@ static void do_seen(int idx, char *prefix, char *nick, char *hand, char *channel
   /* Check if nick is on a channel */
   chan = chanset;
   while (chan) {
-    if ((m = ismember(chan, whotarget))) {
+    m = ismember(chan, whotarget);
+    if (m) {
       onchan = 1;
       sprintf(word1, "%s!%s", whotarget, m->userhost);
       urec = get_user_by_host(word1);
-      if (!urec || !strcasecmp(whotarget, urec->handle))
+      if (!urec || !egg_strcasecmp(whotarget, urec->handle))
 	break;
       strcat(whoredirect, whotarget);
       strcat(whoredirect, " is ");
@@ -356,10 +369,10 @@ static void do_seen(int idx, char *prefix, char *nick, char *hand, char *channel
     chan = chanset;
     while (chan) {
       m = chan->channel.member;
-      while (m->nick[0]) {
+      while (m && m->nick[0]) {
 	sprintf(word2, "%s!%s", m->nick, m->userhost);
 	urec = get_user_by_host(word2);
-	if (urec && !strcasecmp(urec->handle, whotarget)) {
+	if (urec && !egg_strcasecmp(urec->handle, whotarget)) {
 	  onchan = 1;
 	  strcat(whoredirect, whotarget);
 	  strcat(whoredirect, " is ");
@@ -374,13 +387,11 @@ static void do_seen(int idx, char *prefix, char *nick, char *hand, char *channel
     }
   }
   /* Check if the target was on the channel, but is netsplit */
-  context;
-  chan = findchan(channel);
+  chan = findchan_by_dname(channel);
   if (chan) {
     m = ismember(chan, whotarget);
     if (m && chan_issplit(m)) {
-      dprintf(idx,
-	      "%s%s%s was just here, but got netsplit.\n",
+      dprintf(idx, "%s%s%s was just here, but got netsplit.\n",
 	      prefix, whoredirect, whotarget);
       return;
     }
@@ -398,13 +409,13 @@ static void do_seen(int idx, char *prefix, char *nick, char *hand, char *channel
     if (m && chan_issplit(m)) {
       dprintf(idx,
 	      "%s%s%s was just on %s, but got netsplit.\n",
-	      prefix, whoredirect, whotarget, chan->name);
+	      prefix, whoredirect, whotarget, chan->dname);
       return;
     }
     if (m) {
       dprintf(idx,
 	      "%s%s%s is on %s right now!\n",
-	      prefix, whoredirect, whotarget, chan->name);
+	      prefix, whoredirect, whotarget, chan->dname);
       return;
     }
     chan = chan->next;
@@ -422,14 +433,13 @@ static void do_seen(int idx, char *prefix, char *nick, char *hand, char *channel
   /* Is the target currently DCC CHAT to me on the botnet? */
   for (i = 0; i < dcc_total; i++) {
     if (dcc[i].type->flags & DCT_CHAT) {
-      if (!strcasecmp(whotarget, dcc[i].nick)) {
+      if (!egg_strcasecmp(whotarget, dcc[i].nick)) {
 	if (!rfc_casecmp(channel, dcc[i].u.chat->con_chan) &&
 	    dcc[i].u.chat->con_flags & LOG_PUBLIC) {
 	  strcat(whoredirect, whotarget);
 	  strcat(whoredirect,
 	   " is 'observing' this channel right now from my party line!");
-	  dprintf(idx, "%s%s\n",
-		  prefix, whoredirect);
+	  dprintf(idx, "%s%s\n", prefix, whoredirect);
 	} else {
 	  dprintf(idx,
 		  "%s%s%s is linked to me via DCC CHAT right now!\n",
@@ -441,7 +451,7 @@ static void do_seen(int idx, char *prefix, char *nick, char *hand, char *channel
   }
   /* Target known, but nowhere to be seen.  Give last IRC and botnet time */
   wordshift(word1, text);
-  if (!strcasecmp(word1, "anywhere"))
+  if (!egg_strcasecmp(word1, "anywhere"))
     cr = NULL;
   else
     for (cr = urec->chanrec; cr; cr = cr->next) {
@@ -485,7 +495,7 @@ static void do_seen(int idx, char *prefix, char *nick, char *hand, char *channel
   } else {
     strcpy(word2 + strlen(word2) - 2, " ago.");
   }
-  if (strchr(CHANMETA, lastonplace[0]) != NULL)
+  if (lastonplace[0] && (strchr(CHANMETA, lastonplace[0]) != NULL))
     sprintf(word1, "on IRC channel %s", lastonplace);
   else if (lastonplace[0] == '@')
     sprintf(word1, "on %s", lastonplace + 1);
@@ -522,11 +532,11 @@ static char *match_trigger(char *word)
   trig_data *t = trigdata;
 
   while (t->key) {
-    if (!strcasecmp(word, t->key))
+    if (!egg_strcasecmp(word, t->key))
       return t->text;
     t++;
   }
-  return (char *) 0;
+  return (char *) NULL;
 }
 
 static char *getxtra(char *hand, char *field)
@@ -541,10 +551,10 @@ static char *getxtra(char *hand, char *field)
     ue = find_user_entry(&USERENTRY_XTRA, urec);
     if (ue)
       for (xk = ue->u.extra; xk; xk = xk->next)
-	if (xk->key && !strcasecmp(xk->key, field)) {
+	if (xk->key && !egg_strcasecmp(xk->key, field)) {
 	  if (xk->data[0] == '{' && xk->data[strlen(xk->data) - 1] == '}' &&
 	      strlen(xk->data) > 2) {
-	    strncpy(fixit, &xk->data[1], strlen(xk->data - 9));
+	    strncpy(fixit, &xk->data[1], strlen(xk->data) - 2);
 	    fixit[strlen(xk->data) - 2] = 0;
 	    return fixit;
 	  } else {
@@ -559,12 +569,11 @@ static void wordshift(char *first, char *rest)
 {
   char *p, *q = rest;
 
-  LOOPIT:
-  p = newsplit(&q);
-  strcpy(first, p);
-  strcpy(rest, q);
-  if (!strcasecmp(first, "and") || !strcasecmp(first, "or"))
-    goto LOOPIT;
+  do {
+    p = newsplit(&q);
+    strcpy(first, p);
+    strcpy(rest, q);
+  } while (!egg_strcasecmp(first, "and") || !egg_strcasecmp(first, "or"));
 }
 
 /* Report on current seen info for .modulestat. */
@@ -577,17 +586,20 @@ static void seen_report(int idx, int details)
 /* PUB channel builtin commands. */
 static cmd_t seen_pub[] =
 {
-  {"seen", "", pub_seen, 0},
+  {"seen",	"",	pub_seen,	NULL},
+  {NULL,	NULL,	NULL,		NULL}
 };
 
 static cmd_t seen_dcc[] =
 {
-  {"seen", "", dcc_seen, 0},
+  {"seen",	"",	dcc_seen, 	NULL},
+  {NULL,	NULL,	NULL,		NULL}
 };
 
 static cmd_t seen_msg[] =
 {
-  {"seen", "", msg_seen, 0},
+  {"seen",	"",	msg_seen,	NULL},
+  {NULL,	NULL,	NULL,		NULL}
 };
 
 static int server_seen_setup(char *mod)
@@ -595,7 +607,7 @@ static int server_seen_setup(char *mod)
   p_tcl_bind_list H_temp;
 
   if ((H_temp = find_bind_table("msg")))
-    add_builtins(H_temp, seen_msg, 1);
+    add_builtins(H_temp, seen_msg);
   return 0;
 }
 
@@ -604,32 +616,33 @@ static int irc_seen_setup(char *mod)
   p_tcl_bind_list H_temp;
 
   if ((H_temp = find_bind_table("pub")))
-    add_builtins(H_temp, seen_pub, 1);
+    add_builtins(H_temp, seen_pub);
   return 0;
 }
 
 static cmd_t seen_load[] =
 {
-  {"server", "", server_seen_setup, 0},
-  {"irc", "", irc_seen_setup, 0},
+  {"server",	"",	server_seen_setup,	NULL},
+  {"irc",	"",	irc_seen_setup,		NULL},
+  {NULL,	NULL,	NULL,			NULL}
 };
 
 static char *seen_close()
 {
   p_tcl_bind_list H_temp;
 
-  rem_builtins(H_load, seen_load, 2);
-  rem_builtins(H_dcc, seen_dcc, 1);
+  rem_builtins(H_load, seen_load);
+  rem_builtins(H_dcc, seen_dcc);
   rem_help_reference("seen.help");
   if ((H_temp = find_bind_table("pub")))
-    rem_builtins(H_temp, seen_pub, 1);
+    rem_builtins(H_temp, seen_pub);
   if ((H_temp = find_bind_table("msg")))
-    rem_builtins(H_temp, seen_msg, 1);
+    rem_builtins(H_temp, seen_msg);
   module_undepend(MODULE_NAME);
   return NULL;
 }
 
-char *seen_start();
+EXPORT_SCOPE char *seen_start();
 
 static Function seen_table[] =
 {
@@ -643,16 +656,16 @@ char *seen_start(Function * egg_func_table)
 {
   global = egg_func_table;
 
-  context;
   module_register(MODULE_NAME, seen_table, 2, 0);
-  if (!module_depend(MODULE_NAME, "eggdrop", 103, 0))
-    return
-      "MODULE `seen' cannot be loaded on Eggdrops prior to version 1.3.0";
-  add_builtins(H_load, seen_load, 2);
-  add_builtins(H_dcc, seen_dcc, 1);
+  if (!module_depend(MODULE_NAME, "eggdrop", 106, 0)) {
+    module_undepend(MODULE_NAME);
+    return "This module needs eggdrop1.6.0 or later";
+  }
+  add_builtins(H_load, seen_load);
+  add_builtins(H_dcc, seen_dcc);
   add_help_reference("seen.help");
-  server_seen_setup(0);
-  irc_seen_setup(0);
+  server_seen_setup(NULL);
+  irc_seen_setup(NULL);
   trigdata[4].key = botnetnick;
   return NULL;
 }
