@@ -1,36 +1,50 @@
-/* 
- * wire.c   - An encrypted partyline communication.
- *            Compatible with wire.tcl.
- *
- *            by ButchBub - Scott G. Taylor (staylor@mrynet.com) 
- *
- *      REQUIRED: Eggdrop Modules (see dependancies below).  
- *                Wire will only compile and work on the latest Required
- *                Eggdrop version listed below.
- *
- *    Version   Date            Req'd Eggver    Notes                    Who
- *    .......   ..........      ............    ....................     ......
- *      1.0     1997-07-17      1.2.0           Initial.                 BB
- *      1.1     1997-07-28      1.2.0           Release version.         BB
- *      1.2     1997-08-19      1.2.1           Update and bugfixes.     BB
- *      1.3     1997-09-24      1.2.2.0         Reprogrammed for 1.2.2   BB
- *      1.4     1997-11-25      1.2.2.0         Added language addition  Kirk
- *      1.5     1998-07-12      1.3.0.0         Fixed ;me and updated    BB
- *
+/*
+ * wire.c -- part of wire.mod
+ *   An encrypted partyline communication.
+ *   Compatible with wire.tcl.
  *   This module does not support wire usage in the files area.
  *
+ * by ButchBub - Scott G. Taylor (staylor@mrynet.com)
+ *
+ * Version   Date            Req'd Eggver    Notes                    Who
+ * .......   ..........      ............    ....................     ......
+ * 1.0       1997-07-17      1.2.0           Initial.                 BB
+ * 1.1       1997-07-28      1.2.0           Release version.         BB
+ * 1.2       1997-08-19      1.2.1           Update and bugfixes.     BB
+ * 1.3       1997-09-24      1.2.2.0         Reprogrammed for 1.2.2   BB
+ * 1.4       1997-11-25      1.2.2.0         Added language addition  Kirk
+ * 1.5       1998-07-12      1.3.0.0         Fixed ;me and updated    BB
+ *
+ * $Id: wire.c,v 1.21 2002/01/02 03:46:41 guppy Exp $
+ */
+/*
+ * Copyright (C) 1999, 2000, 2001, 2002 Eggheads Development Team
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#define MAKING_WIRE
 #define MODULE_NAME "wire"
-#include "../module.h"
-#include "../../users.h"
-#include "../../chan.h"
+#define MAKING_WIRE
+#include "src/mod/module.h"
+#include "src/users.h"
+#include "src/chan.h"
 #include <time.h>
 #include "wire.h"
 
 #undef global
-static Function *global = NULL, *blowfish_funcs = NULL;
+static Function *global = NULL, *encryption_funcs = NULL;
 
 typedef struct wire_t {
   int sock;
@@ -55,11 +69,10 @@ static int wire_expmem()
   wire_list *w = wirelist;
   int size = 0;
 
-  context;
   while (w) {
-    size += sizeof(wirelist);
-    size += strlen(w->crypt);
-    size += strlen(w->key);
+    size += sizeof(wire_list);
+    size += strlen(w->crypt) + 1;
+    size += strlen(w->key) + 1;
     w = w->next;
   }
   return size;
@@ -87,7 +100,6 @@ static void wire_filter(char *from, char *cmd, char *param)
   char idle[20];
   char *enctmp;
 
-  context;
   strcpy(wirecrypt, &cmd[5]);
   strcpy(wiretmp, param);
   nsplit(wirereq, param);
@@ -98,18 +110,19 @@ static void wire_filter(char *from, char *cmd, char *param)
  */
 
   if (!strcmp(wirereq, "!wirereq")) {
-    context;
     nsplit(wirewho, param);
     while (w) {
       if (!strcmp(w->crypt, wirecrypt)) {
+	int idx = findanyidx(w->sock);
+
 	reqsock = atoi(wirewho);
-	if (now2 - dcc[findanyidx(w->sock)].timeval > 300) {
+	if (now2 - dcc[idx].timeval > 300) {
 	  unsigned long Days, hrs, mins;
 
-	  Days = (now2 - dcc[findanyidx(w->sock)].timeval) / 86400;
-	  hrs = ((now2 - dcc[findanyidx(w->sock)].timeval) -
+	  Days = (now2 - dcc[idx].timeval) / 86400;
+	  hrs = ((now2 - dcc[idx].timeval) -
 		 (Days * 86400)) / 3600;
-	  mins = ((now2 - dcc[findanyidx(w->sock)].timeval) -
+	  mins = ((now2 - dcc[idx].timeval) -
 		  (hrs * 3600)) / 60;
 	  if (Days > 0)
 	    sprintf(idle, " \[%s %lud%luh]", WIRE_IDLE, Days, hrs);
@@ -120,19 +133,19 @@ static void wire_filter(char *from, char *cmd, char *param)
 	} else
 	  idle[0] = 0;
 	sprintf(wirereq, "----- %c%-9s %-9s  %s%s",
-		geticon(findanyidx(w->sock)),
-		dcc[findanyidx(w->sock)].nick,
+		geticon(idx),
+		dcc[idx].nick,
 		botnetnick,
-		dcc[findanyidx(w->sock)].host, idle);
+		dcc[idx].host, idle);
 	enctmp = encrypt_string(w->key, wirereq);
 	strcpy(wiretmp, enctmp);
 	nfree(enctmp);
 	sprintf(wirereq, "zapf %s %s !wire%s !wireresp %s %s %s",
 		botnetnick, from, wirecrypt, wirewho, param, wiretmp);
 	dprintf(nextbot(from), "%s\n", wirereq);
-	if (dcc[findanyidx(w->sock)].u.chat->away) {
+	if (dcc[idx].u.chat->away) {
 	  sprintf(wirereq, "-----    %s: %s\n", WIRE_AWAY,
-		  dcc[findanyidx(w->sock)].u.chat->away);
+		  dcc[idx].u.chat->away);
 	  enctmp = encrypt_string(w->key, wirereq);
 	  strcpy(wiretmp, enctmp);
 	  nfree(enctmp);
@@ -146,19 +159,20 @@ static void wire_filter(char *from, char *cmd, char *param)
     return;
   }
   if (!strcmp(wirereq, "!wireresp")) {
-    context;
     nsplit(wirewho, param);
     reqsock = atoi(wirewho);
     w = wirelist;
     nsplit(wiretmp2, param);
     while (w) {
       if (w->sock == reqsock) {
+	int idx = findanyidx(reqsock);
+
 	enctmp = decrypt_string(w->key, wiretmp2);
 	strcpy(wirewho, enctmp);
 	nfree(enctmp);
-	if (!strcmp(dcc[findanyidx(reqsock)].nick, wirewho)) {
+	if (!strcmp(dcc[idx].nick, wirewho)) {
 	  enctmp = decrypt_string(w->key, param);
-	  dprintf(findanyidx(reqsock), "%s\n", enctmp);
+	  dprintf(idx, "%s\n", enctmp);
 	  nfree(enctmp);
 	  return;
 	}
@@ -167,7 +181,6 @@ static void wire_filter(char *from, char *cmd, char *param)
     }
     return;
   }
-  context;
   while (w) {
     if (!strcmp(wirecrypt, w->crypt))
       wire_display(findanyidx(w->sock), w->key, wirereq, param);
@@ -179,7 +192,6 @@ static void wire_display(int idx, char *key, char *from, char *message)
 {
   char *enctmp;
 
-  context;
   enctmp = decrypt_string(key, message);
   if (from[0] == '!')
     dprintf(idx, "----- > %s %s\n", &from[1], enctmp + 1);
@@ -193,10 +205,9 @@ static int cmd_wirelist(struct userrec *u, int idx, char *par)
   wire_list *w = wirelist;
   int entry = 0;
 
-  context;
-  dprintf(idx, "Current Wire table:  (Base table address = %U)\n", w);
+  dprintf(idx, "Current Wire table:  (Base table address = %p)\n", w);
   while (w) {
-    dprintf(idx, "entry %d: w=%U  idx=%d  sock=%d  next=%U\n",
+    dprintf(idx, "entry %d: w=%p  idx=%d  sock=%d  next=%p\n",
 	    ++entry, w, findanyidx(w->sock), w->sock, w->next);
     w = w->next;
   }
@@ -210,7 +221,6 @@ static int cmd_onwire(struct userrec *u, int idx, char *par)
   char idle[20], *enctmp;
   time_t now2 = now;
 
-  context;
   w = wirelist;
   while (w) {
     if (w->sock == dcc[idx].sock)
@@ -235,12 +245,14 @@ static int cmd_onwire(struct userrec *u, int idx, char *par)
   w2 = wirelist;
   while (w2) {
     if (!strcmp(w2->key, w->key)) {
-      if (now2 - dcc[findanyidx(w2->sock)].timeval > 300) {
+      int idx2 = findanyidx(w2->sock);
+
+      if (now2 - dcc[idx2].timeval > 300) {
 	unsigned long Days, hrs, mins;
 
-	Days = (now2 - dcc[findanyidx(w2->sock)].timeval) / 86400;
-	hrs = ((now2 - dcc[findanyidx(w2->sock)].timeval) - (Days * 86400)) / 3600;
-	mins = ((now2 - dcc[findanyidx(w2->sock)].timeval) - (hrs * 3600)) / 60;
+	Days = (now2 - dcc[idx2].timeval) / 86400;
+	hrs = ((now2 - dcc[idx2].timeval) - (Days * 86400)) / 3600;
+	mins = ((now2 - dcc[idx2].timeval) - (hrs * 3600)) / 60;
 	if (Days > 0)
 	  sprintf(idle, " \[%s %lud%luh]", WIRE_IDLE, Days, hrs);
 	else if (hrs > 0)
@@ -250,12 +262,12 @@ static int cmd_onwire(struct userrec *u, int idx, char *par)
       } else
 	idle[0] = 0;
       dprintf(idx, "----- %c%-9s %-9s  %s%s\n",
-	      geticon(findanyidx(w2->sock)),
-	      dcc[findanyidx(w2->sock)].nick,
-	      botnetnick, dcc[findanyidx(w2->sock)].host, idle);
-      if (dcc[findanyidx(w2->sock)].u.chat->away)
+	      geticon(idx2),
+	      dcc[idx2].nick,
+	      botnetnick, dcc[idx2].host, idle);
+      if (dcc[idx2].u.chat->away)
 	dprintf(idx, "-----    %s: %s\n", WIRE_AWAY,
-		dcc[findanyidx(w2->sock)].u.chat->away);
+		dcc[idx2].u.chat->away);
     }
     w2 = w2->next;
   }
@@ -266,9 +278,8 @@ static int cmd_wire(struct userrec *u, int idx, char *par)
 {
   wire_list *w = wirelist;
 
-  context;
   if (!par[0]) {
-    dprintf(idx, "%s: .wire [<encrypt-key>|OFF|info]\n", USAGE);
+    dprintf(idx, "%s: .wire [<encrypt-key>|OFF|info]\n", MISC_USAGE);
     return 0;
   }
   while (w) {
@@ -276,7 +287,7 @@ static int cmd_wire(struct userrec *u, int idx, char *par)
       break;
     w = w->next;
   }
-  if (!strcasecmp(par, "off")) {
+  if (!egg_strcasecmp(par, "off")) {
     if (w) {
       wire_leave(w->sock);
       dprintf(idx, "%s\n", WIRE_NOLONGERWIRED);
@@ -285,7 +296,7 @@ static int cmd_wire(struct userrec *u, int idx, char *par)
     dprintf(idx, "%s\n", WIRE_NOTONWIRE);
     return 0;
   }
-  if (!strcasecmp(par, "info")) {
+  if (!egg_strcasecmp(par, "info")) {
     if (w)
       dprintf(idx, "%s '%s'.\n", WIRE_CURRENTLYON, w->key);
     else
@@ -319,7 +330,6 @@ static void wire_join(int idx, char *key)
   char *enctmp;
   wire_list *w = wirelist, *w2;
 
-  context;
   while (w) {
     if (w->next == 0)
       break;
@@ -336,7 +346,6 @@ static void wire_join(int idx, char *key)
   w->key = (char *) nmalloc(strlen(key) + 1);
   strcpy(w->key, key);
   w->next = 0;
-  context;
   enctmp = encrypt_string(w->key, "wire");
   strcpy(wiretmp, enctmp);
   nfree(enctmp);
@@ -372,7 +381,7 @@ static void wire_join(int idx, char *key)
     wire_bot[0].name = wirecmd;
     wire_bot[0].flags = "";
     wire_bot[0].func = (Function) wire_filter;
-    add_builtins(H_bot, wire_bot, 1);
+    add_builtins(H_bot, wire_bot);
   }
 }
 
@@ -386,7 +395,6 @@ static void wire_leave(int sock)
   wire_list *w2 = wirelist;
   wire_list *wlast = wirelist;
 
-  context;
   while (w) {
     if (w->sock == sock)
       break;
@@ -411,12 +419,13 @@ static void wire_leave(int sock)
   while (w2) {
     if (w2->sock != sock && !strcmp(w2->key, w->key)) {
       dprintf(findanyidx(w2->sock), "----- %s %s\n",
-	      dcc[findanyidx(w2->sock)].nick, WIRE_LEFT);
+	      dcc[findanyidx(w->sock)].nick, WIRE_LEFT);
     }
     w2 = w2->next;
   }
   /* Check to see if someone else is using this wire key.
-   * If so, then don't remove the wire filter binding */
+   * If so, then don't remove the wire filter binding.
+   */
   w2 = wirelist;
   while (w2) {
     if (w2 != w && !strcmp(w2->key, w->key))
@@ -427,7 +436,7 @@ static void wire_leave(int sock)
     wire_bot[0].name = wirecmd;
     wire_bot[0].flags = "";
     wire_bot[0].func = (Function) wire_filter;
-    rem_builtins(H_bot, wire_bot, 1);
+    rem_builtins(H_bot, wire_bot);
   }
   w2 = wirelist;
   wlast = 0;
@@ -462,7 +471,6 @@ static char *cmd_putwire(int idx, char *message)
   char wiretmp2[512];
   char *enctmp;
 
-  context;
   while (w) {
     if (w->sock == dcc[idx].sock)
       break;
@@ -505,7 +513,6 @@ static void wire_report(int idx, int details)
   int wiretot = 0;
   wire_list *w = wirelist;
 
-  context;
   while (w) {
     wiretot++;
     w = w->next;
@@ -517,19 +524,22 @@ static void wire_report(int idx, int details)
 
 static cmd_t wire_dcc[] =
 {
-  {"wire", "", cmd_wire, 0},
-  {"onwire", "", cmd_onwire, 0},
-  {"wirelist", "n", cmd_wirelist, 0},
+  {"wire",	"",	(Function) cmd_wire,		NULL},
+  {"onwire",	"",	(Function) cmd_onwire,		NULL},
+  {"wirelist",	"n",	(Function) cmd_wirelist,	NULL},
+  {NULL,	NULL,	NULL,				NULL}
 };
 
 static cmd_t wire_chof[] =
 {
-  {"*", "", (Function) chof_wire, "wire:chof"},
+  {"*",		"",	(Function) chof_wire,		"wire:chof"},
+  {NULL,	NULL,	NULL,				NULL}
 };
 
 static cmd_t wire_filt[] =
 {
-  {";*", "", (Function) cmd_putwire, "wire:filt"},
+  {";*",	"",	(Function) cmd_putwire,		"wire:filt"},
+  {NULL,	NULL,	NULL,				NULL}
 };
 
 static char *wire_close()
@@ -539,9 +549,9 @@ static char *wire_close()
   char *enctmp;
   p_tcl_bind_list H_temp;
 
-  /* Remove any current wire encrypt bindings
-   * for now, don't worry about duplicate unbinds */
-  context;
+  /* Remove any current wire encrypt bindings for now, don't worry
+   * about duplicate unbinds.
+   */
   while (w) {
     enctmp = encrypt_string(w->key, "wire");
     sprintf(wiretmp, "!wire%s", enctmp);
@@ -549,7 +559,7 @@ static char *wire_close()
     wire_bot[0].name = wiretmp;
     wire_bot[0].flags = "";
     wire_bot[0].func = (Function) wire_filter;
-    rem_builtins(H_bot, wire_bot, 1);
+    rem_builtins(H_bot, wire_bot);
     w = w->next;
   }
   w = wirelist;
@@ -559,16 +569,17 @@ static char *wire_close()
     wire_leave(w->sock);
     w = wirelist;
   }
-  rem_builtins(H_dcc, wire_dcc, 3);
+  rem_builtins(H_dcc, wire_dcc);
   H_temp = find_bind_table("filt");
-  rem_builtins(H_temp, wire_filt, 1);
+  rem_builtins(H_temp, wire_filt);
   H_temp = find_bind_table("chof");
-  rem_builtins(H_temp, wire_chof, 1);
+  rem_builtins(H_temp, wire_chof);
+  del_lang_section("wire");
   module_undepend(MODULE_NAME);
   return NULL;
 }
 
-char *wire_start();
+EXPORT_SCOPE char *wire_start();
 
 static Function wire_table[] =
 {
@@ -580,29 +591,25 @@ static Function wire_table[] =
 
 char *wire_start(Function * global_funcs)
 {
-  module_entry *me;
   p_tcl_bind_list H_temp;
 
   global = global_funcs;
 
-  context;
   module_register(MODULE_NAME, wire_table, 2, 0);
-  if (!module_depend(MODULE_NAME, "eggdrop", 103, 0)) {
+  if (!module_depend(MODULE_NAME, "eggdrop", 106, 0)) {
     module_undepend(MODULE_NAME);
-    return WIRE_VERSIONERROR;
+    return "This module requires eggdrop1.6.0 or later";
   }
-  me = module_find("encryption", 2, 0);
-  blowfish_funcs = me->funcs;
-  add_builtins(H_dcc, wire_dcc, 3);
+  if (!(encryption_funcs = module_depend(MODULE_NAME, "encryption", 2, 1))) {
+    module_undepend(MODULE_NAME);
+    return "You need an encryption module to use the wire module.";
+  }    
+  add_builtins(H_dcc, wire_dcc);
   H_temp = find_bind_table("filt");
-  add_builtins(H_filt, wire_filt, 1);
+  add_builtins(H_filt, wire_filt);
   H_temp = find_bind_table("chof");
-  add_builtins(H_chof, wire_chof, 1);
-  wirelist = 0;
-  /* Absolutely HUGE! change to this module
-   * Ah well, at least the language file gets loaded 'on demand' - Kirk */
-  cmd_loadlanguage(0, DP_LOG, "wire.english");
-  /* Even better thing is, all the modules can be lang'ed now :)
-   * and you just need this one line at the bottom of the code - Kirk */
+  add_builtins(H_chof, wire_chof);
+  wirelist = NULL;
+  add_lang_section("wire");
   return NULL;
 }
