@@ -1,36 +1,39 @@
-/*
- * ===================================================================
+/* 
+ * rfc1459.c
+ * 
+ * $Id: rfc1459.c,v 1.5 2002/01/02 08:06:14 tothwolf Exp $
+ */
+/* 
+ * Copyright (C) 1990 Jarkko Oikarinen
+ * Copyright (C) 1999, 2000, 2001, 2002 Eggheads Development Team
+ * 
  * This code was more or less cloned from the ircd-hybrid 5.3 source.
  * The original code was written by Otto Harkoonen and even though it
  * it not entirely in synch with section 2.2 of RFC1459 in that it
  * additionally defines carat as an uppercase version of tilde, it's
  * what is in the servers themselves so we're going with it this way.
- *
+ * 
  * If for some reason someone who maintains the source for ircd decides
  * to change the code to be completely RFC compliant, the change here
  * would be absolutely miniscule.
- *
+ * 
  * BTW, since carat characters are allowed in nicknames and tildes are
  * not, I stronly suggest that people convert to uppercase when doing
  * comparisons or creation of hash elements (which tcl laughably calls
  * arrays) to avoid making entries with impossible nicknames in them.
- *
+ * 
  * --+ Dagmar
- * ===================================================================
  */
 
-#include "rfc1459.h"
-#include <string.h>
+#include "main.h"
 
-#ifdef RFC_COMPLIANT
-
-int rfc_casecmp(char *s1, char *s2)
+int _rfc_casecmp(const char *s1, const char *s2)
 {
   register unsigned char *str1 = (unsigned char *) s1;
   register unsigned char *str2 = (unsigned char *) s2;
   register int res;
 
-  while ((res = rfc_toupper(*str1) - rfc_toupper(*str2)) == 0) {
+  while (!(res = rfc_toupper(*str1) - rfc_toupper(*str2))) {
     if (*str1 == '\0')
       return 0;
     str1++;
@@ -39,22 +42,35 @@ int rfc_casecmp(char *s1, char *s2)
   return (res);
 }
 
-int rfc_ncasecmp(char *str1, char *str2, int n)
+int _rfc_ncasecmp(const char *str1, const char *str2, int n)
 {
   register unsigned char *s1 = (unsigned char *) str1;
   register unsigned char *s2 = (unsigned char *) str2;
   register int res;
 
-  while ((res = rfc_toupper(*s1) - rfc_toupper(*s2)) == 0) {
+  while (!(res = rfc_toupper(*s1) - rfc_toupper(*s2))) {
     s1++;
     s2++;
     n--;
-    if (n == 0 || (*s1 == '\0' && *s2 == '\0'))
+    if (!n || (*s1 == '\0' && *s2 == '\0'))
       return 0;
   }
   return (res);
 }
 
+unsigned char rfc_tolowertab[];
+unsigned char rfc_touppertab[];
+
+int _rfc_tolower(int c)
+{
+  return rfc_tolowertab[(unsigned char)(c)];
+}
+
+int _rfc_toupper(int c)
+{
+  return rfc_touppertab[(unsigned char)(c)];
+}
+
 unsigned char rfc_tolowertab[] =
 {0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xa,
  0xb, 0xc, 0xd, 0xe, 0xf, 0x10, 0x11, 0x12, 0x13, 0x14,
@@ -123,14 +139,3 @@ unsigned char rfc_touppertab[] =
  0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9,
  0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff};
 
-#else				/* Dalnet?? */
-
-int rfc_casecmp(char *s1, char *s2) {
-  return strcasecmp(s1, s2);
-}
-
-int rfc_ncasecmp(char *s1, char *s2, int n) {
-  return strncasecmp(s1, s2, n);
-}
-
-#endif				/* RFC_COMPLIANT */
