@@ -1,36 +1,49 @@
 /*
  * tcldcc.c -- handles:
- * Tcl stubs for the dcc commands
- * 
- * dprintf'ized, 1aug1996
+ *   Tcl stubs for the dcc commands
+ *
+ * $Id: tcldcc.c,v 1.33 2002/01/02 03:46:36 guppy Exp $
  */
 /*
- * This file is part of the eggdrop source code
- * copyright (c) 1997 Robey Pointer
- * and is distributed according to the GNU general public license.
- * For full details, read the top of 'main.c' or the file called
- * COPYING that was distributed with this code.
+ * Copyright (C) 1997 Robey Pointer
+ * Copyright (C) 1999, 2000, 2001, 2002 Eggheads Development Team
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
 #include "main.h"
 #include "tandem.h"
+#include "modules.h"
+
+extern Tcl_Interp	*interp;
+extern tcl_timer_t	*timer,
+			*utimer;
+extern struct dcc_t	*dcc;
+extern int		 dcc_total, backgrd, parties, make_userfile,
+			 do_restart, remote_boots, max_dcc;
+extern char		 botnetnick[];
+extern party_t		*party;
+extern tand_t		*tandbot;
+extern time_t		 now;
+
+/* Traffic stuff. */
+extern unsigned long otraffic_irc, otraffic_irc_today, itraffic_irc, itraffic_irc_today, otraffic_bn, otraffic_bn_today, itraffic_bn, itraffic_bn_today, otraffic_dcc, otraffic_dcc_today, itraffic_dcc, itraffic_dcc_today, otraffic_trans, otraffic_trans_today, itraffic_trans, itraffic_trans_today, otraffic_unknown, otraffic_unknown_today, itraffic_unknown, itraffic_unknown_today;
+
+int			 enable_simul = 0;
+static struct portmap	*root = NULL;
 
-extern Tcl_Interp *interp;
-extern tcl_timer_t *timer, *utimer;
-extern struct dcc_t *dcc;
-extern int dcc_total;
-extern char botnetnick[];
-extern int backgrd;
-extern party_t *party;
-extern int parties;
-extern int make_userfile;
-extern int do_restart;
-extern int remote_boots;
-extern tand_t *tandbot;
-extern int max_dcc;
-int enable_simul = 0;
-extern time_t now;
-static struct portmap *root = NULL;
 
 int expmem_tcldcc(void)
 {
@@ -49,7 +62,6 @@ static int tcl_putdcc STDVAR
 {
   int i, j;
 
-  context;
   BADARGS(3, 3, " idx text");
   i = atoi(argv[1]);
   j = findidx(i);
@@ -58,32 +70,36 @@ static int tcl_putdcc STDVAR
     return TCL_ERROR;
   }
   dumplots(-i, "", argv[2]);
-
   return TCL_OK;
 }
 
-/* function added by drummer@sophia.jpte.hu
- * allows tcl scripts to send out raw data
- * can be used for fast server write (idx=0)
- * usage: putdccraw <idx> <size> <rawdata> 
- * example: putdccraw 6 13 "eggdrop rulz\n" */
+/* Allows tcl scripts to send out raw data. Can be used for fast server
+ * write (idx=0)
+ *
+ * usage:
+ * 	putdccraw <idx> <size> <rawdata>
+ * example:
+ * 	putdccraw 6 13 "eggdrop rulz\n"
+ *
+ * (added by drummer@sophia.jpte.hu)
+ */
+
 static int tcl_putdccraw STDVAR
 {
-  int i, j, z;
+  int i, j = 0, z;
 
-  context;
   BADARGS(4, 4, " idx size text");
   z = atoi(argv[1]);
-  j = 0;
   for (i = 0; i < dcc_total; i++) {
-    if (!(strcmp(dcc[i].nick, "(server)")) && (z == 0)) {
+    if (!z && !strcmp(dcc[i].nick, "(server)")) {
       j = dcc[i].sock;
-    }
-    if (dcc[i].sock == z) {
+      break;
+    } else if (dcc[i].sock == z) {
       j = dcc[i].sock;
+      break;
     }
   }
-  if (j == 0) {
+  if (i == dcc_total) {
     Tcl_AppendResult(irp, "invalid idx", NULL);
     return TCL_ERROR;
   }
@@ -93,27 +109,25 @@ static int tcl_putdccraw STDVAR
 
 static int tcl_dccsimul STDVAR
 {
-  int i, idx;
-
-  context;
   BADARGS(3, 3, " idx command");
   if (enable_simul) {
-    i = atoi(argv[1]);
-    idx = findidx(i);
-    if ((idx >= 0) && (dcc[idx].type->flags & DCT_SIMUL)) {
+    int idx = findidx(atoi(argv[1]));
+
+    if (idx >= 0 && (dcc[idx].type->flags & DCT_SIMUL)) {
       int l = strlen(argv[2]);
 
       if (l > 510) {
 	l = 510;
-	argv[2][510] = 0;	/* restrict length of cmd */
+	argv[2][510] = 0;	/* Restrict length of cmd */
       }
       if (dcc[idx].type && dcc[idx].type->activity) {
 	dcc[idx].type->activity(idx, argv[2], l);
 	return TCL_OK;
       }
-    }
-  }
-  Tcl_AppendResult(irp, "invalid idx", NULL);
+    } else
+      Tcl_AppendResult(irp, "invalid idx", NULL);
+  } else
+    Tcl_AppendResult(irp, "simul disabled", NULL);
   return TCL_ERROR;
 }
 
@@ -121,10 +135,8 @@ static int tcl_dccbroadcast STDVAR
 {
   char msg[401];
 
-  context;
   BADARGS(2, 2, " message");
-  strncpy(msg, argv[1], 400);
-  msg[400] = 0;
+  strncpyz(msg, argv[1], sizeof msg);
   chatout("*** %s\n", msg);
   botnet_send_chat(-1, botnetnick, msg);
   return TCL_OK;
@@ -133,69 +145,58 @@ static int tcl_dccbroadcast STDVAR
 static int tcl_hand2idx STDVAR
 {
   int i;
-  char s[10];
+  char s[11];
 
-  context;
   BADARGS(2, 2, " nickname");
   for (i = 0; i < dcc_total; i++)
-    if ((!strcasecmp(argv[1], dcc[i].nick)) &&
-	 (dcc[i].type->flags & DCT_SIMUL)) {
-      simple_sprintf(s, "%d", dcc[i].sock);
+    if ((dcc[i].type->flags & DCT_SIMUL) &&
+        !egg_strcasecmp(argv[1], dcc[i].nick)) {
+      egg_snprintf(s, sizeof s, "%ld", dcc[i].sock);
       Tcl_AppendResult(irp, s, NULL);
       return TCL_OK;
     }
   Tcl_AppendResult(irp, "-1", NULL);
-
   return TCL_OK;
 }
 
 static int tcl_getchan STDVAR
 {
-  char s[10];
-  int idx, i;
+  char s[7];
+  int idx;
 
-  context;
   BADARGS(2, 2, " idx");
-  i = atoi(argv[1]);
-  idx = findidx(i);
-  if (idx < 0) {
-    Tcl_AppendResult(irp, "invalid idx", NULL);
-    return TCL_ERROR;
-  }
-  if ((dcc[idx].type != &DCC_CHAT) && (dcc[idx].type != &DCC_SCRIPT)) {
+  idx = findidx(atoi(argv[1]));
+  if (idx < 0 ||
+      (dcc[idx].type != &DCC_CHAT && dcc[idx].type != &DCC_SCRIPT)) {
     Tcl_AppendResult(irp, "invalid idx", NULL);
     return TCL_ERROR;
   }
   if (dcc[idx].type == &DCC_SCRIPT)
-    sprintf(s, "%d", dcc[idx].u.script->u.chat->channel);
+    egg_snprintf(s, sizeof s, "%d", dcc[idx].u.script->u.chat->channel);
   else
-    sprintf(s, "%d", dcc[idx].u.chat->channel);
+    egg_snprintf(s, sizeof s, "%d", dcc[idx].u.chat->channel);
   Tcl_AppendResult(irp, s, NULL);
   return TCL_OK;
 }
 
 static int tcl_setchan STDVAR
 {
-  int idx, i, chan;
+  int idx, chan;
+  module_entry *me;
 
-  context;
   BADARGS(3, 3, " idx channel");
-  i = atoi(argv[1]);
-  idx = findidx(i);
-  if (idx < 0) {
+  idx = findidx(atoi(argv[1]));
+  if (idx < 0 ||
+      (dcc[idx].type != &DCC_CHAT && dcc[idx].type != &DCC_SCRIPT)) {
     Tcl_AppendResult(irp, "invalid idx", NULL);
     return TCL_ERROR;
   }
-  if ((dcc[idx].type != &DCC_CHAT) && (dcc[idx].type != &DCC_SCRIPT)) {
-    Tcl_AppendResult(irp, "invalid idx", NULL);
-    return TCL_ERROR;
-  }
-  if ((argv[2][0] < '0') || (argv[2][0] > '9')) {
-    if ((!strcmp(argv[2], "-1")) || (!strcasecmp(argv[2], "off")))
+  if (argv[2][0] < '0' || argv[2][0] > '9') {
+    if (!strcmp(argv[2], "-1") || !egg_strcasecmp(argv[2], "off"))
       chan = (-1);
     else {
       Tcl_SetVar(irp, "chan", argv[2], 0);
-      if ((Tcl_VarEval(irp, "assoc ", "$chan", NULL) != TCL_OK) ||
+      if (Tcl_VarEval(irp, "assoc ", "$chan", NULL) != TCL_OK ||
 	  !interp->result[0]) {
 	Tcl_AppendResult(irp, "channel name is invalid", NULL);
 	return TCL_ERROR;
@@ -226,6 +227,12 @@ static int tcl_setchan STDVAR
     check_tcl_chjn(botnetnick, dcc[idx].nick, chan, geticon(idx),
 		   dcc[idx].sock, dcc[idx].host);
   }
+  /* Console autosave. */
+  if ((me = module_find("console", 1, 1))) {
+    Function *func = me->funcs;
+
+    (func[CONSOLE_DOSTORE]) (idx);
+  }
   return TCL_OK;
 }
 
@@ -234,7 +241,6 @@ static int tcl_dccputchan STDVAR
   int chan;
   char msg[401];
 
-  context;
   BADARGS(3, 3, " channel message");
   chan = atoi(argv[1]);
   if ((chan < 0) || (chan > 199999)) {
@@ -242,9 +248,7 @@ static int tcl_dccputchan STDVAR
 		     NULL);
     return TCL_ERROR;
   }
-  strncpy(msg, argv[2], 400);
-
-  msg[400] = 0;
+  strncpyz(msg, argv[2], sizeof msg);
   chanout_but(-1, chan, "*** %s\n", argv[2]);
   botnet_send_chan(-1, botnetnick, NULL, chan, argv[2]);
   check_tcl_bcst(botnetnick, chan, argv[2]);
@@ -254,32 +258,32 @@ static int tcl_dccputchan STDVAR
 static int tcl_console STDVAR
 {
   int i, j, pls, arg;
+  module_entry *me;
 
-  context;
   BADARGS(2, 4, " idx ?channel? ?console-modes?");
-  j = atoi(argv[1]);
-  i = findidx(j);
-  if (i < 0) {
-    Tcl_AppendResult(irp, "invalid idx", NULL);
-    return TCL_ERROR;
-  }
-  if (dcc[i].type != &DCC_CHAT) {
+  i = findidx(atoi(argv[1]));
+  if (i < 0 || dcc[i].type != &DCC_CHAT) {
     Tcl_AppendResult(irp, "invalid idx", NULL);
     return TCL_ERROR;
   }
   pls = 1;
   for (arg = 2; arg < argc; arg++) {
-    if ((strchr(CHANMETA, argv[arg][0]) != NULL) ||
-	(argv[arg][0] == '*')) {
-      if ((argv[arg][0] != '*') && (!findchan(argv[arg]))) {
+    if (argv[arg][0] && ((strchr(CHANMETA, argv[arg][0]) != NULL) ||
+	(argv[arg][0] == '*'))) {
+      if ((argv[arg][0] != '*') && (!findchan_by_dname(argv[arg]))) {
+        /* If we dont find the channel, and it starts with a +... assume it
+         * should be the console flags to set.
+         */
+        if (argv[arg][0]=='+')
+          goto do_console_flags;
 	Tcl_AppendResult(irp, "invalid channel", NULL);
 	return TCL_ERROR;
       }
-      strncpy(dcc[i].u.chat->con_chan, argv[arg], 80);
-      dcc[i].u.chat->con_chan[80] = 0;
+      strncpyz(dcc[i].u.chat->con_chan, argv[arg], 81);
     } else {
       if ((argv[arg][0] != '+') && (argv[arg][0] != '-'))
 	dcc[i].u.chat->con_flags = 0;
+      do_console_flags:
       for (j = 0; j < strlen(argv[arg]); j++) {
 	if (argv[arg][j] == '+')
 	  pls = 1;
@@ -300,22 +304,23 @@ static int tcl_console STDVAR
   }
   Tcl_AppendElement(irp, dcc[i].u.chat->con_chan);
   Tcl_AppendElement(irp, masktype(dcc[i].u.chat->con_flags));
+  /* Console autosave. */
+  if (argc > 2 && (me = module_find("console", 1, 1))) {
+    Function *func = me->funcs;
+
+    (func[CONSOLE_DOSTORE]) (i);
+  }
   return TCL_OK;
 }
 
 static int tcl_strip STDVAR
 {
   int i, j, pls, arg;
+  module_entry *me;
 
-  context;
   BADARGS(2, 4, " idx ?strip-flags?");
-  j = atoi(argv[1]);
-  i = findidx(j);
-  if (i < 0) {
-    Tcl_AppendResult(irp, "invalid idx", NULL);
-    return TCL_ERROR;
-  }
-  if (dcc[i].type != &DCC_CHAT) {
+  i = findidx(atoi(argv[1]));
+  if (i < 0 || dcc[i].type != &DCC_CHAT) {
     Tcl_AppendResult(irp, "invalid idx", NULL);
     return TCL_ERROR;
   }
@@ -341,22 +346,23 @@ static int tcl_strip STDVAR
     }
   }
   Tcl_AppendElement(irp, stripmasktype(dcc[i].u.chat->strip_flags));
+  /* Console autosave. */
+  if (argc > 2 && (me = module_find("console", 1, 1))) {
+    Function *func = me->funcs;
+
+    (func[CONSOLE_DOSTORE]) (i);
+  }
   return TCL_OK;
 }
 
 static int tcl_echo STDVAR
 {
-  int i, j;
+  int i;
+  module_entry *me;
 
-  context;
   BADARGS(2, 3, " idx ?status?");
-  j = atoi(argv[1]);
-  i = findidx(j);
-  if (i < 0) {
-    Tcl_AppendResult(irp, "invalid idx", NULL);
-    return TCL_ERROR;
-  }
-  if (dcc[i].type != &DCC_CHAT) {
+  i = findidx(atoi(argv[1]));
+  if (i < 0 || dcc[i].type != &DCC_CHAT) {
     Tcl_AppendResult(irp, "invalid idx", NULL);
     return TCL_ERROR;
   }
@@ -370,30 +376,31 @@ static int tcl_echo STDVAR
     Tcl_AppendResult(irp, "1", NULL);
   else
     Tcl_AppendResult(irp, "0", NULL);
+  /* Console autosave. */
+  if (argc > 2 && (me = module_find("console", 1, 1))) {
+    Function *func = me->funcs;
+
+    (func[CONSOLE_DOSTORE]) (i);
+  }
   return TCL_OK;
 }
 
 static int tcl_page STDVAR
 {
-  int i, j;
+  int i;
   char x[20];
+  module_entry *me;
 
-  context;
   BADARGS(2, 3, " idx ?status?");
-  j = atoi(argv[1]);
-  i = findidx(j);
-  if (i < 0) {
-    Tcl_AppendResult(irp, "invalid idx", NULL);
-    return TCL_ERROR;
-  }
-  if (dcc[i].type != &DCC_CHAT) {
+  i = findidx(atoi(argv[1]));
+  if (i < 0 || dcc[i].type != &DCC_CHAT) {
     Tcl_AppendResult(irp, "invalid idx", NULL);
     return TCL_ERROR;
   }
   if (argc == 3) {
     int l = atoi(argv[2]);
 
-    if (l == 0)
+    if (!l)
       dcc[i].status &= ~STAT_PAGE;
     else {
       dcc[i].status |= STAT_PAGE;
@@ -401,22 +408,26 @@ static int tcl_page STDVAR
     }
   }
   if (dcc[i].status & STAT_PAGE) {
-    sprintf(x, "%d", dcc[i].u.chat->max_line);
+    egg_snprintf(x, sizeof x, "%d", dcc[i].u.chat->max_line);
     Tcl_AppendResult(irp, x, NULL);
   } else
     Tcl_AppendResult(irp, "0", NULL);
+  /* Console autosave. */
+  if ((argc > 2) && (me = module_find("console", 1, 1))) {
+    Function *func = me->funcs;
+
+    (func[CONSOLE_DOSTORE]) (i);
+  }
   return TCL_OK;
 }
 
 static int tcl_control STDVAR
 {
-  int idx, i;
+  int idx;
   void *hold;
 
-  context;
   BADARGS(3, 3, " idx command");
-  i = atoi(argv[1]);
-  idx = findidx(i);
+  idx = findidx(atoi(argv[1]));
   if (idx < 0) {
     Tcl_AppendResult(irp, "invalid idx", NULL);
     return TCL_ERROR;
@@ -433,12 +444,13 @@ static int tcl_control STDVAR
   }
   hold = dcc[idx].u.other;
   dcc[idx].u.script = get_data_ptr(sizeof(struct script_info));
-
   dcc[idx].u.script->u.other = hold;
   dcc[idx].u.script->type = dcc[idx].type;
   dcc[idx].type = &DCC_SCRIPT;
-  strncpy(dcc[idx].u.script->command, argv[2], 120);
-  dcc[idx].u.script->command[119] = 0;
+  /* Do not buffer data anymore. All received and stored data is passed
+     over to the dcc functions from now on.  */
+  sockoptions(dcc[idx].sock, EGG_OPTION_UNSET, SOCK_BUFFER);
+  strncpyz(dcc[idx].u.script->command, argv[2], 120);
   return TCL_OK;
 }
 
@@ -446,10 +458,9 @@ static int tcl_valididx STDVAR
 {
   int idx;
 
-  context;
   BADARGS(2, 2, " idx");
   idx = findidx(atoi(argv[1]));
-  if ((idx < 0) || !(dcc[idx].type->flags & DCT_VALIDIDX))
+  if (idx < 0 || !(dcc[idx].type->flags & DCT_VALIDIDX))
      Tcl_AppendResult(irp, "0", NULL);
   else
      Tcl_AppendResult(irp, "1", NULL);
@@ -458,22 +469,18 @@ static int tcl_valididx STDVAR
 
 static int tcl_killdcc STDVAR
 {
-  int idx, i;
+  int idx;
 
-  contextnote(argv[1] ? argv[1] : "argv[1] == NULL!");
   BADARGS(2, 3, " idx ?reason?");
-  i = atoi(argv[1]);
-  idx = findidx(i);
+  idx = findidx(atoi(argv[1]));
   if (idx < 0) {
     Tcl_AppendResult(irp, "invalid idx", NULL);
     return TCL_ERROR;
   }
-  context;
-
-  /* don't kill terminal socket */
+  /* Don't kill terminal socket */
   if ((dcc[idx].sock == STDOUT) && !backgrd)
     return TCL_OK;
-  /* make sure 'whom' info is updated for other bots */
+  /* Make sure 'whom' info is updated for other bots */
   if (dcc[idx].type->flags & DCT_CHAT) {
     chanout_but(idx, dcc[idx].u.chat->channel, "*** %s has left the %s%s%s\n",
 		dcc[idx].nick, dcc[idx].u.chat ? "channel" : "partyline",
@@ -484,8 +491,7 @@ static int tcl_killdcc STDVAR
       check_tcl_chpt(botnetnick, dcc[idx].nick, dcc[idx].sock,
 		     dcc[idx].u.chat->channel);
     check_tcl_chof(dcc[idx].nick, dcc[idx].sock);
-    /* no notice is sent to the party line -- that's the scripts' job */
-    /* well now it does go through, but the script can add a reason */
+    /* Notice is sent to the party line, the script can add a reason. */
   }
   killsock(dcc[idx].sock);
   lostdcc(idx);
@@ -497,16 +503,13 @@ static int tcl_putbot STDVAR
   int i;
   char msg[401];
 
-  context;
   BADARGS(3, 3, " botnick message");
   i = nextbot(argv[1]);
   if (i < 0) {
     Tcl_AppendResult(irp, "bot is not in the botnet", NULL);
     return TCL_ERROR;
   }
-  strncpy(msg, argv[2], 400);
-
-  msg[400] = 0;
+  strncpyz(msg, argv[2], sizeof msg);
   botnet_send_zapf(i, botnetnick, argv[1], msg);
   return TCL_OK;
 }
@@ -515,22 +518,18 @@ static int tcl_putallbots STDVAR
 {
   char msg[401];
 
-  context;
   BADARGS(2, 2, " message");
-  strncpy(msg, argv[1], 400);
-  msg[400] = 0;
+  strncpyz(msg, argv[1], sizeof msg);
   botnet_send_zapf_broad(-1, botnetnick, NULL, msg);
   return TCL_OK;
 }
 
 static int tcl_idx2hand STDVAR
 {
-  int i, idx;
+  int idx;
 
-  context;
   BADARGS(2, 2, " idx");
-  i = atoi(argv[1]);
-  idx = findidx(i);
+  idx = findidx(atoi(argv[1]));
   if (idx < 0) {
     Tcl_AppendResult(irp, "invalid idx", NULL);
     return TCL_ERROR;
@@ -543,7 +542,6 @@ static int tcl_islinked STDVAR
 {
   int i;
 
-  context;
   BADARGS(2, 2, " bot");
   i = nextbot(argv[1]);
   if (i < 0)
@@ -557,7 +555,6 @@ static int tcl_bots STDVAR
 {
   tand_t *bot;
 
-  context;
   BADARGS(1, 1, "");
   for (bot = tandbot; bot; bot = bot->next)
      Tcl_AppendElement(irp, bot->bot);
@@ -570,7 +567,6 @@ static int tcl_botlist STDVAR
   char *list[4], *p;
   char sh[2], string[20];
 
-  context;
   BADARGS(1, 1, "");
   sh[1] = 0;
   list[3] = sh;
@@ -578,57 +574,59 @@ static int tcl_botlist STDVAR
   for (bot = tandbot; bot; bot = bot->next) {
     list[0] = bot->bot;
     list[1] = (bot->uplink == (tand_t *) 1) ? botnetnick : bot->uplink->bot;
-    strcpy(string, int_to_base10(bot->ver));
+    strncpyz(string, int_to_base10(bot->ver), sizeof string);
     sh[0] = bot->share;
     p = Tcl_Merge(4, list);
     Tcl_AppendElement(irp, p);
-    n_free(p, "", 0);
+    Tcl_Free((char *) p);
   }
   return TCL_OK;
 }
 
-/* list of { idx nick host type } */
-/* type: "chat", "files", "script" */
+/* list of { idx nick host type {other}  timestamp}
+ */
 static int tcl_dcclist STDVAR
 {
   int i;
   char idxstr[10];
-  char *list[5], *p;
+  char timestamp[11];
+  char *list[6], *p;
   char other[160];
 
-  context;
   BADARGS(1, 2, " ?type?");
   for (i = 0; i < dcc_total; i++) {
-    if ((argc == 1) ||
-	(dcc[i].type && !strcasecmp(dcc[i].type->name, argv[1]))) {
-      sprintf(idxstr, "%ld", dcc[i].sock);
+    if (argc == 1 ||
+	((argc == 2) && (dcc[i].type && !egg_strcasecmp(dcc[i].type->name, argv[1])))) {
+      egg_snprintf(idxstr, sizeof idxstr, "%ld", dcc[i].sock);
+      egg_snprintf(timestamp, sizeof timestamp, "%ld", dcc[i].timeval);
       if (dcc[i].type && dcc[i].type->display)
 	dcc[i].type->display(i, other);
       else {
-	sprintf(other, "?:%lX  !! ERROR !!", (long) dcc[i].type);
+	egg_snprintf(other, sizeof other, "?:%lX  !! ERROR !!",
+		     (long) dcc[i].type);
 	break;
       }
       list[0] = idxstr;
-
       list[1] = dcc[i].nick;
       list[2] = dcc[i].host;
       list[3] = dcc[i].type ? dcc[i].type->name : "*UNKNOWN*";
       list[4] = other;
-      p = Tcl_Merge(5, list);
+      list[5] = timestamp;
+      p = Tcl_Merge(6, list);
       Tcl_AppendElement(irp, p);
-      n_free(p, "", 0);
+      Tcl_Free((char *) p);
     }
   }
   return TCL_OK;
 }
 
-/* list of { nick bot host flag idletime awaymsg [channel]} */
+/* list of { nick bot host flag idletime awaymsg [channel]}
+ */
 static int tcl_whom STDVAR
 {
-  char c[2], idle[10], work[20], *list[7], *p;
+  char c[2], idle[11], work[20], *list[7], *p;
   int chan, i;
 
-  context;
   BADARGS(2, 2, " chan");
   if (argv[1][0] == '*')
      chan = -1;
@@ -651,10 +649,10 @@ static int tcl_whom STDVAR
   }
   for (i = 0; i < dcc_total; i++)
     if (dcc[i].type == &DCC_CHAT) {
-      if ((dcc[i].u.chat->channel == chan) || (chan == -1)) {
+      if (dcc[i].u.chat->channel == chan || chan == -1) {
 	c[0] = geticon(i);
 	c[1] = 0;
-	sprintf(idle, "%lu", (now - dcc[i].timeval) / 60);
+	egg_snprintf(idle, sizeof idle, "%lu", (now - dcc[i].timeval) / 60);
 	list[0] = dcc[i].nick;
 	list[1] = botnetnick;
 	list[2] = dcc[i].host;
@@ -662,22 +660,22 @@ static int tcl_whom STDVAR
 	list[4] = idle;
 	list[5] = dcc[i].u.chat->away ? dcc[i].u.chat->away : "";
 	if (chan == -1) {
-	  sprintf(work, "%d", dcc[i].u.chat->channel);
+	  egg_snprintf(work, sizeof work, "%d", dcc[i].u.chat->channel);
 	  list[6] = work;
 	}
 	p = Tcl_Merge((chan == -1) ? 7 : 6, list);
 	Tcl_AppendElement(irp, p);
-	n_free(p, "", 0);
+	Tcl_Free((char *) p);
       }
     }
   for (i = 0; i < parties; i++) {
-    if ((party[i].chan == chan) || (chan == -1)) {
+    if (party[i].chan == chan || chan == -1) {
       c[0] = party[i].flag;
       c[1] = 0;
       if (party[i].timer == 0L)
 	strcpy(idle, "0");
       else
-	sprintf(idle, "%lu", (now - party[i].timer) / 60);
+	egg_snprintf(idle, sizeof idle, "%lu", (now - party[i].timer) / 60);
       list[0] = party[i].nick;
       list[1] = party[i].bot;
       list[2] = party[i].from ? party[i].from : "";
@@ -685,12 +683,12 @@ static int tcl_whom STDVAR
       list[4] = idle;
       list[5] = party[i].status & PLSTAT_AWAY ? party[i].away : "";
       if (chan == -1) {
-	sprintf(work, "%d", party[i].chan);
+	egg_snprintf(work, sizeof work, "%d", party[i].chan);
 	list[6] = work;
       }
       p = Tcl_Merge((chan == -1) ? 7 : 6, list);
       Tcl_AppendElement(irp, p);
-      n_free(p, "", 0);
+      Tcl_Free((char *) p);
     }
   }
   return TCL_OK;
@@ -700,45 +698,36 @@ static int tcl_dccused STDVAR
 {
   char s[20];
 
-  context;
   BADARGS(1, 1, "");
-  sprintf(s, "%d", dcc_total);
+  egg_snprintf(s, sizeof s, "%d", dcc_total);
   Tcl_AppendResult(irp, s, NULL);
   return TCL_OK;
 }
 
 static int tcl_getdccidle STDVAR
 {
-  int i, x, idx;
+  int  x, idx;
   char s[21];
 
-  context;
   BADARGS(2, 2, " idx");
-  i = atoi(argv[1]);
-  idx = findidx(i);
-  if ((idx < 0) || (dcc[idx].type == &DCC_SCRIPT)) {
+  idx = findidx(atoi(argv[1]));
+  if (idx < 0) {
     Tcl_AppendResult(irp, "invalid idx", NULL);
     return TCL_ERROR;
   }
-  x = (now - (dcc[idx].timeval));
-  sprintf(s, "%d", x);
+  x = (now - dcc[idx].timeval);
+  egg_snprintf(s, sizeof s, "%d", x);
   Tcl_AppendElement(irp, s);
   return TCL_OK;
 }
 
 static int tcl_getdccaway STDVAR
 {
-  int i, idx;
+  int idx;
 
-  context;
   BADARGS(2, 2, " idx");
-  i = atol(argv[1]);
-  idx = findidx(i);
-  if (idx < 0) {
-    Tcl_AppendResult(irp, "invalid idx", NULL);
-    return TCL_ERROR;
-  }
-  if (dcc[idx].type != &DCC_CHAT) {
+  idx = findidx(atol(argv[1]));
+  if (idx < 0 || dcc[idx].type != &DCC_CHAT) {
     Tcl_AppendResult(irp, "invalid idx", NULL);
     return TCL_ERROR;
   }
@@ -750,17 +739,11 @@ static int tcl_getdccaway STDVAR
 
 static int tcl_setdccaway STDVAR
 {
-  int i, idx;
+  int idx;
 
-  context;
   BADARGS(3, 3, " idx message");
-  i = atol(argv[1]);
-  idx = findidx(i);
-  if (idx < 0) {
-    Tcl_AppendResult(irp, "invalid idx", NULL);
-    return TCL_ERROR;
-  }
-  if (dcc[idx].type != &DCC_CHAT) {
+  idx = findidx(atol(argv[1]));
+  if (idx < 0 || dcc[idx].type != &DCC_CHAT) {
     Tcl_AppendResult(irp, "invalid idx", NULL);
     return TCL_ERROR;
   }
@@ -780,23 +763,19 @@ static int tcl_link STDVAR
   int x, i;
   char bot[HANDLEN + 1], bot2[HANDLEN + 1];
 
-  context;
   BADARGS(2, 3, " ?via-bot? bot");
-  strncpy(bot, argv[1], HANDLEN);
-  bot[HANDLEN] = 0;
-  if (argc == 2)
-     x = botlink("", -2, bot);
-  else {
+  strncpyz(bot, argv[1], sizeof bot);
+  if (argc == 3) {
     x = 1;
-    strncpy(bot2, argv[2], HANDLEN);
-    bot2[HANDLEN] = 0;
+    strncpyz(bot2, argv[2], sizeof bot2);
     i = nextbot(bot);
     if (i < 0)
       x = 0;
     else
       botnet_send_link(i, botnetnick, bot, bot2);
-  }
-  sprintf(bot, "%d", x);
+  } else
+     x = botlink("", -2, bot);
+  egg_snprintf(bot, sizeof bot, "%d", x);
   Tcl_AppendResult(irp, bot, NULL);
   return TCL_OK;
 }
@@ -806,21 +785,19 @@ static int tcl_unlink STDVAR
   int i, x;
   char bot[HANDLEN + 1];
 
-  context;
   BADARGS(2, 3, " bot ?comment?");
-  strncpy(bot, argv[1], HANDLEN);
-  bot[HANDLEN] = 0;
+  strncpyz(bot, argv[1], sizeof bot);
   i = nextbot(bot);
   if (i < 0)
      x = 0;
   else {
     x = 1;
-    if (!strcasecmp(bot, dcc[i].nick))
+    if (!egg_strcasecmp(bot, dcc[i].nick))
       x = botunlink(-2, bot, argv[2]);
     else
       botnet_send_unlink(i, botnetnick, lastbot(bot), bot, argv[2]);
   }
-  sprintf(bot, "%d", x);
+  egg_snprintf(bot, sizeof bot, "%d", x);
   Tcl_AppendResult(irp, bot, NULL);
   return TCL_OK;
 }
@@ -830,47 +807,50 @@ static int tcl_connect STDVAR
   int i, z, sock;
   char s[81];
 
-  context;
   BADARGS(3, 3, " hostname port");
   if (dcc_total == max_dcc) {
     Tcl_AppendResult(irp, "out of dcc table space", NULL);
     return TCL_ERROR;
   }
-  i = dcc_total;
   sock = getsock(0);
+  if (sock < 0) {
+    Tcl_AppendResult(irp, MISC_NOFREESOCK, NULL);
+    return TCL_ERROR;
+  }
   z = open_telnet_raw(sock, argv[1], atoi(argv[2]));
   if (z < 0) {
+    killsock(sock);
     if (z == (-2))
-      strcpy(s, "DNS lookup failed");
+      strncpyz(s, "DNS lookup failed", sizeof s);
     else
       neterror(s);
     Tcl_AppendResult(irp, s, NULL);
     return TCL_ERROR;
   }
-  /* well well well... it worked! */
+  /* Well well well... it worked! */
   i = new_dcc(&DCC_SOCKET, 0);
   dcc[i].sock = sock;
   dcc[i].port = atoi(argv[2]);
   strcpy(dcc[i].nick, "*");
-  strncpy(dcc[i].host, argv[1], UHOSTLEN);
-  dcc[i].host[UHOSTLEN] = 0;
-  sprintf(s, "%d", sock);
+  strncpyz(dcc[i].host, argv[1], UHOSTMAX);
+  egg_snprintf(s, sizeof s, "%d", sock);
   Tcl_AppendResult(irp, s, NULL);
   return TCL_OK;
 }
 
-/* create a new listening port (or destroy one) */
-/* listen <port> bots/all/users [mask]
- * listen <port> script <proc>
- * listen <port> off */
+/* Create a new listening port (or destroy one)
+ *
+ * listen <port> bots/all/users [mask]
+ * listen <port> script <proc> [flag]
+ * listen <port> off
+ */
 static int tcl_listen STDVAR
 {
   int i, j, idx = (-1), port, realport;
-  char s[10];
+  char s[11];
   struct portmap *pmap = NULL, *pold = NULL;
 
-  context;
-  BADARGS(3, 4, " port type ?mask/proc?");
+  BADARGS(3, 5, " port type ?mask?/?proc ?flag??");
   port = realport = atoi(argv[1]);
   for (pmap = root; pmap; pold = pmap, pmap = pmap->next)
     if (pmap->realport == port) {
@@ -880,7 +860,7 @@ static int tcl_listen STDVAR
   for (i = 0; i < dcc_total; i++)
     if ((dcc[i].type == &DCC_TELNET) && (dcc[i].port == port))
       idx = i;
-  if (!strcasecmp(argv[2], "off")) {
+  if (!egg_strcasecmp(argv[2], "off")) {
     if (pmap) {
       if (pold)
 	pold->next = pmap->next;
@@ -888,7 +868,7 @@ static int tcl_listen STDVAR
 	root = pmap->next;
       nfree(pmap);
     }
-    /* remove */
+    /* Remove */
     if (idx < 0) {
       Tcl_AppendResult(irp, "no such listen port is open", NULL);
       return TCL_ERROR;
@@ -898,49 +878,64 @@ static int tcl_listen STDVAR
     return TCL_OK;
   }
   if (idx < 0) {
-    /* make new one */
+    /* Make new one */
     if (dcc_total >= max_dcc) {
       Tcl_AppendResult(irp, "no more DCC slots available", NULL);
       return TCL_ERROR;
     }
-    /* try to grab port */
+    /* Try to grab port */
     j = port + 20;
     i = (-1);
-    while ((port < j) && (i < 0)) {
+    while (port < j && i < 0) {
       i = open_listen(&port);
-      if (i < 0)
+      if (i == -1)
 	port++;
+      else if (i == -2)
+        break;
     }
-    if (i < 0) {
-      Tcl_AppendResult(irp, "couldn't grab nearby port", NULL);
+    if (i == -1) {
+      Tcl_AppendResult(irp, "Couldn't grab nearby port", NULL);
+      return TCL_ERROR;
+    } else if (i == -2) {
+      Tcl_AppendResult(irp, "Couldn't assign the requested IP. Please make sure 'my-ip' is set properly.", NULL);
       return TCL_ERROR;
     }
     idx = new_dcc(&DCC_TELNET, 0);
     dcc[idx].addr = iptolong(getmyip());
     dcc[idx].port = port;
     dcc[idx].sock = i;
+    dcc[idx].timeval = now;
   }
   /* script? */
-  if (!strcasecmp(argv[2], "script")) {
+  if (!strcmp(argv[2], "script")) {
     strcpy(dcc[idx].nick, "(script)");
     if (argc < 4) {
       Tcl_AppendResult(irp, "must give proc name for script listen", NULL);
       killsock(dcc[idx].sock);
-      dcc_total--;
+      lostdcc(idx);
       return TCL_ERROR;
     }
-    strncpy(dcc[idx].host, argv[3], UHOSTLEN - 1);
-    dcc[idx].host[UHOSTLEN - 1] = 0;
-    sprintf(s, "%d", port);
+    if (argc == 5) {
+      if (strcmp(argv[4], "pub")) {
+	Tcl_AppendResult(irp, "unknown flag: ", argv[4], ". allowed flags: pub",
+		         NULL);
+	killsock(dcc[idx].sock);
+	lostdcc(idx);
+	return TCL_ERROR;
+      }
+      dcc[idx].status = LSTN_PUBLIC;
+    }
+    strncpyz(dcc[idx].host, argv[3], UHOSTMAX);
+    egg_snprintf(s, sizeof s, "%d", port);
     Tcl_AppendResult(irp, s, NULL);
     return TCL_OK;
   }
   /* bots/users/all */
-  if (!strcasecmp(argv[2], "bots"))
+  if (!strcmp(argv[2], "bots"))
     strcpy(dcc[idx].nick, "(bots)");
-  else if (!strcasecmp(argv[2], "users"))
+  else if (!strcmp(argv[2], "users"))
     strcpy(dcc[idx].nick, "(users)");
-  else if (!strcasecmp(argv[2], "all"))
+  else if (!strcmp(argv[2], "all"))
     strcpy(dcc[idx].nick, "(telnet)");
   if (!dcc[idx].nick[0]) {
     Tcl_AppendResult(irp, "illegal listen type: must be one of ",
@@ -950,15 +945,13 @@ static int tcl_listen STDVAR
     return TCL_ERROR;
   }
   if (argc == 4) {
-    strncpy(dcc[idx].host, argv[3], UHOSTLEN - 1);
-    dcc[idx].host[UHOSTLEN - 1] = 0;
+    strncpyz(dcc[idx].host, argv[3], UHOSTMAX);
   } else
     strcpy(dcc[idx].host, "*");
-  sprintf(s, "%d", port);
+  egg_snprintf(s, sizeof s, "%d", port);
   Tcl_AppendResult(irp, s, NULL);
   if (!pmap) {
     pmap = nmalloc(sizeof(struct portmap));
-
     pmap->next = root;
     root = pmap;
   }
@@ -970,30 +963,31 @@ static int tcl_listen STDVAR
 
 static int tcl_boot STDVAR
 {
-  char who[512];
+  char who[NOTENAMELEN + 1];
   int i, ok = 0;
 
-  context;
   BADARGS(2, 3, " user@bot ?reason?");
-  strcpy(who, argv[1]);
+  strncpyz(who, argv[1], sizeof who);
+
   if (strchr(who, '@') != NULL) {
-    char whonick[161];
-     splitc(whonick, who, '@');
-     whonick[161] = 0;
-    if (!strcasecmp(who, botnetnick))
-       strcpy(who, whonick);
-    else if (remote_boots > 1) {
+    char whonick[HANDLEN + 1];
+
+    splitc(whonick, who, '@');
+    whonick[HANDLEN] = 0;
+    if (!egg_strcasecmp(who, botnetnick))
+       strncpyz(who, whonick, sizeof who);
+    else if (remote_boots > 0) {
       i = nextbot(who);
       if (i < 0)
 	return TCL_OK;
-      botnet_send_reject(i, botnetnick, NULL, whonick, who, argv[2]);
+      botnet_send_reject(i, botnetnick, NULL, whonick, who, argv[2] ? argv[2] : "");
     } else {
       return TCL_OK;
     }
   }
   for (i = 0; i < dcc_total; i++)
-    if ((!strcasecmp(dcc[i].nick, who)) && !ok &&
-	(dcc[i].type->flags & DCT_CANBOOT)) {
+    if (!ok && (dcc[i].type->flags & DCT_CANBOOT) &&
+        !egg_strcasecmp(dcc[i].nick, who)) {
       do_boot(i, botnetnick, argv[2] ? argv[2] : "");
       ok = 1;
     }
@@ -1002,7 +996,6 @@ static int tcl_boot STDVAR
 
 static int tcl_rehash STDVAR
 {
-  context;
   BADARGS(1, 1, " ");
   if (make_userfile) {
     putlog(LOG_MISC, "*", USERF_NONEEDNEW);
@@ -1016,7 +1009,6 @@ static int tcl_rehash STDVAR
 
 static int tcl_restart STDVAR
 {
-  context;
   BADARGS(1, 1, " ");
   if (!backgrd) {
     Tcl_AppendResult(interp, "You can't restart a -n bot", NULL);
@@ -1034,42 +1026,89 @@ static int tcl_restart STDVAR
   return TCL_OK;
 }
 
+static int tcl_traffic STDVAR
+{
+  char buf[1024];
+  unsigned long out_total_today, out_total;
+  unsigned long in_total_today, in_total;
+
+  /* IRC traffic */
+  sprintf(buf, "irc %ld %ld %ld %ld", itraffic_irc_today, itraffic_irc +
+	  itraffic_irc_today, otraffic_irc_today, otraffic_irc + otraffic_irc_today);
+  Tcl_AppendElement(irp, buf);  
+
+  /* Botnet traffic */
+  sprintf(buf, "botnet %ld %ld %ld %ld", itraffic_bn_today, itraffic_bn +
+          itraffic_bn_today, otraffic_bn_today, otraffic_bn + otraffic_bn_today);
+  Tcl_AppendElement(irp, buf);
+
+  /* Partyline */
+  sprintf(buf, "partyline %ld %ld %ld %ld", itraffic_dcc_today, itraffic_dcc +  
+          itraffic_dcc_today, otraffic_dcc_today, otraffic_dcc + otraffic_dcc_today);    
+  Tcl_AppendElement(irp, buf);
+
+  /* Transfer */
+  sprintf(buf, "transfer %ld %ld %ld %ld", itraffic_trans_today, itraffic_trans +  
+          itraffic_trans_today, otraffic_trans_today, otraffic_trans + otraffic_trans_today);    
+  Tcl_AppendElement(irp, buf);
+
+  /* Misc traffic */
+  sprintf(buf, "misc %ld %ld %ld %ld", itraffic_unknown_today, itraffic_unknown +  
+          itraffic_unknown_today, otraffic_unknown_today, otraffic_unknown + 
+	  otraffic_unknown_today);    
+  Tcl_AppendElement(irp, buf);
+
+  /* Totals */
+  in_total_today = itraffic_irc_today + itraffic_bn_today + itraffic_dcc_today + 
+		itraffic_trans_today + itraffic_unknown_today,
+  in_total = in_total_today + itraffic_irc + itraffic_bn + itraffic_dcc + 
+	      itraffic_trans + itraffic_unknown;
+  out_total_today = otraffic_irc_today + otraffic_bn_today + otraffic_dcc_today +
+                itraffic_trans_today + otraffic_unknown_today,
+  out_total = out_total_today + otraffic_irc + otraffic_bn + otraffic_dcc +
+              otraffic_trans + otraffic_unknown;	  
+  sprintf(buf, "total %ld %ld %ld %ld", in_total_today, in_total, out_total_today, out_total);
+  Tcl_AppendElement(irp, buf);
+  return(TCL_OK);
+}
+
 tcl_cmds tcldcc_cmds[] =
 {
-  {"putdcc", tcl_putdcc},
-  {"putdccraw", tcl_putdccraw},
-  {"putidx", tcl_putdcc},
-  {"dccsimul", tcl_dccsimul},
-  {"dccbroadcast", tcl_dccbroadcast},
-  {"hand2idx", tcl_hand2idx},
-  {"getchan", tcl_getchan},
-  {"setchan", tcl_setchan},
-  {"dccputchan", tcl_dccputchan},
-  {"console", tcl_console},
-  {"strip", tcl_strip},
-  {"echo", tcl_echo},
-  {"page", tcl_page},
-  {"control", tcl_control},
-  {"valididx", tcl_valididx},
-  {"killdcc", tcl_killdcc},
-  {"putbot", tcl_putbot},
-  {"putallbots", tcl_putallbots},
-  {"idx2hand", tcl_idx2hand},
-  {"bots", tcl_bots},
-  {"botlist", tcl_botlist},
-  {"dcclist", tcl_dcclist},
-  {"whom", tcl_whom},
-  {"dccused", tcl_dccused},
-  {"getdccidle", tcl_getdccidle},
-  {"getdccaway", tcl_getdccaway},
-  {"setdccaway", tcl_setdccaway},
-  {"islinked", tcl_islinked},
-  {"link", tcl_link},
-  {"unlink", tcl_unlink},
-  {"connect", tcl_connect},
-  {"listen", tcl_listen},
-  {"boot", tcl_boot},
-  {"rehash", tcl_rehash},
-  {"restart", tcl_restart},
-  {0, 0}
+  {"putdcc",		tcl_putdcc},
+  {"putdccraw",		tcl_putdccraw},
+  {"putidx",		tcl_putdcc},
+  {"dccsimul",		tcl_dccsimul},
+  {"dccbroadcast",	tcl_dccbroadcast},
+  {"hand2idx",		tcl_hand2idx},
+  {"getchan",		tcl_getchan},
+  {"setchan",		tcl_setchan},
+  {"dccputchan",	tcl_dccputchan},
+  {"console",		tcl_console},
+  {"strip",		tcl_strip},
+  {"echo",		tcl_echo},
+  {"page",		tcl_page},
+  {"control",		tcl_control},
+  {"valididx",		tcl_valididx},
+  {"killdcc",		tcl_killdcc},
+  {"putbot",		tcl_putbot},
+  {"putallbots",	tcl_putallbots},
+  {"idx2hand",		tcl_idx2hand},
+  {"bots",		tcl_bots},
+  {"botlist",		tcl_botlist},
+  {"dcclist",		tcl_dcclist},
+  {"whom",		tcl_whom},
+  {"dccused",		tcl_dccused},
+  {"getdccidle",	tcl_getdccidle},
+  {"getdccaway",	tcl_getdccaway},
+  {"setdccaway",	tcl_setdccaway},
+  {"islinked",		tcl_islinked},
+  {"link",		tcl_link},
+  {"unlink",		tcl_unlink},
+  {"connect",		tcl_connect},
+  {"listen",		tcl_listen},
+  {"boot",		tcl_boot},
+  {"rehash",		tcl_rehash},
+  {"restart",		tcl_restart},
+  {"traffic",		tcl_traffic},
+  {NULL,		NULL}
 };
