@@ -1,27 +1,42 @@
 /*
  * userent.c -- handles:
- * user-entry handling, new stylem more versatile.
+ *   user-entry handling, new stylem more versatile.
+ *
+ * $Id: userent.c,v 1.23 2002/01/02 03:46:36 guppy Exp $
  */
 /*
- * This file is part of the eggdrop source code
- * copyright (c) 1997 Robey Pointer
- * and is distributed according to the GNU general public license.
- * For full details, read the top of 'main.c' or the file called
- * COPYING that was distributed with this code.
+ * Copyright (C) 1997 Robey Pointer
+ * Copyright (C) 1999, 2000, 2001, 2002 Eggheads Development Team
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
+
 #include "main.h"
 #include "users.h"
 
-extern int noshare;
-extern struct userrec *userlist;
-extern struct dcc_t *dcc;
-extern Tcl_Interp *interp;
-extern char whois_fields[];
+extern int		 noshare;
+extern struct userrec	*userlist;
+extern struct dcc_t	*dcc;
+extern Tcl_Interp	*interp;
+extern char		 whois_fields[];
 
-int share_greet = 0;		/* share greeting info */
 
+int share_greet = 0;		/* Share greeting info			*/
 static struct user_entry_type *entry_type_list;
 
+
 void init_userent()
 {
   entry_type_list = 0;
@@ -60,35 +75,28 @@ int list_type_expmem(struct list_type *t)
 
 int def_unpack(struct userrec *u, struct user_entry *e)
 {
-  context;
-  if (e->name) {
-    char *p;
+  char *tmp;
 
-    p = e->u.list->extra;
-    e->u.list->extra = NULL;
-    list_type_kill(e->u.list);
-    e->u.string = p;
-  }
+  tmp = e->u.list->extra;
+  e->u.list->extra = NULL;
+  list_type_kill(e->u.list);
+  e->u.string = tmp;
   return 1;
 }
 
 int def_pack(struct userrec *u, struct user_entry *e)
 {
-  if (!e->name) {
-    char *p;
+  char *tmp;
 
-    p = e->u.string;
-    e->u.list = user_malloc(sizeof(struct list_type));
-
-    e->u.list->next = NULL;
-    e->u.list->extra = p;
-  }
+  tmp = e->u.string;
+  e->u.list = user_malloc(sizeof(struct list_type));
+  e->u.list->next = NULL;
+  e->u.list->extra = tmp;
   return 1;
 }
 
 int def_kill(struct user_entry *e)
 {
-  context;
   nfree(e->u.string);
   nfree(e);
   return 1;
@@ -96,7 +104,6 @@ int def_kill(struct user_entry *e)
 
 int def_write_userfile(FILE * f, struct userrec *u, struct user_entry *e)
 {
-  context;
   if (fprintf(f, "--%s %s\n", e->type->name, e->u.string) == EOF)
     return 0;
   return 1;
@@ -109,33 +116,39 @@ void *def_get(struct userrec *u, struct user_entry *e)
 
 int def_set(struct userrec *u, struct user_entry *e, void *buf)
 {
-  contextnote("drummer's bug?");
-  if (buf)
-    if (!((char *) buf)[0])
-      buf = 0;
-  if (e->u.string)
+  char *string = (char *) buf;
+
+  if (string && !string[0])
+    string = NULL;
+  if (!string && !e->u.string)
+    return 1;
+  if (string) {
+    int l = strlen (string);
+    char *i;
+
+    if (l > 160)
+      l = 160;
+
+
+    e->u.string = user_realloc (e->u.string, l + 1);
+
+    strncpyz (e->u.string, string, l + 1);
+
+    for (i = e->u.string; *i; i++)
+      /* Allow bold, inverse, underline, color text here...
+       * But never add cr or lf!! --rtc
+       */
+     if ((unsigned int) *i < 32 && !strchr ("\002\003\026\037", *i))
+        *i = '?';
+  } else { /* string == NULL && e->u.string != NULL */
     nfree(e->u.string);
-  context;
-  if (buf) {
-    unsigned char *p = (unsigned char *) buf, *q = (unsigned char *) buf;
-    while (*p && (q - (unsigned char *) buf < 160)) {
-      if (*p < 32)
-	p++;
-      else
-	*q++ = *p++;
-    }
-    *q = 0;
-    e->u.string = user_malloc(((q - (unsigned char *) buf) + 1));
-    strncpy(e->u.string, (char *) buf, (q - (unsigned char *) buf));
-    (e->u.string)[(q - (unsigned char *) buf)] = 0;
-  } else
     e->u.string = NULL;
-  if (!noshare && (!u || !(u->flags & (USER_BOT | USER_UNSHARED)))) {
-    if ((e->type == &USERENTRY_INFO) && !share_greet)
-      return 1;
-    shareout(NULL, "c %s %s %s\n", e->type->name, u->handle, buf ? buf : "");
   }
-  context;
+  if (!noshare && !(u->flags & (USER_BOT | USER_UNSHARED))) {
+    if (e->type != &USERENTRY_INFO || share_greet)
+      shareout(NULL, "c %s %s %s\n", e->type->name, u->handle,
+	       e->u.string ? e->u.string : "");
+  }
   return 1;
 }
 
@@ -164,13 +177,11 @@ int def_tcl_set(Tcl_Interp * irp, struct userrec *u,
 
 int def_expmem(struct user_entry *e)
 {
-  context;
   return strlen(e->u.string) + 1;
 }
 
 void def_display(int idx, struct user_entry *e)
 {
-  context;
   dprintf(idx, "  %s: %s\n", e->type->name, e->u.string);
 }
 
@@ -182,7 +193,6 @@ int def_dupuser(struct userrec *new, struct userrec *old,
 
 static void comment_display(int idx, struct user_entry *e)
 {
-  context;
   if (dcc[idx].user && (dcc[idx].user->flags & USER_MASTER))
     dprintf(idx, "  COMMENT: %.70s\n", e->u.string);
 }
@@ -258,7 +268,7 @@ static int pass_tcl_set(Tcl_Interp * irp, struct userrec *u,
 			struct user_entry *e, int argc, char **argv)
 {
   BADARGS(3, 4, " handle PASS ?newpass?");
-  pass_set(u, e, argv[3]);
+  pass_set(u, e, argc == 3 ? NULL : argv[3]);
   return TCL_OK;
 }
 
@@ -282,49 +292,36 @@ struct user_entry_type USERENTRY_PASS =
 
 static int laston_unpack(struct userrec *u, struct user_entry *e)
 {
-  context;
-  if (e->name) {
-    char *p, *q;
-    struct laston_info *li;
+  char *par, *arg;
+  struct laston_info *li;
 
-    p = e->u.list->extra;
-    e->u.list->extra = NULL;
-    list_type_kill(e->u.list);
-    q = strchr(p, ' ');
-    if (q) {
-      q++;
-    } else {
-      q = "???";
-    }
-    li = user_malloc(sizeof(struct laston_info));
-
-    li->lastonplace = user_malloc(strlen(q) + 1);
-    li->laston = atoi(p);
-    strcpy(li->lastonplace, q);
-    e->u.extra = li;
-    nfree(p);
-  }
+  par = e->u.list->extra;
+  arg = newsplit (&par);
+  if (!par[0])
+    par = "???";
+  li = user_malloc(sizeof(struct laston_info));
+  li->lastonplace = user_malloc(strlen(par) + 1);
+  li->laston = atoi(arg);
+  strcpy(li->lastonplace, par);
+  list_type_kill(e->u.list);
+  e->u.extra = li;
   return 1;
 }
 
 static int laston_pack(struct userrec *u, struct user_entry *e)
 {
-  if (!e->name) {
-    char number[1024];
-    struct list_type *l;
-    struct laston_info *li = (struct laston_info *) e->u.extra;
-    int r;
+  char work[1024];
+  struct laston_info *li;
+  int l;
 
-    r = sprintf(number, "%lu %s", li->laston, li->lastonplace);
-    l = user_malloc(sizeof(struct list_type));
-
-    l->extra = user_malloc(r + 1);
-    strcpy(l->extra, number);
-    l->next = 0;
-    e->u.extra = l;
-    nfree(li->lastonplace);
-    nfree(li);
-  }
+  li = (struct laston_info *) e->u.extra;
+  l = sprintf(work, "%lu %s", li->laston, li->lastonplace);
+  e->u.list = user_malloc(sizeof(struct list_type));
+  e->u.list->next = NULL;
+  e->u.list->extra = user_malloc(l + 1);
+  strcpy(e->u.list->extra, work);
+  nfree(li->lastonplace);
+  nfree(li);
   return 1;
 }
 
@@ -334,7 +331,6 @@ static int laston_write_userfile(FILE * f,
 {
   struct laston_info *li = (struct laston_info *) e->u.extra;
 
-  context;
   if (fprintf(f, "--LASTON %lu %s\n", li->laston,
 	      li->lastonplace ? li->lastonplace : "") == EOF)
     return 0;
@@ -343,7 +339,6 @@ static int laston_write_userfile(FILE * f,
 
 static int laston_kill(struct user_entry *e)
 {
-  context;
   if (((struct laston_info *) (e->u.extra))->lastonplace)
     nfree(((struct laston_info *) (e->u.extra))->lastonplace);
   nfree(e->u.extra);
@@ -353,25 +348,24 @@ static int laston_kill(struct user_entry *e)
 
 static int laston_set(struct userrec *u, struct user_entry *e, void *buf)
 {
-  context;
-  if ((e->u.list != buf) && e->u.list) {
-    if (((struct laston_info *) (e->u.extra))->lastonplace)
-      nfree(((struct laston_info *) (e->u.extra))->lastonplace);
-    nfree(e->u.extra);
+  struct laston_info *li = (struct laston_info *) e->u.extra;
+
+  if (li != buf) {
+    if (li) {
+      nfree(li->lastonplace);
+      nfree(li);
+    }
+
+    li = e->u.extra = buf;
   }
-  contextnote("SEGV with sharing bug track");
-  if (buf) {
-    e->u.list = (struct list_type *) buf;
-  } else
-    e->u.list = NULL;
-  /* donut share laston unfo */
+  /* donut share laston info */
   return 1;
 }
 
 static int laston_tcl_get(Tcl_Interp * irp, struct userrec *u,
 			  struct user_entry *e, int argc, char **argv)
 {
-  struct laston_info *li = (struct laston_info *) e->u.list;
+  struct laston_info *li = (struct laston_info *) e->u.extra;
   char number[20];
   struct chanuserrec *cr;
 
@@ -397,27 +391,35 @@ static int laston_tcl_set(Tcl_Interp * irp, struct userrec *u,
   struct laston_info *li;
   struct chanuserrec *cr;
 
-  BADARGS(5, 6, " handle LASTON time ?place?");
-  if (argc == 5) {
-    li = user_malloc(sizeof(struct laston_info));
+  BADARGS(4, 5, " handle LASTON time ?place?");
 
-    li->lastonplace = user_malloc(strlen(argv[4]) + 1);
-    li->laston = atoi(argv[3]);
-    strcpy(li->lastonplace, argv[4]);
-    set_user(&USERENTRY_LASTON, u, li);
-  } else {
+  if ((argc == 5) && argv[4][0] && strchr(CHANMETA, argv[4][0])) {
+    /* Search for matching channel */
     for (cr = u->chanrec; cr; cr = cr->next)
-      if (!rfc_casecmp(cr->channel, argv[4]))
+      if (!rfc_casecmp(cr->channel, argv[4])) {
 	cr->laston = atoi(argv[3]);
+	break;
+      }
   }
+  /* Save globally */
+  li = user_malloc(sizeof(struct laston_info));
+
+  if (argc == 5) {
+    li->lastonplace = user_malloc(strlen(argv[4]) + 1);
+    strcpy(li->lastonplace, argv[4]);
+  } else {
+    li->lastonplace = user_malloc(1);
+    li->lastonplace[0] = 0;
+  }
+  li->laston = atoi(argv[3]);
+  set_user(&USERENTRY_LASTON, u, li);
   return TCL_OK;
 }
 
 static int laston_expmem(struct user_entry *e)
 {
-  context;
   return sizeof(struct laston_info) +
-   strlen(((struct laston_info *) (e->u.list))->lastonplace) + 1;
+    strlen(((struct laston_info *) (e->u.extra))->lastonplace) + 1;
 }
 
 static int laston_dupuser(struct userrec *new, struct userrec *old,
@@ -456,74 +458,64 @@ struct user_entry_type USERENTRY_LASTON =
 
 static int botaddr_unpack(struct userrec *u, struct user_entry *e)
 {
-  context;
-  if (e->name) {
-    char *p, *q;
-    struct bot_addr *bi = user_malloc(sizeof(struct bot_addr));
+  char *p, *q;
+  struct bot_addr *bi = user_malloc(sizeof(struct bot_addr));
 
-    p = e->u.list->extra;
-    e->u.list->extra = NULL;
-    list_type_kill(e->u.list);
-    q = strchr(p, ':');
-    if (!q) {
-      bi->address = p;
-      bi->telnet_port = 3333;
-      bi->relay_port = 3333;
-    } else {
-      bi->address = user_malloc((q - p) + 1);
-      strncpy(bi->address, p, q - p);
-      bi->address[q - p] = 0;
-      q++;
-      bi->telnet_port = atoi(q);
-      q = strchr(q, '/');
-      if (!q) {
-	bi->relay_port = bi->telnet_port;
-      } else {
-	bi->relay_port = atoi(q + 1);
-      }
-    }
-    e->u.extra = bi;
-    nfree(p);
+  egg_bzero(bi, sizeof(struct bot_addr));
+
+  if (!(q = strchr ((p = e->u.list->extra), ':'))) {
+    bi->address = user_malloc(strlen (p) + 1);
+    strcpy (bi->address, p);
+  } else {
+    bi->address = user_malloc((q - p) + 1);
+    strncpy(bi->address, p, q - p);
+    bi->address[q - p] = 0;
+    q++;
+    bi->telnet_port = atoi(q);
+    if ((q = strchr(q, '/')))
+      bi->relay_port = atoi(q + 1);
   }
+  if (!bi->telnet_port)
+    bi->telnet_port = 3333;
+  if (!bi->relay_port)
+    bi->relay_port = bi->telnet_port;
+  list_type_kill(e->u.list);
+  e->u.extra = bi;
   return 1;
 }
 
 static int botaddr_pack(struct userrec *u, struct user_entry *e)
 {
-  if (!e->name) {
-    char work[1024];
-    struct bot_addr *bi = (struct bot_addr *) e->u.extra;
-    int l;
+  char work[1024];
+  struct bot_addr *bi;
+  int l;
 
-    l = sprintf(work, "%s:%d/%d", bi->address, bi->telnet_port,
-		bi->relay_port);
-    e->u.list = user_malloc(sizeof(struct list_type));
-
-    e->u.list->extra = user_malloc(l + 1);
-    strcpy(e->u.list->extra, work);
-    e->u.list->next = NULL;
-    nfree(bi->address);
-    nfree(bi);
-  }
+  bi = (struct bot_addr *) e->u.extra;
+  l = simple_sprintf(work, "%s:%u/%u", bi->address, bi->telnet_port,
+              bi->relay_port);
+  e->u.list = user_malloc(sizeof(struct list_type));
+  e->u.list->next = NULL;
+  e->u.list->extra = user_malloc(l + 1);
+  strcpy(e->u.list->extra, work);
+  nfree(bi->address);
+  nfree(bi);
   return 1;
 }
 
 static int botaddr_kill(struct user_entry *e)
 {
-  context;
   nfree(((struct bot_addr *) (e->u.extra))->address);
   nfree(e->u.extra);
   nfree(e);
-  context;
   return 1;
 }
 
-static int botaddr_write_userfile(FILE * f, struct userrec *u, struct user_entry *e)
+static int botaddr_write_userfile(FILE *f, struct userrec *u,
+				  struct user_entry *e)
 {
   register struct bot_addr *bi = (struct bot_addr *) e->u.extra;
 
-  context;
-  if (fprintf(f, "--%s %s:%d/%d\n", e->type->name, bi->address,
+  if (fprintf(f, "--%s %s:%u/%u\n", e->type->name, bi->address,
 	      bi->telnet_port, bi->relay_port) == EOF)
     return 0;
   return 1;
@@ -531,34 +523,30 @@ static int botaddr_write_userfile(FILE * f, struct userrec *u, struct user_entry
 
 static int botaddr_set(struct userrec *u, struct user_entry *e, void *buf)
 {
-  context;
-  if ((e->u.list != buf) && e->u.list) {
-    if (((struct bot_addr *) (e->u.extra))->address)
-      nfree(((struct bot_addr *) (e->u.extra))->address);
-    nfree(e->u.extra);
-  }
-  contextnote("SEGV with sharing bug track");
-  if (buf) {
-    e->u.list = (struct list_type *) buf;
-    /* donut share laston unfo */
-    if (!noshare && !(u && (u->flags & USER_UNSHARED))) {
-      register struct bot_addr *bi = (struct bot_addr *) e->u.extra;
+  register struct bot_addr *bi = (struct bot_addr *) e->u.extra;
 
-      shareout(NULL, "c BOTADDR %s %s %d %d\n", u->handle,
-	       bi->address, bi->telnet_port, bi->relay_port);
+  if (!bi && !buf)
+    return 1;
+  if (bi != buf) {
+    if (bi) {
+      nfree(bi->address);
+      nfree(bi);
     }
-  } else
-    e->u.list = NULL;
+    bi = e->u.extra = buf;
+  }
+  if (bi && !noshare && !(u->flags & USER_UNSHARED)) {
+    shareout(NULL, "c BOTADDR %s %s %d %d\n", u->handle,
+             bi->address, bi->telnet_port, bi->relay_port);
+  }
   return 1;
 }
 
-static int botaddr_tcl_get(Tcl_Interp * interp, struct userrec *u,
+static int botaddr_tcl_get(Tcl_Interp *interp, struct userrec *u,
 			   struct user_entry *e, int argc, char **argv)
 {
   register struct bot_addr *bi = (struct bot_addr *) e->u.extra;
   char number[20];
 
-  context;
   sprintf(number, " %d", bi->telnet_port);
   Tcl_AppendResult(interp, bi->address, number, NULL);
   sprintf(number, " %d", bi->relay_port);
@@ -566,25 +554,30 @@ static int botaddr_tcl_get(Tcl_Interp * interp, struct userrec *u,
   return TCL_OK;
 }
 
-static int botaddr_tcl_set(Tcl_Interp * irp, struct userrec *u,
+static int botaddr_tcl_set(Tcl_Interp *irp, struct userrec *u,
 			   struct user_entry *e, int argc, char **argv)
 {
   register struct bot_addr *bi = (struct bot_addr *) e->u.extra;
 
   BADARGS(4, 6, " handle type address ?telnetport ?relayport??");
   if (u->flags & USER_BOT) {
-    /* silently ignore for users */
-    if (!bi)
+    /* Silently ignore for users */
+    if (!bi) {
       bi = user_malloc(sizeof(struct bot_addr));
-
-    else if (bi->address)
+      egg_bzero(bi, sizeof (struct bot_addr));
+    } else {
       nfree(bi->address);
+    }
     bi->address = user_malloc(strlen(argv[3]) + 1);
     strcpy(bi->address, argv[3]);
     if (argc > 4)
       bi->telnet_port = atoi(argv[4]);
     if (argc > 5)
       bi->relay_port = atoi(argv[5]);
+    if (!bi->telnet_port)
+      bi->telnet_port = 3333;
+    if (!bi->relay_port)
+      bi->relay_port = bi->telnet_port;
     botaddr_set(u, e, bi);
   }
   return TCL_OK;
@@ -594,7 +587,6 @@ static int botaddr_expmem(struct user_entry *e)
 {
   register struct bot_addr *bi = (struct bot_addr *) e->u.extra;
 
-  context;
   return strlen(bi->address) + 1 + sizeof(struct bot_addr);
 }
 
@@ -602,32 +594,27 @@ static void botaddr_display(int idx, struct user_entry *e)
 {
   register struct bot_addr *bi = (struct bot_addr *) e->u.extra;
 
-  context;
   dprintf(idx, "  ADDRESS: %.70s\n", bi->address);
-  dprintf(idx, "     telnet: %d, relay: %d\n", bi->telnet_port, bi->relay_port);
+  dprintf(idx, "     users: %d, bots: %d\n", bi->relay_port, bi->telnet_port);
 }
 
 static int botaddr_gotshare(struct userrec *u, struct user_entry *e,
 			    char *buf, int idx)
 {
   struct bot_addr *bi = user_malloc(sizeof(struct bot_addr));
-  int i;
-  char *key;
+  char *arg;
 
-  key = newsplit(&buf);
-  bi->address = user_malloc(strlen(key) + 1);
-  strcpy(bi->address, key);
-  key = newsplit(&buf);
-  i = atoi(key);
-  if (i)
-    bi->telnet_port = i;
-  else
+  egg_bzero(bi, sizeof(struct bot_addr));
+  arg = newsplit(&buf);
+  bi->address = user_malloc(strlen(arg) + 1);
+  strcpy(bi->address, arg);
+  arg = newsplit(&buf);
+  bi->telnet_port = atoi(arg);
+  bi->relay_port = atoi(buf);
+  if (!bi->telnet_port)
     bi->telnet_port = 3333;
-  i = atoi(buf);
-  if (i)
-    bi->relay_port = i;
-  else
-    bi->relay_port = 3333;
+  if (!bi->relay_port)
+    bi->relay_port = bi->telnet_port;
   if (!(dcc[idx].status & STAT_GETTING))
     putlog(LOG_CMDS, "*", "%s: change botaddr %s", dcc[idx].nick,
 	   u->handle);
@@ -671,112 +658,120 @@ struct user_entry_type USERENTRY_BOTADDR =
   "BOTADDR"
 };
 
-static int xtra_set(struct userrec *u, struct user_entry *e, void *buf)
+int xtra_set(struct userrec *u, struct user_entry *e, void *buf)
 {
-  struct xtra_key *x, *y = 0, *z = buf;
+  struct xtra_key *curr, *old = NULL, *new = buf;
 
-  for (x = e->u.extra; x; x = x->next) {
-    if (x->key && !strcasecmp(z->key, x->key)) {
-      y = x;
+  for (curr = e->u.extra; curr; curr = curr->next) {
+    if (curr->key && !egg_strcasecmp(curr->key, new->key)) {
+      old = curr;
       break;
     }
   }
-  if (y && (!z->data || (z != y))) {
-    nfree(y->key);
-    nfree(y->data);
-    list_delete((struct list_type **) (&e->u.extra),
-		(struct list_type *) y);
-    nfree(y);
+  if (!old && (!new->data || !new->data[0])) {
+    /* Delete non-existant entry -- doh ++rtc */
+    nfree(new->key);
+    if (new->data)
+      nfree(new->data);
+    nfree(new);
+    return TCL_OK;
   }
-  if (z->data && (z != y))
-    list_insert((&e->u.extra), z);
+
+  /* We will possibly free new below, so let's send the information
+   * to the botnet now
+   */
   if (!noshare && !(u->flags & (USER_BOT | USER_UNSHARED)))
-    shareout(NULL, "c XTRA %s %s %s\n", u->handle, z->key, z->data ? z->data : "");
+    shareout(NULL, "c XTRA %s %s %s\n", u->handle, new->key,
+	     new->data ? new->data : "");
+  if ((old && old != new) || !new->data || !new->data[0]) {
+    list_delete((struct list_type **) (&e->u.extra),
+		(struct list_type *) old);
+    nfree(old->key);
+    nfree(old->data);
+    nfree(old);
+  }
+  if (old != new && new->data) {
+    if (new->data[0])
+      list_insert((&e->u.extra), new) /* do not add a ';' here */
+  } else {
+    if (new->data)
+      nfree(new->data);
+    nfree(new->key);
+    nfree(new);
+  }
   return TCL_OK;
 }
 
 static int xtra_tcl_set(Tcl_Interp * irp, struct userrec *u,
 			struct user_entry *e, int argc, char **argv)
 {
+  struct xtra_key *xk;
+  int l;
+
   BADARGS(4, 5, " handle type key ?value?");
+  xk = user_malloc(sizeof(struct xtra_key));
+  l = strlen(argv[3]);
+  egg_bzero(xk, sizeof (struct xtra_key));
+  if (l > 500)
+    l = 500;
+  xk->key = user_malloc(l + 1);
+  strncpyz(xk->key, argv[3], l + 1);
 
-  if (argc == 4) {
-    struct xtra_key xk =
-    {
-      0, argv[3], 0
-    };
+  if (argc == 5) {
+    int k = strlen(argv[4]);
 
-    xtra_set(u, e, &xk);
-    return TCL_OK;
-  } else {
-    struct xtra_key *y = user_malloc(sizeof(struct xtra_key));
-    int l = strlen(argv[3]), k = strlen(argv[4]);
-
-    if (l > 500) {
-      l = 500;
-      k = 0;
-    }
-    y->key = user_malloc(l + 1);
-    strncpy(y->key, argv[3], l);
-    y->key[l] = 0;
     if (k > 500 - l)
       k = 500 - l;
-    y->data = user_malloc(k + 1);
-    strncpy(y->data, argv[4], k);
-    y->data[k] = 0;
-    xtra_set(u, e, y);
+    xk->data = user_malloc(k + 1);
+    strncpyz(xk->data, argv[4], k + 1);
   }
+  xtra_set(u, e, xk);
   return TCL_OK;
 }
 
 int xtra_unpack(struct userrec *u, struct user_entry *e)
 {
-  context;
-  if (e->name) {
-    struct list_type *x, *y;
-    struct xtra_key *t;
-    char *ptr, *thing;
+  struct list_type *curr, *head;
+  struct xtra_key *t;
+  char *key, *data;
 
-    y = e->u.extra;
-    e->u.list = NULL;
-    for (x = y; x; x = y) {
-      y = x->next;
-      t = user_malloc(sizeof(struct xtra_key));
+  head = curr = e->u.list;
+  e->u.extra = NULL;
+  while (curr) {
+    t = user_malloc(sizeof(struct xtra_key));
 
+    data = curr->extra;
+    key = newsplit(&data);
+    if (data[0]) {
+      t->key = user_malloc(strlen(key) + 1);
+      strcpy(t->key, key);
+      t->data = user_malloc(strlen(data) + 1);
+      strcpy(t->data, data);
       list_insert((&e->u.extra), t);
-      thing = x->extra;
-      ptr = newsplit(&thing);
-      t->key = user_malloc(strlen(ptr) + 1);
-      strcpy(t->key, ptr);
-      t->data = user_malloc(strlen(thing) + 1);
-      strcpy(t->data, thing);
-      nfree(x->extra);
-      nfree(x);
     }
+    curr = curr->next;
   }
+  list_type_kill(head);
   return 1;
 }
 
 static int xtra_pack(struct userrec *u, struct user_entry *e)
 {
-  context;
-  if (!e->name) {
-    struct list_type *t;
-    struct xtra_key *x, *y;
+  struct list_type *t;
+  struct xtra_key *curr, *next;
 
-    y = e->u.extra;
-    e->u.list = NULL;
-    for (x = y; x; x = y) {
-      y = x->next;
-      t = user_malloc(sizeof(struct list_type));
-
-      list_insert((&e->u.extra), t);
-      t->extra = user_malloc(strlen(x->key) + strlen(x->data) + 4);
-      sprintf(t->extra, "%s %s", x->key, x->data);
-      nfree(x->key);
-      nfree(x->data);
-      nfree(x);
-    }
+  curr = e->u.extra;
+  e->u.list = NULL;
+  while (curr) {
+    t = user_malloc(sizeof(struct list_type));
+    t->extra = user_malloc(strlen(curr->key) + strlen(curr->data) + 4);
+    sprintf(t->extra, "%s %s", curr->key, curr->data);
+    list_insert((&e->u.list), t);
+    next = curr->next;
+    nfree(curr->key);
+    nfree(curr->data);
+    nfree(curr);
+    curr = next;
   }
   return 1;
 }
@@ -787,63 +782,48 @@ static void xtra_display(int idx, struct user_entry *e)
   struct xtra_key *xk;
   char **list;
 
-  context;
   code = Tcl_SplitList(interp, whois_fields, &lc, &list);
   if (code == TCL_ERROR)
     return;
-  /* scan thru xtra field, searching for matches */
-  context;
+  /* Scan thru xtra field, searching for matches */
   for (xk = e->u.extra; xk; xk = xk->next) {
-    /* ok, it's a valid xtra field entry */
-    context;
+    /* Ok, it's a valid xtra field entry */
     for (j = 0; j < lc; j++) {
-      if (strcasecmp(list[j], xk->key) == 0)
+      if (!egg_strcasecmp(list[j], xk->key))
 	dprintf(idx, "  %s: %s\n", xk->key, xk->data);
     }
   }
-  n_free(list, "", 0);
-  context;
+  Tcl_Free((char *) list);
 }
 
 static int xtra_gotshare(struct userrec *u, struct user_entry *e,
 			 char *buf, int idx)
 {
-  char *x = strchr(buf, ' ');
-  struct xtra_key *xk, *y = 0;
+  char *arg;
+  struct xtra_key *xk;
+  int l;
 
-  if (x) {
-    int l = x - buf, k = strlen(++x);
+  arg = newsplit (&buf);
+  if (!arg[0])
+    return 1;
 
-    xk = user_malloc(sizeof(struct xtra_key));
+  xk = user_malloc (sizeof(struct xtra_key));
+  egg_bzero(xk, sizeof(struct xtra_key));
+  l = strlen(arg);
+  if (l > 500)
+    l = 500;
+  xk->key = user_malloc(l + 1);
+  strncpyz(xk->key, arg, l + 1);
+
+  if (buf[0]) {
+    int k = strlen(buf);
 
-    if (l > 500) {
-      l = 500;
-      k = 0;
-    }
-    xk->key = user_malloc(l + 1);
-    strncpy(xk->key, buf, l);
-    xk->key[l] = 0;
     if (k > 500 - l)
-      k = 500;
+      k = 500 - l;
     xk->data = user_malloc(k + 1);
-    strncpy(xk->data, x, k);
-    xk->data[k] = 0;
-    return xtra_set(u, e, xk);
-  } else {
-    for (xk = e->u.extra; xk; xk = xk->next) {
-      if (xk->key && !strcasecmp(buf, xk->key)) {
-	y = xk;
-	break;
-      }
-    }
-    if (y) {
-      nfree(y->key);
-      nfree(y->data);
-      list_delete((struct list_type **) (&e->u.extra),
-		  (struct list_type *) y);
-      nfree(y);
-    }
+    strncpyz(xk->data, buf, k + 1);
   }
+  xtra_set(u, e, xk);
   return 1;
 }
 
@@ -864,7 +844,7 @@ static int xtra_dupuser(struct userrec *new, struct userrec *old,
   return 1;
 }
 
-static int xtra_write_userfile(FILE * f, struct userrec *u, struct user_entry *e)
+static int xtra_write_userfile(FILE *f, struct userrec *u, struct user_entry *e)
 {
   struct xtra_key *x;
 
@@ -878,7 +858,6 @@ int xtra_kill(struct user_entry *e)
 {
   struct xtra_key *x, *y;
 
-  context;
   for (x = e->u.extra; x; x = y) {
     y = x->next;
     nfree(x->key);
@@ -886,11 +865,10 @@ int xtra_kill(struct user_entry *e)
     nfree(x);
   }
   nfree(e);
-  context;
   return 1;
 }
 
-static int xtra_tcl_get(Tcl_Interp * irp, struct userrec *u,
+static int xtra_tcl_get(Tcl_Interp *irp, struct userrec *u,
 			struct user_entry *e, int argc, char **argv)
 {
   struct xtra_key *x;
@@ -898,7 +876,7 @@ static int xtra_tcl_get(Tcl_Interp * irp, struct userrec *u,
   BADARGS(3, 4, " handle XTRA ?key?");
   if (argc == 4) {
     for (x = e->u.extra; x; x = x->next)
-      if (!strcasecmp(argv[3], x->key)) {
+      if (!egg_strcasecmp(argv[3], x->key)) {
 	Tcl_AppendResult(irp, x->data, NULL);
 	return TCL_OK;
       }
@@ -911,7 +889,7 @@ static int xtra_tcl_get(Tcl_Interp * irp, struct userrec *u,
     list[1] = x->data;
     p = Tcl_Merge(2, list);
     Tcl_AppendElement(irp, p);
-    n_free(p, "", 0);
+    Tcl_Free((char *) p);
   }
   return TCL_OK;
 }
@@ -963,7 +941,7 @@ static int hosts_null(struct userrec *u, struct user_entry *e)
   return 1;
 }
 
-static int hosts_write_userfile(FILE * f, struct userrec *u, struct user_entry *e)
+static int hosts_write_userfile(FILE *f, struct userrec *u, struct user_entry *e)
 {
   struct list_type *h;
 
@@ -996,11 +974,11 @@ static void hosts_display(int idx, struct user_entry *e)
     if (s[0] && !s[9])
       strcat(s, q->extra);
     else if (!s[0])
-      simple_sprintf(s, "         %s", q->extra);
+      sprintf(s, "         %s", q->extra);
     else {
       if (strlen(s) + strlen(q->extra) + 2 > 65) {
 	dprintf(idx, "%s\n", s);
-	simple_sprintf(s, "         %s", q->extra);
+	sprintf(s, "         %s", q->extra);
       } else {
 	strcat(s, ", ");
 	strcat(s, q->extra);
@@ -1013,24 +991,15 @@ static void hosts_display(int idx, struct user_entry *e)
 
 static int hosts_set(struct userrec *u, struct user_entry *e, void *buf)
 {
-  context;
-  if (!buf || !strcasecmp(buf, "none")) {
-    contextnote("SEGV with sharing bug track");
-    /* when the bot crashes, it's in this part, not in the 'else' part */
-    contextnote((e->u.list) ? "e->u.list is valid" : "e->u.list is NULL!")
-      contextnote(e ? "e is valid" : "e is NULL!")
-      if (e) {
-      list_type_kill(e->u.list);
-      contextnote("SEGV with sharing bug track - added 99/03/26");
-      e->u.list = NULL;
-      contextnote("SEGV with sharing bug track - added 99/03/26");
-    }
+  if (!buf || !egg_strcasecmp(buf, "none")) {
+    /* When the bot crashes, it's in this part, not in the 'else' part */
+    list_type_kill(e->u.list);
+    e->u.list = NULL;
   } else {
     char *host = buf, *p = strchr(host, ',');
     struct list_type **t;
 
-    contextnote("SEGV with sharing bug track");
-    /* can't have ,'s in hostmasks */
+    /* Can't have ,'s in hostmasks */
     while (p) {
       *p = '?';
       p = strchr(host, ',');
@@ -1057,11 +1026,10 @@ static int hosts_set(struct userrec *u, struct user_entry *e, void *buf)
     (*t)->extra = user_malloc(strlen(host) + 1);
     strcpy((*t)->extra, host);
   }
-  contextnote("please visit http://www.eggheads.org/bugs.html now.");
   return 1;
 }
 
-static int hosts_tcl_get(Tcl_Interp * irp, struct userrec *u,
+static int hosts_tcl_get(Tcl_Interp *irp, struct userrec *u,
 			 struct user_entry *e, int argc, char **argv)
 {
   struct list_type *x;
@@ -1078,10 +1046,7 @@ static int hosts_tcl_set(Tcl_Interp * irp, struct userrec *u,
   BADARGS(3, 4, " handle HOSTS ?host?");
   if (argc == 4)
     addhost_by_handle(u->handle, argv[3]);
-  else /* {
-     while (e->u.list && strcasecmp(e->u.list->extra, "none"))
-       delhost_by_handle(u->handle, e->u.list->extra);
-  } */
+  else
     addhost_by_handle(u->handle, "none"); /* drummer */
   return TCL_OK;
 }
@@ -1178,7 +1143,7 @@ struct user_entry_type *find_entry_type(char *name)
   struct user_entry_type *p;
 
   for (p = entry_type_list; p; p = p->next) {
-    if (!strcasecmp(name, p->name))
+    if (!egg_strcasecmp(name, p->name))
       return p;
   }
   return NULL;
@@ -1191,7 +1156,7 @@ struct user_entry *find_user_entry(struct user_entry_type *et,
 
   for (e = &(u->entries); *e; e = &((*e)->next)) {
     if (((*e)->type == et) ||
-	((*e)->name && !strcasecmp((*e)->name, et->name))) {
+	((*e)->name && !egg_strcasecmp((*e)->name, et->name))) {
       t = *e;
       *e = t->next;
       t->next = u->entries;
