@@ -1,63 +1,105 @@
-/* 
+/*
  * userrec.c -- handles:
- * add_q() del_q() str2flags() flags2str() str2chflags() chflags2str()
- * a bunch of functions to find and change user records
- * change and check user (and channel-specific) flags
- * 
- * dprintf'ized, 10nov1995
+ *   add_q() del_q() str2flags() flags2str() str2chflags() chflags2str()
+ *   a bunch of functions to find and change user records
+ *   change and check user (and channel-specific) flags
+ *
+ * $Id: userrec.c,v 1.37 2002/01/02 03:46:36 guppy Exp $
  */
 /*
- * This file is part of the eggdrop source code
- * copyright (c) 1997 Robey Pointer
- * and is distributed according to the GNU general public license.
- * For full details, read the top of 'main.c' or the file called
- * COPYING that was distributed with this code.
+ * Copyright (C) 1997 Robey Pointer
+ * Copyright (C) 1999, 2000, 2001, 2002 Eggheads Development Team
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#include "main.h"
 #include <sys/stat.h>
+#include "main.h"
 #include "users.h"
 #include "chan.h"
 #include "modules.h"
 #include "tandem.h"
 
-extern struct dcc_t *dcc;
-extern int dcc_total;
-extern char userfile[];
-extern int share_greet;
-extern struct chanset_t *chanset;
-extern char ver[];
-extern char botnetnick[];
-extern time_t now;
-extern int default_flags, default_uflags;
+extern struct dcc_t	*dcc;
+extern struct chanset_t	*chanset;
+extern int		 default_flags, default_uflags, quiet_save,
+			 dcc_total, share_greet;
+extern char		 userfile[], ver[], botnetnick[];
+extern time_t		 now;
 
-int noshare = 1;		/* don't send out to sharebots */
-int sort_users = 0;		/* sort the userlist when saving */
-struct userrec *userlist = NULL;	/* user records are stored here */
-struct userrec *lastuser = NULL;	/* last accessed user record */
-struct banrec *global_bans = NULL;
-struct igrec *global_ign = NULL;
-struct exemptrec *global_exempts = NULL;
-struct inviterec *global_invites = NULL;
-int cache_hit = 0, cache_miss = 0;	/* temporary cache accounting */
+int		 noshare = 1;		/* don't send out to sharebots	    */
+int		 sort_users = 0;	/* sort the userlist when saving    */
+struct userrec	*userlist = NULL;	/* user records are stored here	    */
+struct userrec	*lastuser = NULL;	/* last accessed user record	    */
+maskrec		*global_bans = NULL,
+		*global_exempts = NULL,
+		*global_invites = NULL;
+struct igrec	*global_ign = NULL;
+int		cache_hit = 0,
+		cache_miss = 0;		/* temporary cache accounting	    */
+int		strict_host = 0;
+int		userfile_perm = 0600;	/* Userfile permissions,
+					   default rw-------		    */
 
-/* FIXME: rename EBUG_MEM to DEBUG_MEM */
-#ifdef EBUG_MEM
-void *_user_malloc(int size, char *file, int line)
+
+void *_user_malloc(int size, const char *file, int line)
 {
-  char x[1024];
+#ifdef DEBUG_MEM
+  char		 x[1024];
+  const char	*p;
 
-  simple_sprintf(x, "userrec.c:%s", file);
+  p = strrchr(file, '/');
+  simple_sprintf(x, "userrec.c:%s", p ? p + 1 : file);
   return n_malloc(size, x, line);
-}
 #else
-void *_user_malloc(int size, char *file, int line)
-{
   return nmalloc(size);
-}
 #endif
+}
 
-/* memory we should be using */
+void *_user_realloc(void *ptr, int size, const char *file, int line)
+{
+#ifdef DEBUG_MEM
+  char		 x[1024];
+  const char	*p;
+
+  p = strrchr(file, '/');
+  simple_sprintf(x, "userrec.c:%s", p ? p + 1 : file);
+  return n_realloc(ptr, size, x, line);
+#else
+  return nrealloc(ptr, size);
+#endif
+}
+
+inline int expmem_mask(struct maskrec *m)
+{
+  int result = 0;
+
+  for (; m; m = m->next) {
+    result += sizeof(struct maskrec);
+    result += strlen(m->mask) + 1;
+    if (m->user)
+      result += strlen(m->user) + 1;
+    if (m->desc)
+      result += strlen(m->desc) + 1;
+  }
+
+  return result;
+}
+
+/* Memory we should be using
+ */
 int expmem_users()
 {
   int tot;
@@ -65,22 +107,15 @@ int expmem_users()
   struct chanuserrec *ch;
   struct chanset_t *chan;
   struct user_entry *ue;
-  struct banrec *b;
   struct igrec *i;
-  struct exemptrec * e;
-  struct inviterec * inv;
 
-  context;
   tot = 0;
-  u = userlist;
-  while (u != NULL) {
-    ch = u->chanrec;
-    while (ch) {
+  for (u = userlist; u; u = u->next) { 
+    for (ch = u->chanrec; ch; ch = ch->next) {
       tot += sizeof(struct chanuserrec);
 
       if (ch->info != NULL)
 	tot += strlen(ch->info) + 1;
-      ch = ch->next;
     }
     tot += sizeof(struct userrec);
 
@@ -94,64 +129,24 @@ int expmem_users()
 	tot += ue->type->expmem(ue);
       }
     }
-    u = u->next;
   }
-  /* account for each channel's ban-list user */
+  /* Account for each channel's masks */
   for (chan = chanset; chan; chan = chan->next) {
 
-    for (b = chan->bans; b; b = b->next) {
-      tot += sizeof(struct banrec);
+    /* Account for each channel's ban-list user */
+    tot += expmem_mask(chan->bans);
 
-      tot += strlen(b->banmask) + 1;
-      if (b->user)
-	tot += strlen(b->user) + 1;
-      if (b->desc)
-	tot += strlen(b->desc) + 1;
-    }
-    /* account for each channel's exempt-list user */
-    for (e = chan->exempts;e;e=e->next) {
-      tot += sizeof(struct exemptrec);
-      tot += strlen(e->exemptmask)+1;
-      if (e->user)
-	tot += strlen(e->user)+1;
-      if (e->desc)
-	tot += strlen(e->desc)+1;
-    }
-    /* account for each channel's invite-list user */
-    for (inv = chan->invites;inv;inv=inv->next) {
-      tot += sizeof(struct inviterec);  
-      tot += strlen(inv->invitemask)+1;
-      if (inv->user)
-	tot += strlen(inv->user)+1;
-      if (inv->desc)
-	tot += strlen(inv->desc)+1;
-    }
-  }
-  for (b = global_bans; b; b = b->next) {
-    tot += sizeof(struct banrec);
+    /* Account for each channel's exempt-list user */
+    tot += expmem_mask(chan->exempts);
 
-    tot += strlen(b->banmask) + 1;
-    if (b->user)
-      tot += strlen(b->user) + 1;
-    if (b->desc)
-      tot += strlen(b->desc) + 1;
-  }
-  for (e = global_exempts;e;e=e->next) {
-    tot += sizeof(struct exemptrec);
-    tot += strlen(e->exemptmask)+1;
-    if (e->user)
-      tot += strlen(e->user)+1;
-    if (e->desc)
-      tot += strlen(e->desc)+1;
-  }
-  for (inv = global_invites;inv;inv=inv->next) {
-    tot += sizeof(struct inviterec);
-    tot += strlen(inv->invitemask)+1;
-    if (inv->user)
-      tot += strlen(inv->user)+1;
-    if (inv->desc)
-      tot += strlen(inv->desc)+1;  
+    /* Account for each channel's invite-list user */
+    tot += expmem_mask(chan->invites);
   }
+
+  tot += expmem_mask(global_bans);
+  tot += expmem_mask(global_exempts);
+  tot += expmem_mask(global_invites);
+
   for (i = global_ign; i; i = i->next) {
     tot += sizeof(struct igrec);
 
@@ -167,36 +162,60 @@ int expmem_users()
 int count_users(struct userrec *bu)
 {
   int tot = 0;
-  struct userrec *u = bu;
+  struct userrec *u ;
 
-  while (u != NULL) {
+  for (u = bu; u; u = u->next)
     tot++;
-    u = u->next;
-  }
   return tot;
 }
 
+/* Convert "nick!~user@host", "nick!+user@host" and "nick!-user@host"
+ * to "nick!user@host" if necessary. (drummer)
+ */
+char *fixfrom(char *s)
+{
+  char *p;
+  static char buf[512];
+
+  if (s == NULL)
+    return NULL;
+  strncpyz(buf, s, sizeof buf);
+  if (strict_host)
+    return buf;
+  if ((p = strchr(buf, '!')))
+    p++;
+  else
+    p = s;			/* Sometimes we get passed just a
+				 * user@host here... */
+  /* These are ludicrous. */
+  if (strchr("~+-^=", *p) && (p[1] != '@')) /* added check for @ - drummer */
+    strcpy(p, p + 1);
+  /* Bug was: n!~@host -> n!@host  now: n!~@host */
+  return buf;
+}
+
 struct userrec *check_dcclist_hand(char *handle)
 {
   int i;
 
   for (i = 0; i < dcc_total; i++)
-    if (!strcasecmp(dcc[i].nick, handle))
+    if (!egg_strcasecmp(dcc[i].nick, handle))
       return dcc[i].user;
   return NULL;
 }
 
 struct userrec *get_user_by_handle(struct userrec *bu, char *handle)
 {
-  struct userrec *u = bu, *ret;
+  struct userrec *u, *ret;
 
   if (!handle)
     return NULL;
+  /* FIXME: This should be done outside of this function. */
   rmspace(handle);
   if (!handle[0] || (handle[0] == '*'))
     return NULL;
   if (bu == userlist) {
-    if (lastuser && !strcasecmp(lastuser->handle, handle)) {
+    if (lastuser && !egg_strcasecmp(lastuser->handle, handle)) {
       cache_hit++;
       return lastuser;
     }
@@ -212,18 +231,17 @@ struct userrec *get_user_by_handle(struct userrec *bu, char *handle)
     }
     cache_miss++;
   }
-  while (u) {
-    if (!strcasecmp(u->handle, handle)) {
+  for (u = bu; u; u = u->next)
+    if (!egg_strcasecmp(u->handle, handle)) {
       if (bu == userlist)
 	lastuser = u;
       return u;
     }
-    u = u->next;
-  }
   return NULL;
 }
 
-/* fix capitalization, etc */
+/* Fix capitalization, etc
+ */
 void correct_handle(char *handle)
 {
   struct userrec *u;
@@ -234,16 +252,33 @@ void correct_handle(char *handle)
   strcpy(handle, u->handle);
 }
 
+/* This will be usefull in a lot of places, much more code re-use so we
+ * endup with a smaller executable bot. <cybah>
+ */
+void clear_masks(maskrec *m)
+{
+  maskrec *temp = NULL;
+
+  for (; m; m = temp) {
+    temp = m->next;
+    if (m->mask)
+      nfree(m->mask);
+    if (m->user)
+      nfree(m->user);
+    if (m->desc)
+      nfree(m->desc);
+    nfree(m);
+  }
+}
+
 void clear_userlist(struct userrec *bu)
 {
-  struct userrec *u = bu, *v;
+  struct userrec *u, *v;
   int i;
 
-  context;
-  while (u != NULL) {
+  for (u = bu; u; u = v) {
     v = u->next;
     freeuser(u);
-    u = v;
   }
   if (userlist == bu) {
     struct chanset_t *cst;
@@ -252,94 +287,37 @@ void clear_userlist(struct userrec *bu)
       dcc[i].user = NULL;
     clear_chanlist();
     lastuser = NULL;
-    while (global_bans) {
-      struct banrec *b = global_bans;
 
-      global_bans = b->next;
-      if (b->banmask)
-	nfree(b->banmask);
-      if (b->user)
-	nfree(b->user);
-      if (b->desc)
-	nfree(b->desc);
-      nfree(b);
-    }
     while (global_ign)
       delignore(global_ign->igmask);
-    while (global_exempts) {
-      struct exemptrec * e = global_exempts;
-      global_exempts = e->next;
-      if (e->exemptmask)
-	nfree(e->exemptmask);
-      if (e->user)
-	nfree(e->user);
-      if (e->desc)
-	nfree(e->desc);
-      nfree(e);
-    }
-    while (global_invites) {
-      struct inviterec * inv = global_invites;
-      global_invites = inv->next;
-      if (inv->invitemask)
-	nfree(inv->invitemask);
-      if (inv->user)  
-	nfree(inv->user);
-      if (inv->desc)
-	nfree(inv->desc);
-      nfree(inv);
-    }
-    for (cst = chanset; cst; cst = cst->next)
-      while (cst->bans) {
-	struct banrec *b = cst->bans;
 
-	cst->bans = b->next;
-	if (b->banmask)
-	  nfree(b->banmask);
-	if (b->user)
-	  nfree(b->user);
-	if (b->desc)
-	  nfree(b->desc);
-	nfree(b);
-      }
-    for (cst = chanset;cst;cst=cst->next)
-      while (cst->exempts) {
-	struct exemptrec * e = cst->exempts;
-	
-	cst->exempts = e->next; 
-	if (e->exemptmask)
-	  nfree(e->exemptmask); 
-	if (e->user)
-	  nfree(e->user);
-	if (e->desc)
-	  nfree(e->desc);
-	nfree(e);
-      }
-    for (cst = chanset;cst;cst=cst->next)
-      while (cst->invites) {
-	struct inviterec * inv = cst->invites;
-	
-	cst->invites = inv->next;
-	if (inv->invitemask)
-	  nfree(inv->invitemask);
-	if (inv->user)
-	  nfree(inv->user);
-	if (inv->desc)
-	  nfree(inv->desc);
-	nfree(inv);
-      }
+    clear_masks(global_bans);
+    clear_masks(global_exempts);
+    clear_masks(global_invites);
+    global_exempts = global_invites = global_bans = NULL;
+
+    for (cst = chanset; cst; cst = cst->next) {
+      clear_masks(cst->bans);
+      clear_masks(cst->exempts);
+      clear_masks(cst->invites);
+
+      cst->bans = cst->exempts = cst->invites = NULL;
+    }
   }
-  /* remember to set your userlist to NULL after calling this */
-  context;
+  /* Remember to set your userlist to NULL after calling this */
 }
 
-/* find CLOSEST host match */
-/* (if "*!*@*" and "*!*@*clemson.edu" both match, use the latter!) */
-/* 26feb: CHECK THE CHANLIST FIRST to possibly avoid needless search */
+/* Find CLOSEST host match
+ * (if "*!*@*" and "*!*@*clemson.edu" both match, use the latter!)
+ *
+ * Checks the chanlist first, to possibly avoid needless search.
+ */
 struct userrec *get_user_by_host(char *host)
 {
-  struct userrec *u = userlist, *ret;
+  struct userrec *u, *ret;
   struct list_type *q;
   int cnt, i;
+  char host2[UHOSTLEN];
 
   if (host == NULL)
     return NULL;
@@ -353,44 +331,42 @@ struct userrec *get_user_by_host(char *host)
     return ret;
   }
   cache_miss++;
-  while (u != NULL) {
+  strncpyz(host2, host, sizeof host2);
+  host = fixfrom(host);
+  for (u = userlist; u; u = u->next) {
     q = get_user(&USERENTRY_HOSTS, u);
-    while (q != NULL) {
+    for (; q; q = q->next) {
       i = wild_match(q->extra, host);
       if (i > cnt) {
 	ret = u;
 	cnt = i;
       }
-      q = q->next;
     }
-    u = u->next;
   }
   if (ret != NULL) {
     lastuser = ret;
-    set_chanlist(host, ret);
+    set_chanlist(host2, ret);
   }
   return ret;
 }
 
+/* use fixfrom() or dont? (drummer)
+ */
 struct userrec *get_user_by_equal_host(char *host)
 {
-  struct userrec *u = userlist;
+  struct userrec *u;
   struct list_type *q;
 
-  while (u != NULL) {
-    q = get_user(&USERENTRY_HOSTS, u);
-    while (q != NULL) {
+  for (u = userlist; u; u = u->next)
+    for (q = get_user(&USERENTRY_HOSTS, u); q; q = q->next)
       if (!rfc_casecmp(q->extra, host))
 	return u;
-      q = q->next;
-    }
-    u = u->next;
-  }
   return NULL;
 }
 
-/* try: pass_match_by_host("-",host)
- * will return 1 if no password is set for that host */
+/* Try: pass_match_by_host("-",host)
+ * will return 1 if no password is set for that host
+ */
 int u_pass_match(struct userrec *u, char *pass)
 {
   char *cmp, new[32];
@@ -398,9 +374,9 @@ int u_pass_match(struct userrec *u, char *pass)
   if (!u)
     return 0;
   cmp = get_user(&USERENTRY_PASS, u);
-  if (!cmp)
+  if (!cmp && (!pass[0] || (pass[0] == '-')))
     return 1;
-  if (!pass || !pass[0] || (pass[0] == '-'))
+  if (!cmp || !pass || !pass[0] || (pass[0] == '-'))
     return 0;
   if (u->flags & USER_BOT) {
     if (!strcmp(cmp, pass))
@@ -421,17 +397,15 @@ int write_user(struct userrec *u, FILE * f, int idx)
   struct chanuserrec *ch;
   struct chanset_t *cst;
   struct user_entry *ue;
-  struct flag_record fr =
-  {FR_GLOBAL, 0, 0, 0, 0, 0};
+  struct flag_record fr = {FR_GLOBAL, 0, 0, 0, 0, 0};
 
   fr.global = u->flags;
-
   fr.udef_global = u->flags_udef;
   build_flags(s, &fr, NULL);
   if (fprintf(f, "%-10s - %-24s\n", u->handle, s) == EOF)
     return 0;
   for (ch = u->chanrec; ch; ch = ch->next) {
-    cst = findchan(ch->channel);
+    cst = findchan_by_dname(ch->channel);
     if (cst && ((idx < 0) || channel_shared(cst))) {
       if (idx >= 0) {
 	fr.match = (FR_CHAN | FR_BOT);
@@ -467,10 +441,11 @@ int write_user(struct userrec *u, FILE * f, int idx)
 
 int sort_compare(struct userrec *a, struct userrec *b)
 {
-  /* order by flags, then alphabetically
+  /* Order by flags, then alphabetically
    * first bots: +h / +a / +l / other bots
    * then users: +n / +m / +o / other users
-   * return true if (a > b) */
+   * return true if (a > b)
+   */
   if (a->flags & b->flags & USER_BOT) {
     if (~bot_flags(a) & bot_flags(b) & BOT_HUB)
       return 1;
@@ -502,7 +477,7 @@ int sort_compare(struct userrec *a, struct userrec *b)
     if (a->flags & ~b->flags & USER_OP)
       return 0;
   }
-  return (strcasecmp(a->handle, b->handle) > 0);
+  return (egg_strcasecmp(a->handle, b->handle) > 0);
 }
 
 void sort_userlist()
@@ -535,86 +510,79 @@ void sort_userlist()
   }
 }
 
-/* rewrite the entire user file */
+/* Rewrite the entire user file. Call USERFILE hook as well, probably
+ * causing the channel file to be rewritten as well.
+ */
 void write_userfile(int idx)
 {
   FILE *f;
-  char s[121], s1[81];
+  char *new_userfile;
+  char s1[81];
   time_t tt;
   struct userrec *u;
   int ok;
 
-  context;
-  /* also write the channel file at the same time */
   if (userlist == NULL)
-    return;			/* no point in saving userfile */
-  sprintf(s, "%s~new", userfile);
-  f = fopen(s, "w");
-  chmod(s, 0600);		/* make it -rw------- */
+    return;			/* No point in saving userfile */
+
+  new_userfile = nmalloc(strlen(userfile) + 5);
+  sprintf(new_userfile, "%s~new", userfile);
+
+  f = fopen(new_userfile, "w");
+  chmod(new_userfile, userfile_perm);
   if (f == NULL) {
     putlog(LOG_MISC, "*", USERF_ERRWRITE);
+    nfree(new_userfile);
     return;
   }
-  putlog(LOG_MISC, "*", USERF_WRITING);
+  if (!quiet_save)
+    putlog(LOG_MISC, "*", USERF_WRITING);
   if (sort_users)
     sort_userlist();
   tt = now;
   strcpy(s1, ctime(&tt));
   fprintf(f, "#4v: %s -- %s -- written %s", ver, botnetnick, s1);
-  context;
   ok = 1;
-  u = userlist;
-  while ((u != NULL) && (ok)) {
+  for (u = userlist; u && ok; u = u->next)
     ok = write_user(u, f, idx);
-    u = u->next;
-  }
-  context;
   if (!ok || fflush(f)) {
     putlog(LOG_MISC, "*", "%s (%s)", USERF_ERRWRITE, strerror(ferror(f)));
     fclose(f);
+    nfree(new_userfile);
     return;
   }
   fclose(f);
-  context;
   call_hook(HOOK_USERFILE);
-  context;
-  unlink(userfile);
-  sprintf(s, "%s~new", userfile);
-#ifdef RENAME
-  rename(s, userfile);
-#else
-  movefile(s, userfile);
-#endif
+  movefile(new_userfile, userfile);
+  nfree(new_userfile);
 }
 
 int change_handle(struct userrec *u, char *newh)
 {
   int i;
-  char s[16];
+  char s[HANDLEN + 1];
 
   if (!u)
     return 0;
-  /* nothing that will confuse the userfile */
-  if ((newh[1] == 0) && strchr("+*:=.-#&", newh[0]))
+  /* Nothing that will confuse the userfile */
+  if (!newh[1] && strchr(BADHANDCHARS, newh[0]))
     return 0;
   check_tcl_nkch(u->handle, newh);
-  /* yes, even send bot nick changes now: */
-  if ((!noshare) && !(u->flags & USER_UNSHARED))
+  /* Yes, even send bot nick changes now: */
+  if (!noshare && !(u->flags & USER_UNSHARED))
     shareout(NULL, "h %s %s\n", u->handle, newh);
-  strcpy(s, u->handle);
-  strcpy(u->handle, newh);
-  for (i = 0; i < dcc_total; i++) {
-    if (!strcasecmp(dcc[i].nick, s) &&
-	(dcc[i].type != &DCC_BOT)) {
-      strcpy(dcc[i].nick, newh);
-      if ((dcc[i].type == &DCC_CHAT) && (dcc[i].u.chat->channel >= 0)) {
+  strncpyz(s, u->handle, sizeof s);
+  strncpyz(u->handle, newh, sizeof u->handle);
+  for (i = 0; i < dcc_total; i++)
+    if (dcc[i].type != &DCC_BOT && !egg_strcasecmp(dcc[i].nick, s)) {
+      strncpyz(dcc[i].nick, newh, sizeof dcc[i].nick);
+      if (dcc[i].type == &DCC_CHAT && dcc[i].u.chat->channel >= 0) {
 	chanout_but(-1, dcc[i].u.chat->channel,
-		    "*** Nick change: %s -> %s\n", s, newh);
+		    "*** Handle change: %s -> %s\n", s, newh);
 	if (dcc[i].u.chat->channel < 100000)
 	  botnet_send_nkch(i, s);
       }
     }
-  }
   return 1;
 }
 
@@ -631,31 +599,39 @@ struct userrec *adduser(struct userrec *bu, char *handle, char *host,
   u = (struct userrec *) nmalloc(sizeof(struct userrec));
 
   /* u->next=bu; bu=u; */
-  strncpy(u->handle, handle, HANDLEN);
-  u->handle[HANDLEN] = 0;
+  strncpyz(u->handle, handle, sizeof u->handle);
   u->next = NULL;
   u->chanrec = NULL;
   u->entries = NULL;
   if (flags != USER_DEFAULT) { /* drummer */
-  u->flags = flags;
-  u->flags_udef = 0;
+    u->flags = flags;
+    u->flags_udef = 0;
   } else {
     u->flags = default_flags;
     u->flags_udef = default_uflags;
-  }  
+  }
   set_user(&USERENTRY_PASS, u, pass);
   if (!noxtra) {
     xk = nmalloc(sizeof(struct xtra_key));
-
     xk->key = nmalloc(8);
     strcpy(xk->key, "created");
-    xk->data = nmalloc(10);
-    sprintf(xk->data, "%09lu", now);
+    xk->data = nmalloc(15);              /* should be enough, this should be changed  */
+    sprintf(xk->data, "%lu", now);      /* when time_t is a 64 bit integer.. -poptix */
     set_user(&USERENTRY_XTRA, u, xk);
+/*    nfree(xk->data);    If not here, where?
+    nfree(xk); */
   }
-  /* strip out commas -- they're illegal */
+  /* Strip out commas -- they're illegal */
   if (host && host[0]) {
-    char *p = strchr(host, ',');
+    char *p;
+
+    /* About this fixfrom():
+     *   We should use this fixfrom before every call of adduser()
+     *   but its much easier to use here...  (drummer)
+     *   Only use it if we have a host :) (dw)
+     */
+    host = fixfrom(host);
+    p = strchr(host, ',');
 
     while (p != NULL) {
       *p = '?';
@@ -669,12 +645,14 @@ struct userrec *adduser(struct userrec *bu, char *handle, char *host,
   noshare = oldshare;
   if ((!noshare) && (handle[0] != '*') && (!(flags & USER_UNSHARED)) &&
       (bu == userlist)) {
-    struct flag_record fr =
-    {FR_GLOBAL, u->flags, u->flags_udef, 0, 0, 0};    
+    struct flag_record fr = {FR_GLOBAL, 0, 0, 0, 0, 0};
     char x[100];
 
+    fr.global = u->flags;
+    fr.udef_global = u->flags_udef;
     build_flags(x, &fr, 0);
-    shareout(NULL, "n %s %s %s %s\n", handle, host, pass, x);
+    shareout(NULL, "n %s %s %s %s\n", handle, host && host[0] ? host : "none",
+             pass, x);
   }
   if (bu == NULL)
     bu = u;
@@ -733,7 +711,7 @@ int deluser(char *handle)
   int fnd = 0;
 
   while ((u != NULL) && (!fnd)) {
-    if (!strcasecmp(u->handle, handle))
+    if (!egg_strcasecmp(u->handle, handle))
       fnd = 1;
     else {
       prev = u;
@@ -750,7 +728,7 @@ int deluser(char *handle)
     shareout(NULL, "k %s\n", handle);
   for (fnd = 0; fnd < dcc_total; fnd++)
     if (dcc[fnd].user == u)
-      dcc[fnd].user = 0;	/* clear any dcc users for this entry,
+      dcc[fnd].user = 0;	/* Clear any dcc users for this entry,
 				 * null is safe-ish */
   clear_chanlist();
   freeuser(u);
@@ -761,34 +739,44 @@ int deluser(char *handle)
 int delhost_by_handle(char *handle, char *host)
 {
   struct userrec *u;
-  struct list_type *q, *t = 0;
-  struct user_entry *e;
+  struct list_type *q, *qnext, *qprev;
+  struct user_entry *e = NULL;
   int i = 0;
 
-  context;
   u = get_user_by_handle(userlist, handle);
-  if (u == NULL)
+  if (!u)
     return 0;
   q = get_user(&USERENTRY_HOSTS, u);
-  if (q && !rfc_casecmp(q->extra, host)) {
-    e = find_user_entry(&USERENTRY_HOSTS, u);
-    e->u.extra = t = q->next;
-    nfree(q->extra);
-    nfree(q);
-    i = 1;
-    q = t;
-  } else if (q)
-    while (q->next) {
-      if (!rfc_casecmp(q->next->extra, host)) {
-	t = q->next;
-	q->next = t->next;
-	nfree(t->extra);
-	nfree(t);
-	i = 1;
+  qprev = q;
+  if (q) {
+    if (!rfc_casecmp(q->extra, host)) {
+      e = find_user_entry(&USERENTRY_HOSTS, u);
+      e->u.extra = q->next;
+      nfree(q->extra);
+      nfree(q);
+      i++;
+      qprev = NULL;
+      q = e->u.extra;
+    } else
+      q = q->next;
+    while (q) {
+      qnext = q->next;
+      if (!rfc_casecmp(q->extra, host)) {
+	if (qprev)
+	  qprev->next = q->next;
+	else if (e) {
+	  e->u.extra = q->next;
+	  qprev = NULL;
+	}
+	nfree(q->extra);
+	nfree(q);
+	i++;
       } else
-	q = q->next;
+        qprev = q;
+      q = qnext;
     }
-  if (!q)
+  }
+  if (!qprev)
     set_user(&USERENTRY_HOSTS, u, "none");
   if (!noshare && i && !(u->flags & USER_UNSHARED))
     shareout(NULL, "-h %s %s\n", handle, host);
@@ -835,3 +823,55 @@ void touch_laston(struct userrec *u, char *where, time_t timeval)
     set_user(&USERENTRY_LASTON, u, 0);
   }
 }
+
+/*  Go through all channel records and try to find a matching
+ *  nick. Will return the user's user record if that is known
+ *  to the bot.  (Fabian)
+ *
+ *  Warning: This is unreliable by concept!
+ */
+struct userrec *get_user_by_nick(char *nick)
+{
+  struct chanset_t *chan;
+  memberlist *m;
+
+  for (chan = chanset; chan; chan = chan->next) {
+    for (m = chan->channel.member; m && m->nick[0] ;m = m->next) {
+      if (!rfc_casecmp(nick, m->nick)) {
+  	char word[512];
+
+	egg_snprintf(word, sizeof word, "%s!%s", m->nick, m->userhost);
+	/* No need to check the return value ourself */
+	return get_user_by_host(word);;
+      }
+    }
+  }
+  /* Sorry, no matches */
+  return NULL;
+}
+
+void user_del_chan(char *dname)
+{
+  struct chanuserrec *ch, *och;
+  struct userrec *u;
+
+  for (u = userlist; u; u = u->next) {
+    ch = u->chanrec;
+    och = NULL;
+    while (ch) {
+      if (!rfc_casecmp(dname, ch->channel)) {
+	if (och)
+	  och->next = ch->next;
+	else
+	  u->chanrec = ch->next;
+
+	if (ch->info)
+	  nfree(ch->info);
+	nfree(ch);
+	break;
+      }
+      och = ch;
+      ch = ch->next;
+    }
+  }
+}
