@@ -1,21 +1,34 @@
-/* 
+/*
  * users.c -- handles:
- * testing and enforcing ignores
- * adding and removing ignores
- * listing ignores
- * auto-linking bots
- * sending and receiving a userfile from a bot
- * listing users ('.whois' and '.match')
- * reading the user file
- * 
+ *   testing and enforcing ignores
+ *   adding and removing ignores
+ *   listing ignores
+ *   auto-linking bots
+ *   sending and receiving a userfile from a bot
+ *   listing users ('.whois' and '.match')
+ *   reading the user file
+ *
  * dprintf'ized, 9nov1995
+ *
+ * $Id: users.c,v 1.28 2002/01/02 03:46:36 guppy Exp $
  */
 /*
- * This file is part of the eggdrop source code
- * copyright (c) 1997 Robey Pointer
- * and is distributed according to the GNU general public license.
- * For full details, read the top of 'main.c' or the file called
- * COPYING that was distributed with this code.
+ * Copyright (C) 1997 Robey Pointer
+ * Copyright (C) 1999, 2000, 2001, 2002 Eggheads Development Team
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
 #include "main.h"
@@ -31,18 +44,12 @@ char spaces[33] = "                                 ";
 char spaces2[33] = "                                 ";
 
 extern struct dcc_t *dcc;
-extern int dcc_total;
-extern int noshare;
 extern struct userrec *userlist, *lastuser;
-extern struct banrec *global_bans;
-extern struct igrec *global_ign;
-extern struct exemptrec *global_exempts;
-extern struct inviterec *global_invites;
-extern char botnetnick[];
 extern struct chanset_t *chanset;
+extern int dcc_total, noshare;
+extern char botnetnick[];
 extern Tcl_Interp *interp;
 extern time_t now;
-extern int use_silence;
 
 char userfile[121] = "";	/* where the user records are stored */
 int ignore_time = 10;		/* how many minutes will ignores last? */
@@ -81,8 +88,6 @@ int delignore(char *ign)
   struct igrec **u;
   struct igrec *t;
 
-  context;
-
   i = 0;
   if (!strchr(ign, '!') && (j = atoi(ign))) {
     for (u = &global_ign, j--; *u && j; u = &((*u)->next), j--);
@@ -99,8 +104,14 @@ int delignore(char *ign)
       }
   }
   if (i) {
-    if (!noshare)
-      shareout(NULL, "-i %s\n", ign);
+    if (!noshare) {
+      char *mask = str_escape(ign, ':', '\\');
+
+      if (mask) {
+	shareout(NULL, "-i %s\n", mask);
+	nfree(mask);
+      }
+    }
     nfree((*u)->igmask);
     if ((*u)->msg)
       nfree((*u)->msg);
@@ -132,9 +143,15 @@ void addignore(char *ign, char *from, char *mnote, time_t expire_time)
   strcpy(p->user, from);
   p->msg = user_malloc(strlen(mnote) + 1);
   strcpy(p->msg, mnote);
-  if (!noshare)
-    shareout(NULL, "+i %s %lu %c %s %s\n", ign, expire_time - now,
-	     (p->flags & IGREC_PERM) ? 'p' : '-', from, mnote);
+  if (!noshare) {
+    char *mask = str_escape(ign, ':', '\\');
+
+    if (mask) {
+      shareout(NULL, "+i %s %lu %c %s %s\n", mask, expire_time - now,
+	       (p->flags & IGREC_PERM) ? 'p' : '-', from, mnote);
+      nfree(mask);
+    }
+  }
 }
 
 /* take host entry from ignore list and display it ignore-style */
@@ -162,7 +179,7 @@ void display_ignore(int idx, int number, struct igrec *ignore)
   if (ignore->msg && ignore->msg[0])
     dprintf(idx, "        %s: %s\n", ignore->user, ignore->msg);
   else
-    dprintf(idx, "        %s %s\n", BANS_PLACEDBY, ignore->user);
+    dprintf(idx, "        %s %s\n", MODES_PLACEDBY, ignore->user);
   if (dates[0])
     dprintf(idx, "        %s\n", dates);
 }
@@ -201,17 +218,6 @@ void check_expired_ignores()
     if (!((*u)->flags & IGREC_PERM) && (now >= (*u)->expire)) {
       putlog(LOG_MISC, "*", "%s %s (%s)", IGN_NOLONGER, (*u)->igmask,
 	     MISC_EXPIRED);
-      if (use_silence) {
-	char *p;
-
-	/* possibly an ircu silence was added for this user */
-	p = strchr((*u)->igmask, '!');
-	if (p == NULL)
-	  p = (*u)->igmask;
-	else
-	  p++;
-	dprintf(DP_SERVER, "SILENCE -%s\n", p);
-      }
       delignore((*u)->igmask);
     } else {
       u = &((*u)->next);
@@ -219,123 +225,46 @@ void check_expired_ignores()
   }
 }
 
-/* channel ban loaded from user file */
-static void addban_fully(struct chanset_t *chan, char *ban, char *from,
+/*        Channel mask loaded from user file. This function is
+ *      add(ban|invite|exempt)_fully merged into one. <cybah>
+ */
+static void addmask_fully(struct chanset_t *chan, maskrec **m, maskrec **global,
+                         char *mask, char *from,
 			 char *note, time_t expire_time, int flags,
 			 time_t added, time_t last)
 {
-  struct banrec *p = user_malloc(sizeof(struct banrec));
-  struct banrec **u = chan ? &chan->bans : &global_bans;
-  char *t;
+  maskrec *p = user_malloc(sizeof(maskrec));
+  maskrec **u = (chan) ? m : global;
 
-  /* decode gibberish stuff */
-  t = strchr(note, '~');
-  while (t != NULL) {
-    *t = ' ';
-    t = strchr(note, '~');
-  }
-  t = strchr(note, '`');
-  while (t != NULL) {
-    *t = ',';
-    t = strchr(note, '`');
-  }
   p->next = *u;
   *u = p;
   p->expire = expire_time;
   p->added = added;
   p->lastactive = last;
   p->flags = flags;
-  p->banmask = user_malloc(strlen(ban) + 1);
-  strcpy(p->banmask, ban);
+  p->mask = user_malloc(strlen(mask) + 1);
+  strcpy(p->mask, mask);
   p->user = user_malloc(strlen(from) + 1);
   strcpy(p->user, from);
   p->desc = user_malloc(strlen(note) + 1);
   strcpy(p->desc, note);
 }
 
-/* channel exempt loaded from user file */
-static void addexempt_fully (struct chanset_t * chan, char * exempt, char * from,
-			     char * note, time_t expire_time, int flags, time_t added,
-			     time_t last) {
-  struct exemptrec * p = user_malloc(sizeof(struct exemptrec));
-  struct exemptrec ** u = chan? &chan->exempts : &global_exempts;
-  char * t;
-  
-  /* decode gibberish stuff */
-  t = strchr(note, '~');
-  while (t != NULL) {
-    *t = ' ';
-    t = strchr(note, '~');
-  }
-  t = strchr(note, '`');
-  while (t != NULL) {
-    *t = ',';
-    t = strchr(note, '`');
-  }
-  p->next = *u;
-  *u = p;
-  p->expire = expire_time;
-  p->added = added;
-  p->lastactive = last;
-  p->flags = flags;
-  p->exemptmask = user_malloc(strlen(exempt)+1);
-  strcpy(p->exemptmask,exempt);
-  p->user = user_malloc(strlen(from)+1);
-  strcpy(p->user,from);
-  p->desc = user_malloc(strlen(note)+1);
-  strcpy(p->desc,note);
-}
-
-/* channel invite loaded from user file */
-static void addinvite_fully (struct chanset_t * chan, char * invite, char * from,
-			     char * note, time_t expire_time, int flags, time_t added,
-			     time_t last) {
-  struct inviterec * p = user_malloc(sizeof(struct inviterec));
-  struct inviterec ** u = chan? &chan->invites : &global_invites;
-  char * t;
-  
-  /* decode gibberish stuff */
-  t = strchr(note, '~');
-  while (t != NULL) { 
-    *t = ' ';
-    t = strchr(note, '~');
-  }
-  t = strchr(note, '`');
-  while (t != NULL) {   
-    *t = ',';
-    t = strchr(note, '`');
-  }
-  p->next = *u;
-  *u = p;
-  p->expire = expire_time;
-  p->added = added;
-  p->lastactive = last;
-  p->flags = flags;
-  p->invitemask = user_malloc(strlen(invite)+1);
-  strcpy(p->invitemask,invite);
-  p->user = user_malloc(strlen(from)+1);
-  strcpy(p->user,from);
-  p->desc = user_malloc(strlen(note)+1);
-  strcpy(p->desc,note);
-}
-
 static void restore_chanban(struct chanset_t *chan, char *host)
 {
   char *expi, *add, *last, *user, *desc;
   int flags = 0;
 
-  expi = strchr(host, ':');
+  expi = strchr_unescape(host, ':', '\\');
   if (expi) {
-    *expi = 0;
-    expi++;
     if (*expi == '+') {
-      flags |= BANREC_PERM;
+      flags |= MASKREC_PERM;
       expi++;
     }
     add = strchr(expi, ':');
     if (add) {
       if (add[-1] == '*') {
-	flags |= BANREC_STICKY;
+	flags |= MASKREC_STICKY;
 	add[-1] = 0;
       } else
 	*add = 0;
@@ -353,8 +282,8 @@ static void restore_chanban(struct chanset_t *chan, char *host)
 	    if (desc) {
 	      *desc = 0;
 	      desc++;
-	      addban_fully(chan, host, user, desc, atoi(expi), flags,
-			   atoi(add), atoi(last));
+	      addmask_fully(chan, &chan->bans, &global_bans, host, user,
+			    desc, atoi(expi), flags, atoi(add), atoi(last));
 	      return;
 	    }
 	  }
@@ -364,140 +293,133 @@ static void restore_chanban(struct chanset_t *chan, char *host)
 	if (desc) {
 	  *desc = 0;
 	  desc++;
-	  addban_fully(chan, host, add, desc, atoi(expi), flags, now, 0);
+	  addmask_fully(chan, &chan->bans, &global_bans, host, add, desc,
+			atoi(expi), flags, now, 0);
 	  return;
 	}
       }
     }
   }
-  putlog(LOG_MISC, "*", "*** Malformed banline for %s.\n",
-	 chan ? chan->name : "global_bans");
+  putlog(LOG_MISC, "*", "*** Malformed banline for %s.",
+	 chan ? chan->dname : "global_bans");
 }
 
-static void restore_chanexempt (struct chanset_t * chan, char * host)
+static void restore_chanexempt(struct chanset_t *chan, char *host)
 {
-  char * expi, * add, * last, * user, * desc;
+  char *expi, *add, *last, *user, *desc;
   int flags = 0;
-  
-  expi = strchr(host,':');
+
+  expi = strchr_unescape(host, ':', '\\');
   if (expi) {
-    *expi = 0;
-    expi++;
       if (*expi == '+') {
-	flags |= EXEMPTREC_PERM;
+	flags |= MASKREC_PERM;
 	expi++;
       }
-    add = strchr(expi,':');
-    if (add) { 
-      if (add[-1]=='*') {
-	flags |= EXEMPTREC_STICKY;
-	add[-1] = 0;   
+    add = strchr(expi, ':');
+    if (add) {
+      if (add[-1] == '*') {
+	flags |= MASKREC_STICKY;
+	add[-1] = 0;
       } else
 	*add = 0;
       add++;
       if (*add == '+') {
-	last = strchr(add,':');
+	last = strchr(add, ':');
 	if (last) {
 	  *last = 0;
 	  last++;
-	  user = strchr(last,':');
+	  user = strchr(last, ':');
 	  if (user) {
 	    *user = 0;
 	    user++;
-	    desc = strchr(user,':');
+	    desc = strchr(user, ':');
 	    if (desc) {
 	      *desc = 0;
 	      desc++;
-	      addexempt_fully(chan,host,user,desc,atoi(expi),flags,
-			      atoi(add), atoi(last));
+	      addmask_fully(chan, &chan->exempts, &global_exempts, host, user,
+			    desc, atoi(expi), flags, atoi(add), atoi(last));
 	      return;
 	    }
 	  }
 	}
       } else {
-	desc = strchr(add,':');
-	
+	desc = strchr(add, ':');
 	if (desc) {
 	  *desc = 0;
 	  desc++;
-	  addexempt_fully(chan,host,add,desc,atoi(expi),flags,
-			  now, 0);
-	  return;
-	}
-      } 
-    }
-  }
-  putlog(LOG_MISC,"*","*** Malformed exemptline for %s.\n",
-	 chan?chan->name:"global_exempts");
-}
-
-static void restore_chaninvite (struct chanset_t * chan, char * host)
-{
-  char * expi, * add, * last, * user, * desc;
-  int flags = 0;
-  
-  expi = strchr(host,':');
-  if (expi) { 
-    *expi = 0;
-    expi++;  
-    if (*expi == '+') {  
-      flags |= INVITEREC_PERM;  
-      expi++;
-    }
-    add = strchr(expi,':');
-    if (add) {
-      if (add[-1]=='*') {
-	flags |= INVITEREC_STICKY;
-	add[-1] = 0; 
-      } else
-	*add = 0;
-      add++;
-      if (*add == '+') { 
-	last = strchr(add,':');
-	if (last) {
-	  *last = 0;   
-	  last++;
-	  user = strchr(last,':');
-	  if (user) {
-	    *user = 0;
-	    user++;
-	    desc = strchr(user,':');
-	    if (desc) {
-	      *desc = 0;
-	      desc++;
-	      addinvite_fully(chan,host,user,desc,atoi(expi),flags,
-			      atoi(add), atoi(last));
-	      return;
-	    }
-	  }
-	}
-      } else {
-	desc = strchr(add,':');
-	
-	if (desc) {
-	  *desc = 0;
-	  desc++;
-	  addinvite_fully(chan,host,add,desc,atoi(expi),flags,
-			  now, 0);
+	  addmask_fully(chan, &chan->exempts, &global_exempts, host, add,
+			desc, atoi(expi), flags, now, 0);
 	  return;
 	}
       }
     }
   }
-  putlog(LOG_MISC,"*","*** Malformed inviteline for %s.\n",
-	 chan?chan->name:"global_invites");
+  putlog(LOG_MISC, "*", "*** Malformed exemptline for %s.",
+	 chan ? chan->dname : "global_exempts");
+}
+
+static void restore_chaninvite(struct chanset_t *chan, char *host)
+{
+  char *expi, *add, *last, *user, *desc;
+  int flags = 0;
+
+  expi = strchr_unescape(host, ':', '\\');
+  if (expi) {
+    if (*expi == '+') {
+      flags |= MASKREC_PERM;
+      expi++;
+    }
+    add = strchr(expi, ':');
+    if (add) {
+      if (add[-1] == '*') {
+	flags |= MASKREC_STICKY;
+	add[-1] = 0;
+      } else
+	*add = 0;
+      add++;
+      if (*add == '+') {
+	last = strchr(add, ':');
+	if (last) {
+	  *last = 0;
+	  last++;
+	  user = strchr(last, ':');
+	  if (user) {
+	    *user = 0;
+	    user++;
+	    desc = strchr(user, ':');
+	    if (desc) {
+	      *desc = 0;
+	      desc++;
+	      addmask_fully(chan, &chan->invites, &global_invites, host, user,
+			    desc, atoi(expi), flags, atoi(add), atoi(last));
+	      return;
+	    }
+	  }
+	}
+      } else {
+	desc = strchr(add, ':');
+	if (desc) {
+	  *desc = 0;
+	  desc++;
+	  addmask_fully(chan, &chan->invites, &global_invites, host, add,
+			desc, atoi(expi), flags, now, 0);
+	  return;
+	}
+      }
+    }
+  }
+  putlog(LOG_MISC, "*", "*** Malformed inviteline for %s.",
+	 chan ? chan->dname : "global_invites");
 }
 
 static void restore_ignore(char *host)
 {
-  char *expi, *user, *added, *desc, *t;
+  char *expi, *user, *added, *desc;
   int flags = 0;
   struct igrec *p;
 
-  expi = strchr(host, ':');
+  expi = strchr_unescape(host, ':', '\\');
   if (expi) {
-    *expi = 0;
-    expi++;
     if (*expi == '+') {
       flags |= IGREC_PERM;
       expi++;
@@ -514,17 +436,6 @@ static void restore_ignore(char *host)
 	if (desc) {
 	  *desc = 0;
 	  desc++;
-	  /* decode gibberish stuff */
-	  t = strchr(desc, '~');
-	  while (t != NULL) {
-	    *t = ' ';
-	    t = strchr(desc, '~');
-	  }
-	  t = strchr(desc, '`');
-	  while (t != NULL) {
-	    *t = ',';
-	    t = strchr(desc, '`');
-	  }
 	} else
 	  desc = NULL;
       } else {
@@ -550,7 +461,7 @@ static void restore_ignore(char *host)
       return;
     }
   }
-  putlog(LOG_MISC, "*", "*** Malformed ignore line.\n");
+  putlog(LOG_MISC, "*", "*** Malformed ignore line.");
 }
 
 void tell_user(int idx, struct userrec *u, int master)
@@ -561,12 +472,9 @@ void tell_user(int idx, struct userrec *u, int master)
   struct chanuserrec *ch;
   struct user_entry *ue;
   struct laston_info *li;
-  struct flag_record fr =
-  {FR_GLOBAL, 0, 0, 0, 0, 0};
+  struct flag_record fr = {FR_GLOBAL, 0, 0, 0, 0, 0};
 
-  context;
   fr.global = u->flags;
-
   fr.udef_global = u->flags_udef;
   build_flags(s, &fr, NULL);
   Tcl_SetVar(interp, "user", u->handle, 0);
@@ -578,26 +486,18 @@ void tell_user(int idx, struct userrec *u, int master)
     strcpy(s1, "never");
   else {
     now2 = now - li->laston;
-    strcpy(s1, ctime(&li->laston));
-    if (now2 > 86400) {
-      s1[7] = 0;
-      strcpy(&s1[11], &s1[4]);
-      strcpy(s1, &s1[8]);
-    } else {
-      s1[16] = 0;
-      strcpy(s1, &s1[11]);
-    }
+    if (now2 > 86400)
+      egg_strftime(s1, 7, "%d %b", localtime(&li->laston));
+    else
+      egg_strftime(s1, 6, "%H:%M", localtime(&li->laston));
   }
-  context;
   spaces[l] = 0;
   dprintf(idx, "%s%s %-5s%5d %-15s %s (%-10.10s)\n", u->handle, spaces,
 	  get_user(&USERENTRY_PASS, u) ? "yes" : "no", n, s, s1,
 	  (li && li->lastonplace) ? li->lastonplace : "nowhere");
   spaces[l] = ' ';
   /* channel flags? */
-  context;
-  ch = u->chanrec;
-  while (ch != NULL) {
+  for (ch = u->chanrec; ch; ch = ch->next) {
     fr.match = FR_CHAN | FR_GLOBAL;
     get_user_flagrec(dcc[idx].user, &fr, ch->channel);
     if (glob_op(fr) || chan_op(fr)) {
@@ -605,15 +505,10 @@ void tell_user(int idx, struct userrec *u, int master)
 	strcpy(s1, "never");
       else {
 	now2 = now - (ch->laston);
-	strcpy(s1, ctime(&(ch->laston)));
-	if (now2 > 86400) {
-	  s1[7] = 0;
-	  strcpy(&s1[11], &s1[4]);
-	  strcpy(s1, &s1[8]);
-	} else {
-	  s1[16] = 0;
-	  strcpy(s1, &s1[11]);
-	}
+	if (now2 > 86400)
+	  egg_strftime(s1, 7, "%d %b", localtime(&ch->laston));
+	else
+	  egg_strftime(s1, 6, "%H:%M", localtime(&ch->laston));
       }
       fr.match = FR_CHAN;
       fr.chan = ch->flags;
@@ -625,10 +520,8 @@ void tell_user(int idx, struct userrec *u, int master)
       if (ch->info != NULL)
 	dprintf(idx, "    INFO: %s\n", ch->info);
     }
-    ch = ch->next;
   }
   /* user-defined extra fields */
-  context;
   for (ue = u->entries; ue; ue = ue->next)
     if (!ue->name && ue->type->display)
       ue->type->display(idx, ue);
@@ -658,12 +551,11 @@ void tell_user_ident(int idx, char *id, int master)
 void tell_users_match(int idx, char *mtch, int start, int limit,
 		      int master, char *chname)
 {
-  struct userrec *u = userlist;
+  struct userrec *u;
   int fnd = 0, cnt, nomns = 0, flags = 0;
   struct list_type *q;
   struct flag_record user, pls, mns;
 
-  context;
   dprintf(idx, "*** %s '%s':\n", MISC_MATCHING, mtch);
   cnt = 0;
   spaces[HANDLEN - 6] = 0;
@@ -689,7 +581,7 @@ void tell_users_match(int idx, char *mtch, int start, int limit,
       chname = dcc[idx].u.chat->con_chan;
     flags = 1;
   }
-  while (u != NULL) {
+  for (u = userlist; u; u = u->next) {
     if (flags) {
       get_user_flagrec(u, &user, chname);
       if (flagrec_eq(&pls, &user)) {
@@ -721,7 +613,6 @@ void tell_users_match(int idx, char *mtch, int start, int limit,
 	}
       }
     }
-    u = u->next;
   }
   dprintf(idx, MISC_FOUNDMATCH, cnt, cnt == 1 ? "" : "es");
 }
@@ -770,7 +661,6 @@ int readuserfile(char *file, struct userrec **ret)
   struct flag_record fr;
   struct chanuserrec *cr;
 
-  context;
   bu = (*ret);
   ignored[0] = 0;
   if (bu == userlist) {
@@ -786,7 +676,6 @@ int readuserfile(char *file, struct userrec **ret)
   if (f == NULL)
     return 0;
   noshare = noxtra = 1;
-  context;
   /* read opening comment */
   s = buf;
   fgets(s, 180, f);
@@ -802,64 +691,59 @@ int readuserfile(char *file, struct userrec **ret)
     s = buf;
     fgets(s, 511, f);
     if (!feof(f)) {
-      if ((s[0] != '#') && (s[0] != ';') && (s[0])) {
+      if (s[0] != '#' && s[0] != ';' && s[0]) {
 	code = newsplit(&s);
 	rmspace(s);
 	if (!strcmp(code, "-")) {
-	  if (lasthand[0]) {
-	    if (u) {		/* only break it down if there a real users */
+	  if (!lasthand[0])
+	    continue;		/* Skip this entry.	*/
+	  if (u) {		/* only break it down if there a real users */
+	    p = strchr(s, ',');
+	    while (p != NULL) {
+	      splitc(s1, s, ',');
+	      rmspace(s1);
+	      if (s1[0])
+		set_user(&USERENTRY_HOSTS, u, s1);
 	      p = strchr(s, ',');
-	      while (p != NULL) {
-		splitc(s1, s, ',');
-		rmspace(s1);
-		if (s1[0])
-		  set_user(&USERENTRY_HOSTS, u, s1);
-		p = strchr(s, ',');
-	      }
-	    }
-	    /* channel bans are never stacked with , */
-	    if (s[0]) {
-	      if (strchr(CHANMETA, lasthand[0]) != NULL)
-		restore_chanban(cst, s);
-	      else if (lasthand[0] == '*') {
-		if (lasthand[1] == 'i') {
-		  restore_ignore(s);
-		} else {
-		  restore_chanban(NULL, s);
-		  gban_total++;
-		}
-	      } else if (lasthand[0]) {
-		set_user(&USERENTRY_HOSTS, u, s);
-	      }
 	    }
 	  }
-	} else if (strcmp(code, "%") == 0) { /* exemptmasks */
-	  if (lasthand[0]) {
-	    if (s[0]) {
-	      if ((lasthand[0] == '#') || (lasthand[0] == '+')) {
-		restore_chanexempt(cst,s);
+	  /* channel bans are never stacked with , */
+	  if (s[0]) {
+	    if (lasthand[0] && strchr(CHANMETA, lasthand[0]) != NULL)
+	      restore_chanban(cst, s);
+	    else if (lasthand[0] == '*') {
+	      if (lasthand[1] == 'i')
+		restore_ignore(s);
+	      else {
+		restore_chanban(NULL, s);
+		gban_total++;
 	      }
-	      else if (lasthand[0] == '*') {
-		if (lasthand[1] == 'e') {
-		  restore_chanexempt(NULL, s);
-		  gexempt_total++;
-		}
-	      }
-	    }
+	    } else if (lasthand[0])
+	      set_user(&USERENTRY_HOSTS, u, s);
 	  }
-	} else if (strcmp(code, "@") == 0) { /* Invitemasks */
-	  if (lasthand[0]) {
-	    if (s[0]) {
-	      if ((lasthand[0] == '#') || (lasthand[0] == '+')) {
-		restore_chaninvite(cst,s);
+	} else if (!strcmp(code, "%")) { /* exemptmasks */
+	  if (!lasthand[0])
+	    continue;		/* Skip this entry.	*/
+	  if (s[0]) {
+	    if (lasthand[0] == '#' || lasthand[0] == '+')
+	      restore_chanexempt(cst,s);
+	    else if (lasthand[0] == '*')
+	      if (lasthand[1] == 'e') {
+		restore_chanexempt(NULL, s);
+		gexempt_total++;
+	      }
+	  }
+	} else if (!strcmp(code, "@")) { /* Invitemasks */
+	  if (!lasthand[0])
+	    continue;		/* Skip this entry.	*/
+	  if (s[0]) {
+	    if (lasthand[0] == '#' || lasthand[0] == '+')
+	      restore_chaninvite(cst,s);
+	    else if (lasthand[0] == '*')
+	      if (lasthand[1] == 'I') {
+		restore_chaninvite(NULL, s);
+		ginvite_total++;
 	      }
-	      else if (lasthand[0] == '*') {
-		if (lasthand[1] == 'I') {  
-		  restore_chaninvite(NULL, s);
-		  ginvite_total++;
-		}
-	      }   
-	    }      
 	  }
 	} else if (!strcmp(code, "!")) {
 	  /* ! #chan laston flags [info] */
@@ -872,7 +756,7 @@ int readuserfile(char *file, struct userrec **ret)
 	    rmspace(s);
 	    fr.match = FR_CHAN;
 	    break_down_flags(fl, &fr, 0);
-	    if (findchan(chname)) {
+	    if (findchan_by_dname(chname)) {
 	      for (cr = u->chanrec; cr; cr = cr->next)
 		if (!rfc_casecmp(cr->channel, chname))
 		  break;
@@ -882,13 +766,12 @@ int readuserfile(char *file, struct userrec **ret)
 
 		cr->next = u->chanrec;
 		u->chanrec = cr;
-		strncpy(cr->channel, chname, 80);
-		cr->channel[80] = 0;
+		strncpyz(cr->channel, chname, 80);
 		cr->laston = atoi(st);
 		cr->flags = fr.chan;
 		cr->flags_udef = fr.udef_chan;
 		if (s[0]) {
-		  cr->info = (char *) nmalloc(strlen(s) + 1);
+		  cr->info = (char *) user_malloc(strlen(s) + 1);
 		  strcpy(cr->info, s);
 		} else
 		  cr->info = NULL;
@@ -898,109 +781,89 @@ int readuserfile(char *file, struct userrec **ret)
 	} else if (!strncmp(code, "::", 2)) {
 	  /* channel-specific bans */
 	  strcpy(lasthand, &code[2]);
-	  if (!findchan(lasthand)) {
-	    strcat(ignored, lasthand);
-	    strcat(ignored, " ");
+	  u = NULL;
+	  if (!findchan_by_dname(lasthand)) {
+	    strcpy(s1, lasthand);
+	    strcat(s1, " ");
+	    if (strstr(ignored, s1) == NULL) {
+	      strcat(ignored, lasthand);
+	      strcat(ignored, " ");
+	    }
 	    lasthand[0] = 0;
-	    u = 0;
 	  } else {
 	    /* Remove all bans for this channel to avoid dupes */
 	    /* NOTE only remove bans for when getting a userfile
 	     * from another bot & that channel is shared */
-	    cst = findchan(lasthand);
+	    cst = findchan_by_dname(lasthand);
 	    if ((*ret == userlist) || channel_shared(cst)) {
-	      while (cst->bans) {
-		struct banrec *b = cst->bans;
-
-		cst->bans = b->next;
-		if (b->banmask)
-		  nfree(b->banmask);
-		if (b->user)
-		  nfree(b->user);
-		if (b->desc)
-		  nfree(b->desc);
-		nfree(b);
-	      }
+	      clear_masks(cst->bans);
+	      cst->bans = NULL;
 	    } else {
 	      /* otherwise ignore any bans for this channel */
 	      cst = NULL;
 	      lasthand[0] = 0;
 	    }
 	  }
-	} else if (strncmp(code, "&&", 2) == 0) {
+	} else if (!strncmp(code, "&&", 2)) {
 	  /* channel-specific exempts */
 	  strcpy(lasthand, &code[2]);
-	  if (!findchan(lasthand)) {
-	    strcat(ignored, lasthand);
-	    strcat(ignored, " ");
+	  u = NULL;
+	  if (!findchan_by_dname(lasthand)) {
+	    strcpy(s1, lasthand);
+	    strcat(s1, " ");
+	    if (strstr(ignored, s1) == NULL) {
+	      strcat(ignored, lasthand);
+	      strcat(ignored, " ");
+	    }
 	    lasthand[0] = 0;
-	    u = 0;
 	  } else {
 	    /* Remove all exempts for this channel to avoid dupes */
 	    /* NOTE only remove exempts for when getting a userfile
 	     * from another bot & that channel is shared */
-	    cst = findchan(lasthand);
+	    cst = findchan_by_dname(lasthand);
 	    if ((*ret == userlist) || channel_shared(cst)) {
-	      while (cst->exempts) {
-		struct exemptrec * e = cst->exempts;
-		
-		cst->exempts = e->next;
-		if (e->exemptmask)
-		  nfree(e->exemptmask);
-		if (e->user)
-		  nfree(e->user);   
-		if (e->desc)
-		  nfree(e->desc);   
-		nfree(e);
-	      }
+	      clear_masks(cst->exempts);
+	      cst->exempts = NULL;
 	    } else {
 	      /* otherwise ignore any exempts for this channel */
 	      cst = NULL;
 	      lasthand[0] = 0;
 	    }
 	  }
-	} else if (strncmp(code, "$$", 2) == 0) {  
+	} else if (!strncmp(code, "$$", 2)) {
 	  /* channel-specific invites */
 	  strcpy(lasthand, &code[2]);
-	  if (!findchan(lasthand)) {   
-	    strcat(ignored, lasthand);
-	    strcat(ignored, " ");
+	  u = NULL;
+	  if (!findchan_by_dname(lasthand)) {
+	    strcpy(s1, lasthand);
+	    strcat(s1, " ");
+	    if (strstr(ignored, s1) == NULL) {
+	      strcat(ignored, lasthand);
+	      strcat(ignored, " ");
+	    }
 	    lasthand[0] = 0;
-	    u = 0;
 	  } else {
 	    /* Remove all invites for this channel to avoid dupes */
 	    /* NOTE only remove invites for when getting a userfile
 	     * from another bot & that channel is shared */
-	    cst = findchan(lasthand);
+	    cst = findchan_by_dname(lasthand);
 	    if ((*ret == userlist) || channel_shared(cst)) {
-	      while (cst->invites) {
-		struct inviterec * inv = cst->invites;
-		
-		cst->invites = inv->next;
-		if (inv->invitemask)
-		  nfree(inv->invitemask);
-		if (inv->user)
-		  nfree(inv->user);
-		if (inv->desc)   
-		  nfree(inv->desc);
-		nfree(inv);
-	      }
+	      clear_masks(cst->invites);
+              cst->invites = NULL;
 	    } else {
 	      /* otherwise ignore any invites for this channel */
 	      cst = NULL;
 	      lasthand[0] = 0;
 	    }
-	  } 
+	  }
 	} else if (!strncmp(code, "--", 2)) {
-	  /* new format storage */
-	  struct user_entry *ue;
-	  int ok = 0;
-
-	  context;
 	  if (u) {
-	    ue = u->entries;
-	    for (; ue && !ok; ue = ue->next)
-	      if (ue->name && !strcasecmp(code + 2, ue->name)) {
+	    /* new format storage */
+	    struct user_entry *ue;
+	    int ok = 0;
+
+	    for (ue = u->entries; ue && !ok; ue = ue->next)
+	      if (ue->name && !egg_strcasecmp(code + 2, ue->name)) {
 		struct list_type *list;
 
 		list = user_malloc(sizeof(struct list_type));
@@ -1072,7 +935,7 @@ int readuserfile(char *file, struct userrec **ret)
 
 	      u = get_user_by_handle(bu, code);
 	      for (i = 0; i < dcc_total; i++)
-		if (!strcasecmp(code, dcc[i].nick))
+		if (!egg_strcasecmp(code, dcc[i].nick))
 		  dcc[i].user = u;
 	      u->flags_udef = fr.udef_global;
 	      /* if s starts with '/' it's got file info */
@@ -1082,17 +945,20 @@ int readuserfile(char *file, struct userrec **ret)
       }
     }
   }
-  context;
   fclose(f);
   (*ret) = bu;
   if (ignored[0]) {
     putlog(LOG_MISC, "*", "%s %s", USERF_IGNBANS, ignored);
   }
   putlog(LOG_MISC, "*", "Userfile loaded, unpacking...");
-  context;
   for (u = bu; u; u = u->next) {
     struct user_entry *e;
 
+    if (!(u->flags & USER_BOT) && !egg_strcasecmp (u->handle, botnetnick)) {
+      putlog(LOG_MISC, "*", "(!) I have an user record, but without +b");
+      /* u->flags |= USER_BOT; */
+    }
+
     for (e = u->entries; e; e = e->next)
       if (e->name) {
 	struct user_entry_type *uet = find_entry_type(e->name);
@@ -1106,7 +972,6 @@ int readuserfile(char *file, struct userrec **ret)
       }
   }
   noshare = noxtra = 0;
-  context;
   /* process the user data *now* */
   return 1;
 }
@@ -1122,7 +987,6 @@ void autolink_cycle(char *start)
   int got_hub = 0, got_alt = 0, got_shared = 0, linked, ready = 0, i,
    bfl;
 
-  context;
   /* don't start a new cycle if some links are still pending */
   if (!start) {
     for (i = 0; i < dcc_total; i++) {
@@ -1130,6 +994,9 @@ void autolink_cycle(char *start)
 	return;
       if (dcc[i].type == &DCC_FORK_BOT)
 	return;
+      if ((dcc[i].type == &DCC_DNSWAIT) &&
+	  (dcc[i].u.dns && (dcc[i].u.dns->type == &DCC_FORK_BOT)))
+	return;
     }
   }
   if (!start) {
@@ -1155,7 +1022,7 @@ void autolink_cycle(char *start)
 	  if ((bfl & BOT_HUB) && (bfl & BOT_SHARE)) {
 	    if (linked)
 	      got_shared = 1;
-	    else if ((cycle == 0) && ready && !autc)
+	    else if (!cycle && ready && !autc)
 	      autc = u;
 	  } else if ((bfl & BOT_HUB) && cycle > 0) {
 	    if (linked)
@@ -1170,7 +1037,7 @@ void autolink_cycle(char *start)
 	  }
 	  /* did we make it where we're supposed to start?  yay! */
 	  if (!ready)
-	    if (!strcasecmp(u->handle, start)) {
+	    if (!egg_strcasecmp(u->handle, start)) {
 	      ready = 1;
 	      autc = NULL;
 	      /* if starting point is a +h bot, must be in 2nd cycle */
@@ -1183,30 +1050,33 @@ void autolink_cycle(char *start)
 	      }
 	    }
 	}
-	if ((cycle == 0) && (bfl & BOT_REJECT) && in_chain(u->handle)) {
+	if (!cycle && (bfl & BOT_REJECT) && in_chain(u->handle)) {
 	  /* get rid of nasty reject bot */
 	  int i;
 
 	  i = nextbot(u->handle);
-	  if ((i >= 0) && !strcasecmp(dcc[i].nick, u->handle)) {
+	  if ((i >= 0) && !egg_strcasecmp(dcc[i].nick, u->handle)) {
 	    char *p = MISC_REJECTED;
 
 	    /* we're directly connected to the offending bot?! (shudder!) */
 	    putlog(LOG_BOTS, "*", "%s %s", BOT_REJECTING, dcc[i].nick);
 	    chatout("*** %s bot %s\n", p, dcc[i].nick);
 	    botnet_send_unlinked(i, dcc[i].nick, p);
-	    dprintf(i, "bye\n");
+	    dprintf(i, "bye %s\n", BOT_REJECTING);
 	    killsock(dcc[i].sock);
 	    lostdcc(i);
-	  } else {
-	    botnet_send_reject(i, botnetnick, NULL, u->handle, NULL, NULL);
+	  } else if ((i < 0) && egg_strcasecmp(botnetnick, u->handle)) {
+	    /* The bot is not connected, but listed in our tandem list! */
+	    putlog(LOG_BOTS, "*", "(!) BUG: rejecting not connected bot %s!",
+		   u->handle);
+	    rembot(u->handle);
 	  }
 	}
       }
       u = u->next;
     }
     if (!autc) {
-      if ((cycle == 0) && !got_shared) {
+      if (!cycle && !got_shared) {
 	cycle++;
 	u = userlist;
       } else if ((cycle == 1) && !(got_shared || got_hub)) {
@@ -1215,7 +1085,7 @@ void autolink_cycle(char *start)
       }
     }
   }
-  if (got_shared && (cycle == 0))
+  if (got_shared && !cycle)
     autc = NULL;
   else if ((got_shared || got_hub) && (cycle == 1))
     autc = NULL;
