@@ -1,27 +1,48 @@
-/* structures and definitions used by users.c & userrec.c */
 /*
- * This file is part of the eggdrop source code
- * copyright (c) 1997 Robey Pointer
- * and is distributed according to the GNU general public license.
- * For full details, read the top of 'main.c' or the file called
- * COPYING that was distributed with this code.
+ * users.h
+ *   structures and definitions used by users.c and userrec.c
+ *
+ * $Id: users.h,v 1.9 2002/01/02 03:46:36 guppy Exp $
+ */
+/*
+ * Copyright (C) 1997 Robey Pointer
+ * Copyright (C) 1999, 2000, 2001, 2002 Eggheads Development Team
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#ifndef _H_USERS
-#define _H_USERS
+#ifndef _EGG_USERS_H
+#define _EGG_USERS_H
 
-/* list functions :) , next *must* be the 1st item in the struct */
+/* List functions :) , next *must* be the 1st item in the struct */
 struct list_type {
   struct list_type *next;
   char *extra;
 };
 
-#define list_insert(a,b) { b->next = *a; *a = b; }
+#define list_insert(a,b) {						\
+    	(b)->next = *(a);						\
+	*(a) = (b);							\
+}
 int list_append(struct list_type **, struct list_type *);
 int list_delete(struct list_type **, struct list_type *);
 int list_contains(struct list_type *, struct list_type *);
 
-/* new userfile format stuff */
+
+/* New userfile format stuff
+ */
 struct userrec;
 struct user_entry;
 struct user_entry_type {
@@ -44,13 +65,14 @@ struct user_entry_type {
   char *name;
 };
 
-#ifndef MAKING_MODS
-extern struct user_entry_type USERENTRY_EMAIL, USERENTRY_COMMENT, USERENTRY_LASTON,
- USERENTRY_XTRA, USERENTRY_INFO, USERENTRY_BOTADDR, USERENTRY_HOSTS,
- USERENTRY_PASS, USERENTRY_BOTFL, USERENTRY_URL;
 
+#ifndef MAKING_MODS
+extern struct user_entry_type USERENTRY_COMMENT, USERENTRY_LASTON,
+ USERENTRY_XTRA, USERENTRY_INFO, USERENTRY_BOTADDR, USERENTRY_HOSTS,
+ USERENTRY_PASS, USERENTRY_BOTFL;
 #endif
 
+
 struct laston_info {
   time_t laston;
   char *lastonplace;
@@ -87,11 +109,14 @@ struct filesys_stats {
   int dnload_ks;
 };
 
-void *_user_malloc(int, char *, int);
+void *_user_malloc(int size, const char *file, int line);
+void *_user_realloc(void *ptr, int size, const char *file, int line);
 
 #ifndef MAKING_MODS
-#define user_malloc(x) _user_malloc(x,__FILE__,__LINE__)
+#  define user_malloc(x)	_user_malloc(x, __FILE__, __LINE__)
+#  define user_realloc(x, y)	_user_realloc(x, y, __FILE__, __LINE__)
 #endif
+
 int add_entry_type(struct user_entry_type *);
 int del_entry_type(struct user_entry_type *);
 struct user_entry_type *find_entry_type(char *);
@@ -99,18 +124,20 @@ struct user_entry *find_user_entry(struct user_entry_type *, struct userrec *);
 void *get_user(struct user_entry_type *, struct userrec *);
 int set_user(struct user_entry_type *, struct userrec *, void *);
 
-#define bot_flags(u) ((long)get_user(&USERENTRY_BOTFL,u))
-#define is_bot(u) (u && (u->flags & USER_BOT))
-#define is_master(u) (u && (u->flags & USER_MASTER))
-#define is_owner(u) (u && (u->flags & USER_OWNER))
+#define bot_flags(u)	((long)get_user(&USERENTRY_BOTFL, (u)))
+#define is_bot(u)	((u) && ((u)->flags & USER_BOT))
+#define is_master(u)	((u) && ((u)->flags & USER_MASTER))
+#define is_owner(u)	((u) && ((u)->flags & USER_OWNER))
 
-/* fake users used to store ignores and bans */
+/* Fake users used to store ignores and bans
+ */
 #define IGNORE_NAME "*ignore"
 #define BAN_NAME    "*ban"
 #define EXEMPT_NAME "*exempt"
 #define INVITE_NAME "*Invite"
 
-/* channel-specific info */
+/* Channel-specific info
+ */
 struct chanuserrec {
   struct chanuserrec *next;
   char channel[81];
@@ -120,7 +147,8 @@ struct chanuserrec {
   char *info;
 };
 
-/* new-style userlist */
+/* New-style userlist
+ */
 struct userrec {
   struct userrec *next;
   char handle[HANDLEN + 1];
@@ -130,22 +158,6 @@ struct userrec {
   struct user_entry *entries;
 };
 
-/* let's get neat, a struct for each */
-struct banrec {
-  struct banrec *next;
-  char *banmask;
-  time_t expire;
-  time_t added;
-  time_t lastactive;
-  char *user;
-  char *desc;
-  int flags;
-};
-extern struct banrec *global_bans;
-
-#define BANREC_STICKY 1
-#define BANREC_PERM   2
-
 struct igrec {
   struct igrec *next;
   char *igmask;
@@ -159,60 +171,34 @@ extern struct igrec *global_ign;
 
 #define IGREC_PERM   2
 
-struct exemptrec {
-   struct exemptrec * next;
-   char * exemptmask;
-   time_t expire;
-   time_t added;
-   time_t lastactive;
-   char * user;
-   char * desc;  
-   int flags;
-};
-extern struct exemptrec * global_exempts;
-   
-#define EXEMPTREC_STICKY 1
-#define EXEMPTREC_PERM   2
-
-struct inviterec {
-   struct inviterec * next;
-   char * invitemask;
-   time_t expire;
-   time_t added;
-   time_t lastactive;
-   char * user;
-   char * desc;
-   int flags;
-};
-extern struct inviterec * global_invites;
-   
-#define INVITEREC_STICKY 1
-#define INVITEREC_PERM   2
-
-/* flags are in eggdrop.h */
+/*
+ * Note: Flags are in eggdrop.h
+ */
 
 struct userrec *adduser();
 struct userrec *get_user_by_handle(struct userrec *, char *);
 struct userrec *get_user_by_host(char *);
+struct userrec *get_user_by_nick(char *);
 struct userrec *check_chanlist();
 struct userrec *check_chanlist_hand();
 
-/* all the default userentry stuff, for code re-use */
+/* All the default userentry stuff, for code re-use
+ */
 int def_unpack(struct userrec *u, struct user_entry *e);
 int def_pack(struct userrec *u, struct user_entry *e);
 int def_kill(struct user_entry *e);
-int def_write_userfile(FILE * f, struct userrec *u, struct user_entry *e);
+int def_write_userfile(FILE *f, struct userrec *u, struct user_entry *e);
 void *def_get(struct userrec *u, struct user_entry *e);
 int def_set(struct userrec *u, struct user_entry *e, void *buf);
 int def_gotshare(struct userrec *u, struct user_entry *e,
 		 char *data, int idx);
-int def_tcl_get(Tcl_Interp * interp, struct userrec *u,
+int def_tcl_get(Tcl_Interp *interp, struct userrec *u,
 		struct user_entry *e, int argc, char **argv);
-int def_tcl_set(Tcl_Interp * irp, struct userrec *u,
+int def_tcl_set(Tcl_Interp *irp, struct userrec *u,
 		struct user_entry *e, int argc, char **argv);
 int def_expmem(struct user_entry *e);
 void def_display(int idx, struct user_entry *e);
 int def_dupuser(struct userrec *new, struct userrec *old,
 		struct user_entry *e);
 
-#endif
+#endif				/* _EGG_USERS_H */
